@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-// $Id: dwuninst.cpp,v 1.4 2002/02/21 22:24:51 giles Exp $
+// $Id: dwuninst.cpp,v 1.6 2005/03/04 21:58:55 ghostgum Exp $
 
 #define STRICT
 #include <windows.h>
@@ -34,10 +34,17 @@
 #define mkdir(x) _mkdir(x)
 #endif
 #define DELAY_STEP 500
-#define DELAY_FILE 5
+#define DELAY_FILE 0
 #define MAXSTR 256
 #define UNINSTALLKEY TEXT("SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\Uninstall")
 
+#ifdef _WIN64
+#define DLGRETURN INT_PTR
+#else
+#define DLGRETURN BOOL
+#endif
+
+
 HWND hDlgModeless;
 HWND hText1;
 HWND hText2;
@@ -673,7 +680,7 @@ BOOL shell_old(void)
 #ifdef __BORLANDC__
 #pragma argsused
 #endif
-BOOL CALLBACK _export
+DLGRETURN CALLBACK _export
 RemoveDlgProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
 {
   switch(message) {
