@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevbbox.c,v 1.11.2.1 2003/12/12 22:07:59 giles Exp $ */
+/*$Id: gdevbbox.c,v 1.23 2005/03/14 18:08:36 dan Exp $ */
 /* Device for tracking bounding box */
 #include "math_.h"
 #include "memory_.h"
@@ -222,7 +222,7 @@ bbox_copy_params(gx_device_bbox * bdev, bool remap_colors)
 
     if (tdev != 0)
 	gx_device_copy_params((gx_device *)bdev, tdev);
-    if (remap_colors && bdev->is_open) {
+    if (remap_colors) {
 	bdev->black = gx_device_black((gx_device *)bdev);
 	bdev->white = gx_device_white((gx_device *)bdev);
 	bdev->transparent =
@@ -255,10 +255,10 @@ bbox_close_device(gx_device * dev)
 
 /* Initialize a bounding box device. */
 void
-gx_device_bbox_init(gx_device_bbox * dev, gx_device * target)
+gx_device_bbox_init(gx_device_bbox * dev, gx_device * target, gs_memory_t *mem)
 {
     gx_device_init((gx_device *) dev, (const gx_device *)&gs_bbox_device,
-		   (target ? target->memory : NULL), true);
+		   (target ? target->memory : mem), true);
     if (target) {
         gx_device_forward_fill_in_procs((gx_device_forward *) dev);
 	set_dev_proc(dev, get_initial_matrix, gx_forward_get_initial_matrix);
@@ -273,6 +273,9 @@ gx_device_bbox_init(gx_device_bbox * dev, gx_device * target)
 	set_dev_proc(dev, pattern_manage, gx_forward_pattern_manage);
 	set_dev_proc(dev, fill_rectangle_hl_color, gx_forward_fill_rectangle_hl_color);
 	set_dev_proc(dev, include_color_space, gx_forward_include_color_space);
+	set_dev_proc(dev, update_spot_equivalent_colors,
+				gx_forward_update_spot_equivalent_colors);
+	set_dev_proc(dev, get_page_device, gx_forward_get_page_device);
 	gx_device_set_target((gx_device_forward *)dev, target);
     } else {
 	gx_device_fill_in_procs((gx_device *)dev);
@@ -522,7 +525,11 @@ bbox_get_params(gx_device * dev, gs_param_list * plist)
     bbox[2] = fixed2float(fbox.q.x);
     bbox[3] = fixed2float(fbox.q.y);
     bba.data = bbox, bba.size = 4, bba.persistent = false;
-    return param_write_float_array(plist, "PageBoundingBox", &bba);
+    code = param_write_float_array(plist, "PageBoundingBox", &bba);
+    if (code < 0)
+        return code;
+    code = param_write_bool(plist, "WhiteIsOpaque", &bdev->white_is_opaque);
+    return code;
 }
 
 /* We implement put_params to ensure that we keep the important */
@@ -534,6 +541,7 @@ bbox_put_params(gx_device * dev, gs_param_list * plist)
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     int code;
     int ecode = 0;
+    bool white_is_opaque = bdev->white_is_opaque;
     gs_param_name param_name;
     gs_param_float_array bba;
 
@@ -548,20 +556,32 @@ bbox_put_params(gx_device * dev, gs_param_list * plist)
 	    break;
 	default:
 	    ecode = code;
-	  e:param_signal_error(plist, param_name, ecode);
+	    e:param_signal_error(plist, param_name, ecode);
 	case 1:
 	    bba.data = 0;
     }
 
+    switch (code = param_read_bool(plist, (param_name = "WhiteIsOpaque"), &white_is_opaque)) {
+	default:
+	    ecode = code;
+	    param_signal_error(plist, param_name, ecode);
+	case 0:
+        case 1:
+	    break;
+    }
+
     code = gx_forward_put_params(dev, plist);
     if (ecode < 0)
 	code = ecode;
-    if (code >= 0 && bba.data != 0) {
-	BBOX_INIT_BOX(bdev);
-	BBOX_ADD_RECT(bdev, float2fixed(bba.data[0]), float2fixed(bba.data[1]),
-		      float2fixed(bba.data[2]), float2fixed(bba.data[3]));
+    if (code >= 0) {
+        if( bba.data != 0) {
+	    BBOX_INIT_BOX(bdev);
+	    BBOX_ADD_RECT(bdev, float2fixed(bba.data[0]), float2fixed(bba.data[1]),
+	    	          float2fixed(bba.data[2]), float2fixed(bba.data[3]));
+        }
+        bdev->white_is_opaque = white_is_opaque;
     }
-    bbox_copy_params(bdev, true);
+    bbox_copy_params(bdev, bdev->is_open);
     return code;
 }
 
@@ -1146,7 +1166,7 @@ private const gx_device_bbox_procs_t box_procs_forward = {
 private int
 bbox_create_compositor(gx_device * dev,
 		       gx_device ** pcdev, const gs_composite_t * pcte,
-		       const gs_imager_state * pis, gs_memory_t * memory)
+		       gs_imager_state * pis, gs_memory_t * memory)
 {
     gx_device_bbox *const bdev = (gx_device_bbox *) dev;
     gx_device *target = bdev->target;
@@ -1170,8 +1190,11 @@ bbox_create_compositor(gx_device * dev,
 	int code = (*dev_proc(target, create_compositor))
 	    (target, &cdev, pcte, pis, memory);
 
-	if (code < 0)
+	/* If the target did not create a new compositor then we are done. */
+	if (code < 0 || target == cdev) {
+	    *pcdev = dev;
 	    return code;
+	}
 	bbcdev = gs_alloc_struct_immovable(memory, gx_device_bbox,
 					   &st_device_bbox,
 					   "bbox_create_compositor");
@@ -1179,7 +1202,7 @@ bbox_create_compositor(gx_device * dev,
 	    (*dev_proc(cdev, close_device)) (cdev);
 	    return_error(gs_error_VMerror);
 	}
-	gx_device_bbox_init(bbcdev, target);
+	gx_device_bbox_init(bbcdev, target, memory);
 	gx_device_set_target((gx_device_forward *)bbcdev, cdev);
 	bbcdev->box_procs = box_procs_forward;
 	bbcdev->box_proc_data = bdev;
