@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gdevdbit.c,v 1.8.2.1 2003/12/12 22:08:00 giles Exp $ */
+/*$Id: gdevdbit.c,v 1.11 2004/08/05 17:02:36 stefan Exp $ */
 /* Default device bitmap copying implementation */
 #include "gx.h"
 #include "gpcheck.h"
@@ -417,7 +417,7 @@ gx_default_strip_tile_rectangle(gx_device * dev, const gx_strip_bitmap * tiles,
      (*proc_color)(dev, row, srcx, raster, id, tx, ty, tw, th) :\
      (*proc_mono)(dev, row, srcx, raster, id, tx, ty, tw, th, color0, color1));\
   if (code < 0) return_error(code);\
-  return_if_interrupt()
+  return_if_interrupt(dev->memory)
 #ifdef DEBUG
 #define copy_tile(srcx, tx, ty, tw, th, tid)\
   if_debug6('t', "   copy id=%lu sx=%d => x=%d y=%d w=%d h=%d\n",\
