@@ -1,4 +1,4 @@
-/* Copyright (C) 1989, 1995, 1996, 1997, 1998, 1999 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1989-2005 artofcode LLC.  All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpcfb.c,v 1.5 2002/06/16 05:48:55 lpd Exp $ */
+/* $Id: gdevpcfb.c,v 1.7 2005/08/09 20:23:07 giles Exp $ */
 /* IBM PC frame buffer (EGA/VGA) drivers */
 #include "memory_.h"
 #include "gconfigv.h"		/* for USE_ASM */
@@ -178,21 +178,21 @@ svga16_put_params(gx_device * dev, gs_param_list * plist)
 }
 
 /* Map a r-g-b color to an EGA color code. */
-#define Nb gx_color_value_bits
 private gx_color_index
-ega0_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		   gx_color_value b)
+ega0_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
-    return pc_4bit_map_rgb_color(dev, r, r, r);
+    return pc_4bit_map_rgb_color(dev, cv);
 }
 private gx_color_index
-ega1_map_rgb_color(gx_device * dev, gx_color_value r, gx_color_value g,
-		   gx_color_value b)
+ega1_map_rgb_color(gx_device * dev, const gx_color_value cv[])
 {
-#define cvtop (gx_color_value)(1 << (Nb - 1))
-    return pc_4bit_map_rgb_color(dev, r & cvtop, g & cvtop, b & cvtop);
+    const gx_color_value cvtop = (1 << (gx_color_value_bits - 1));
+    gx_color_value cvt[3];
+    cvt[0] = cv[0] & cvtop;
+    cvt[1] = cv[1] & cvtop;
+    cvt[2] = cv[2] & cvtop;
+    return pc_4bit_map_rgb_color(dev, cvt);
 }
-#undef Nb
 
 /* Map a color code to r-g-b. */
 #define icolor (int)color
