@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdf.c,v 1.67.2.6 2004/02/13 08:32:25 igor Exp $ */
+/* $Id: gdevpdf.c,v 1.124 2005/09/12 11:34:50 leonardo Exp $ */
 /* PDF-writing driver */
 #include "fcntl_.h"
 #include "memory_.h"
@@ -35,7 +35,7 @@
 /* Define the default language level and PDF compatibility level. */
 /* Acrobat 4 (PDF 1.3) is the default. */
 #define PSDF_VERSION_INITIAL psdf_version_ll3
-#define PDF_COMPATIBILITY_LEVEL_INITIAL 1.3
+#define PDF_COMPATIBILITY_LEVEL_INITIAL 1.4
 
 /* Define the size of internal stream buffers. */
 /* (This is not a limitation, it only affects performance.) */
@@ -54,7 +54,7 @@ private_st_pdf_substream_save_element();
 private 
 ENUM_PTRS_WITH(device_pdfwrite_enum_ptrs, gx_device_pdf *pdev)
 {
-    index -= gx_device_pdf_num_ptrs + gx_device_pdf_num_strings;
+    index -= gx_device_pdf_num_ptrs + gx_device_pdf_num_param_strings;
     if (index < NUM_RESOURCE_TYPES * NUM_RESOURCE_CHAINS)
 	ENUM_RETURN(pdev->resources[index / NUM_RESOURCE_CHAINS].chains[index % NUM_RESOURCE_CHAINS]);
     index -= NUM_RESOURCE_TYPES * NUM_RESOURCE_CHAINS;
@@ -69,8 +69,12 @@ ENUM_PTRS_WITH(device_pdfwrite_enum_ptrs, gx_device_pdf *pdev)
 #define e1(i,elt) ENUM_PTR(i, gx_device_pdf, elt);
 gx_device_pdf_do_ptrs(e1)
 #undef e1
-#define e1(i,elt) ENUM_STRING_PTR(i + gx_device_pdf_num_ptrs, gx_device_pdf, elt);
-gx_device_pdf_do_strings(e1)
+#define e1(i,elt) ENUM_PARAM_STRING_PTR(i + gx_device_pdf_num_ptrs, gx_device_pdf, elt);
+gx_device_pdf_do_param_strings(e1)
+#undef e1
+#define e1(i,elt) ENUM_STRING_PTR(i + gx_device_pdf_num_ptrs + gx_device_pdf_num_param_strings,\
+				 gx_device_pdf, elt);
+gx_device_pdf_do_const_strings(e1)
 #undef e1
 ENUM_PTRS_END
 private RELOC_PTRS_WITH(device_pdfwrite_reloc_ptrs, gx_device_pdf *pdev)
@@ -79,8 +83,11 @@ private RELOC_PTRS_WITH(device_pdfwrite_reloc_ptrs, gx_device_pdf *pdev)
 #define r1(i,elt) RELOC_PTR(gx_device_pdf,elt);
     gx_device_pdf_do_ptrs(r1)
 #undef r1
-#define r1(i,elt) RELOC_STRING_PTR(gx_device_pdf,elt);
-	gx_device_pdf_do_strings(r1)
+#define r1(i,elt) RELOC_PARAM_STRING_PTR(gx_device_pdf,elt);
+	gx_device_pdf_do_param_strings(r1)
+#undef r1
+#define r1(i,elt) RELOC_CONST_STRING_PTR(gx_device_pdf,elt);
+	gx_device_pdf_do_const_strings(r1)
 #undef r1
     {
 	int i, j;
@@ -117,176 +124,31 @@ private dev_proc_close_device(pdf_close);
 #  define Y_DPI 720
 #endif
 
-const gx_device_pdf gs_pdfwrite_device =
-{std_device_dci_type_body(gx_device_pdf, 0, "pdfwrite",
-			  &st_device_pdfwrite,
-			  DEFAULT_WIDTH_10THS * X_DPI / 10,
-			  DEFAULT_HEIGHT_10THS * Y_DPI / 10,
-			  X_DPI, Y_DPI,
-			  3, 24, 255, 255, 256, 256),
- {pdf_open,
-  gx_upright_get_initial_matrix,
-  NULL,				/* sync_output */
-  pdf_output_page,
-  pdf_close,
-  gx_default_rgb_map_rgb_color,
-  gx_default_rgb_map_color_rgb,
-  gdev_pdf_fill_rectangle,
-  NULL,				/* tile_rectangle */
-  gdev_pdf_copy_mono,
-  gdev_pdf_copy_color,
-  NULL,				/* draw_line */
-  psdf_get_bits,		/* get_bits */
-  gdev_pdf_get_params,
-  gdev_pdf_put_params,
-  NULL,				/* map_cmyk_color */
-  NULL,				/* get_xfont_procs */
-  NULL,				/* get_xfont_device */
-  NULL,				/* map_rgb_alpha_color */
-  gx_page_device_get_page_device,
-  NULL,				/* get_alpha_bits */
-  NULL,				/* copy_alpha */
-  NULL,				/* get_band */
-  NULL,				/* copy_rop */
-  gdev_pdf_fill_path,
-  gdev_pdf_stroke_path,
-  gdev_pdf_fill_mask,
-  NULL,				/* fill_trapezoid */
-  NULL,				/* fill_parallelogram */
-  NULL,				/* fill_triangle */
-  NULL,				/* draw_thin_line */
-  NULL,				/* begin_image */
-  NULL,				/* image_data */
-  NULL,				/* end_image */
-  gdev_pdf_strip_tile_rectangle,
-  NULL,				/* strip_copy_rop */
-  NULL,				/* get_clipping_box */
-  gdev_pdf_begin_typed_image,
-  psdf_get_bits_rectangle,	/* get_bits_rectangle */
-  NULL,				/* map_color_rgb_alpha */
-  psdf_create_compositor,	/* create_compositor */
-  NULL,				/* get_hardware_params */
-  gdev_pdf_text_begin,
-  NULL,				/* finish_copydevice */
-  NULL,				/* begin_transparency_group */
-  NULL,				/* end_transparency_group */
-  NULL,				/* begin_transparency_mask */
-  NULL,				/* end_transparency_mask */
-  NULL,				/* discard_transparency_layer */
-  NULL,				/* get_color_mapping_procs */
-  NULL,				/* get_color_comp_index */
-  NULL,				/* encode_color */
-  NULL,				/* decode_color */
-  gdev_pdf_pattern_manage, 	/* pattern_manage */
-  gdev_pdf_fill_rectangle_hl_color, 	/* fill_rectangle_hl_color */
-  gdev_pdf_include_color_space 	/* include_color_space */
- },
- psdf_initial_values(PSDF_VERSION_INITIAL, 0 /*false */ ),  /* (!ASCII85EncodePages) */
- PDF_COMPATIBILITY_LEVEL_INITIAL,  /* CompatibilityLevel */
- -1,				/* EndPage */
- 1,				/* StartPage */
- 1 /*true*/,			/* Optimize */
- 0 /*false*/,			/* ParseDSCCommentsForDocInfo */
- 1 /*true*/,			/* ParseDSCComments */
- 0 /*false*/,			/* EmitDSCWarnings */
- 0 /*false*/,			/* CreateJobTicket */
- 0 /*false*/,			/* PreserveEPSInfo */
- 1 /*true*/,			/* AutoPositionEPSFiles */
- 1 /*true*/,			/* PreserveCopyPage */
- 0 /*false*/,			/* UsePrologue */
- 0,				/* OffOptimizations */
- 1 /*true*/,			/* ReAssignCharacters */
- 1 /*true*/,			/* ReEncodeCharacters */
- 1,				/* FirstObjectNumber */
- 1 /*true*/,			/* CompressFonts */
- 4000,				/* MaxInlineImageSize */
- {0, 0},			/* OwnerPassword */
- {0, 0},			/* UserPassword */
- 0,				/* KeyLength */
- 4,				/* Permissions */
- 0,				/* EncryptionR */
- {0},				/* EncryptionO */
- {0},				/* EncryptionU */
- {0},				/* EncryptionKey */
- 0,				/* EncryptionV */
- true,				/* EncryptMetadata */
- {0, 0},			/* NoEncrypt */
- 0 /*false*/,			/* is_EPS */
- {-1, -1},			/* doc_dsc_info */
- {-1, -1},			/* page_dsc_info */
- 0 /*false*/,			/* fill_overprint */
- 0 /*false*/,			/* stroke_overprint */
- 0,				/* overprint_mode */
- gs_no_id,			/* halftone_id */
- {gs_no_id, gs_no_id, gs_no_id, gs_no_id}, /* transfer_ids */
- 0,				/* transfer_not_identity */
- gs_no_id,			/* black_generation_id */
- gs_no_id,			/* undercolor_removal_id */
- pdf_compress_none,		/* compression */
- {{0}},				/* xref */
- {{0}},				/* asides */
- {{0}},				/* streams */
- {{0}},				/* pictures */
- 0,				/* next_id */
- 0,				/* Catalog */
- 0,				/* Info */
- 0,				/* Pages */
- 0,				/* outlines_id */
- 0,				/* next_page */
- 0,				/* contents_id */
- PDF_IN_NONE,			/* context */
- 0,				/* contents_length_id */
- 0,				/* contents_pos */
- NoMarks,			/* procsets */
- 0,				/* text */
- {{0}},				/* text_rotation */
- 0,				/* pages */
- 0,				/* num_pages */
- 1,				/* used_mask */
- {
-     {
-	 {0}}},			/* resources */
- {0},				/* cs_Patterns */
- {0},				/* Identity_ToUnicode_CMaps */
- 0,				/* last_resource */
- {
-     {
-	 {0}}},			/* outline_levels */
- 0,				/* outline_depth */
- 0,				/* closed_outline_depth */
- 0,				/* outlines_open */
- 0,				/* articles */
- 0,				/* Dests */
- {0},				/* fileID */
- 0,				/* global_named_objects */
- 0,				/* local_named_objects */
- 0,				/* NI_stack */
- 0,				/* Namespace_stack */
- 0,				/* font_cache */
- {0, 0},			/* char_width */
- 0,				/* clip_path */
- 0,                             /* PageLabels */
- -1,                            /* PageLabels_current_page */
- 0,                             /* PageLabels_current_label */
- 0,				/* */
- {				/* vgstack[2] */
-    {0}, {0}
- },
- 0,				/* vgstack_depth */
- 0,				/* vgstack_bottom */
- {0},				/* vg_initial */
- false,				/* vg_initial_set */
- 0,				/* sbstack_size */
- 0,				/* sbstack_depth */
- 0,				/* sbstack */
- 0,				/* substream_Resources */
- 1,				/* pcm_color_info_index == DeviceRGB */
- false,				/* skip_colors */
- false,				/* AR4_save_bug */
- 0,				/* font3 */
- {0}				/* charproc_ctm */
-};
+/* ---------------- Device prototype ---------------- */
 
+#define PDF_DEVICE_NAME "pdfwrite"
+#define PDF_DEVICE_IDENT gs_pdfwrite_device
+#define PDF_DEVICE_MaxInlineImageSize 4000
+#define PDF_FOR_OPDFREAD 0
+
+#include "gdevpdfb.h"
+
+#undef PDF_DEVICE_NAME
+#undef PDF_DEVICE_IDENT
+#undef PDF_DEVICE_MaxInlineImageSize
+#undef PDF_FOR_OPDFREAD
+
+#define PDF_DEVICE_NAME "ps2write"
+#define PDF_DEVICE_IDENT gs_ps2write_device
+#define PDF_DEVICE_MaxInlineImageSize max_long
+#define PDF_FOR_OPDFREAD 1
+
+#include "gdevpdfb.h"
+
+#undef PDF_DEVICE_NAME
+#undef PDF_DEVICE_IDENT
+#undef PDF_DEVICE_MaxInlineImageSize
+#undef PDF_FOR_OPDFREAD
 /* ---------------- Device open/close ---------------- */
 
 /* Close and remove temporary files. */
@@ -343,6 +205,8 @@ pdf_reset_page(gx_device_pdf * pdev)
     pdev->procsets = NoMarks;
     memset(pdev->cs_Patterns, 0, sizeof(pdev->cs_Patterns));	/* simplest to create for each page */
     pdf_reset_text_page(pdev->text);
+    pdf_remember_clip_path(pdev, 0);
+    pdev->clip_path_id = pdev->no_clip_path_id;
 }
 
 /* Open a temporary file, with or without a stream. */
@@ -439,7 +303,7 @@ pdf_compute_fileID(gx_device_pdf * pdev)
     /* We compute a file identifier when beginning a document
        to allow its usage with PDF encryption. Due to that,
        in contradiction to the Adobe recommendation, our
-       ID doesn't depend on the document size.
+       ID doesn't depend on the document size. 
     */
     gs_memory_t *mem = pdev->pdf_memory;
     stream *strm = pdev->strm;
@@ -475,7 +339,7 @@ private const byte pad[32] = { 0x28, 0xBF, 0x4E, 0x5E, 0x4E, 0x75, 0x8A, 0x41,
 			       0x2F, 0x0C, 0xA9, 0xFE, 0x64, 0x53, 0x69, 0x7A};
 
 private inline void
-copy_padded(byte buf[32], gs_const_string *str)
+copy_padded(byte buf[32], gs_param_string *str)
 {
     memcpy(buf, str->data, min(str->size, 32));
     if (32 > str->size)
@@ -517,6 +381,10 @@ pdf_compute_encryption_data(gx_device_pdf * pdev)
     byte digest[16], buf[32], t;
     stream_arcfour_state sarc4;
 
+    if (pdev->PDFX && pdev->KeyLength != 0) {
+	eprintf("Encryption is not allowed in a PDF/X doucment.\n");
+	return_error(gs_error_rangecheck);
+    }
     if (pdev->KeyLength == 0)
 	pdev->KeyLength = 40;
     if (pdev->EncryptionV == 0 && pdev->KeyLength == 40)
@@ -545,7 +413,8 @@ pdf_compute_encryption_data(gx_device_pdf * pdev)
 	eprintf("PDF encryption key length must be a multiple of 8.\n");
 	return_error(gs_error_rangecheck);
     }
-    if (pdev->EncryptionR == 2 && (pdev->Permissions & (0xF << 8))) {
+    if (pdev->EncryptionR == 2 &&
+	((pdev->Permissions & (0xFFFFFFC3)) != 0xFFFFFFC0)) {
 	eprintf("Some of Permissions are not allowed with R=2.\n");
 	return_error(gs_error_rangecheck);
     }
@@ -599,7 +468,7 @@ pdf_compute_encryption_data(gx_device_pdf * pdev)
 	memcpy(pdev->EncryptionU + sizeof(digest), pad, 
 		sizeof(pdev->EncryptionU) - sizeof(digest));
     } else {
-    memcpy(pdev->EncryptionU, pad, sizeof(pdev->EncryptionU));
+	memcpy(pdev->EncryptionU, pad, sizeof(pdev->EncryptionU));
 	s_arcfour_set_key(&sarc4, pdev->EncryptionKey, pdev->KeyLength / 8);
 	s_arcfour_process_buffer(&sarc4, pdev->EncryptionU, sizeof(pdev->EncryptionU));
     }
@@ -745,8 +614,10 @@ pdf_open(gx_device * dev)
     } else if(pdev->UserPassword.size > 0) {
 	eprintf("User password is specified. Need an Owner password or both.\n");
 	return_error(gs_error_rangecheck);
+    } else if (pdev->KeyLength) {
+	eprintf("Can't accept encryption options without a password.\n");
+	return_error(gs_error_rangecheck);
     }
-
     /* Now create a new dictionary for the local named objects. */
     pdev->local_named_objects =
 	cos_dict_alloc(pdev, "pdf_open(local_named_objects)");
@@ -813,7 +684,7 @@ private int
 pdf_dominant_rotation(const pdf_text_rotation_t *ptr)
 {
     int i, imax = -1;
-    long max_count = -1;
+    long max_count = 0;
     static const int angles[] = { pdf_text_rotation_angle_values };
 
     for (i = 0; i < countof(ptr->counts); ++i) {
@@ -849,12 +720,16 @@ pdf_print_orientation(gx_device_pdf * pdev, pdf_page_t *page)
         const pdf_text_rotation_t *ptr = 
 	    (page != NULL ? &page->text_rotation : &pdev->text_rotation);
 	int angle = -1;
+
+#define  Bug687800
+#ifndef Bug687800 	/* Bug 687800 together with Bug687489.ps . */
 	const gs_point *pbox = &(page != NULL ? page : &pdev->pages[0])->MediaBox;
 
 	if (dsc_orientation >= 0 && pbox->x > pbox->y) {
 	    /* The page is in landscape format. Adjust the rotation accordingly. */
 	    dsc_orientation ^= 1;
 	}
+#endif
 
 	/* Combine DSC rotation with text rotation : */
 	if (dsc_orientation == 0) {
@@ -863,14 +738,18 @@ pdf_print_orientation(gx_device_pdf * pdev, pdf_page_t *page)
 	} else if (dsc_orientation == 1) {
 	    if (ptr->Rotate == 90 || ptr->Rotate == 270)
 		angle = ptr->Rotate;
+	    else 
+		angle = 90;
 	}
 
 	/* If not combinable, prefer text rotation : */
 	if (angle < 0) {
 	    if (ptr->Rotate >= 0)
 		angle = ptr->Rotate;
+#ifdef Bug687800
 	    else
 		angle = dsc_orientation * 90;
+#endif
 	}
 
 	/* If got some, write it out : */
@@ -893,7 +772,15 @@ pdf_close_page(gx_device_pdf * pdev)
      * before doing anything else.
      */
 
-    pdf_open_document(pdev);
+    code = pdf_open_document(pdev);
+    if (code < 0)
+	return code;
+    if (pdev->ForOPDFRead && pdev->context == PDF_IN_NONE) {
+	/* Must create a context stream for empty pages. */
+	code = pdf_open_contents(pdev, PDF_IN_STREAM);
+	if (code < 0)
+	    return code;
+    }
     pdf_close_contents(pdev, true);
 
     /*
@@ -909,6 +796,8 @@ pdf_close_page(gx_device_pdf * pdev)
     page->MediaBox.x = pdev->MediaSize[0];
     page->MediaBox.y = pdev->MediaSize[1];
     page->contents_id = pdev->contents_id;
+    page->NumCopies_set = pdev->NumCopies_set;
+    page->NumCopies = pdev->NumCopies;
     /* pdf_store_page_resources sets procsets, resource_ids[]. */
     code = pdf_store_page_resources(pdev, page);
     if (code < 0)
@@ -917,7 +806,30 @@ pdf_close_page(gx_device_pdf * pdev)
     /* Write the Functions. */
 
     pdf_write_resource_objects(pdev, resourceFunction);
-    /* pdf_free_resource_objects(pdev, resourceFunction); May be referred from resourceColorSpace. */
+
+    /* Save viewer's memory with cleaning resources. */
+
+    if (pdev->MaxViewerMemorySize < 10000000) {
+	/* fixme: the condition above and the cleaning algorithm
+	   may be improved with counting stored resource size
+	   and creating multiple streams per page. */
+
+	if (pdev->ForOPDFRead) {
+	    pdf_resource_t *pres = pdf_find_resource_by_resource_id(pdev, resourcePage, pdev->contents_id);
+	    
+	    if (pres != NULL) {
+		code = cos_dict_put_c_strings((cos_dict_t *)pres->object, "/.CleanResources", "/All");
+		if (code < 0)
+		    return code;
+	    }
+	}
+	code = pdf_close_text_document(pdev);
+	if (code < 0)
+	    return code;
+	code = pdf_write_and_free_all_resource_objects(pdev);
+	if (code < 0)
+	    return code;
+    }
 
     /* Close use of text on the page. */
 
@@ -969,8 +881,20 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
     pprintg2(s, "<</Type/Page/MediaBox [0 0 %g %g]\n",
 	     round_box_coord(page->MediaBox.x),
 	     round_box_coord(page->MediaBox.y));
+    if (pdev->PDFX) {
+	pprintg2(s, "/TrimBox [0 0 %g %g]\n",
+		round_box_coord(page->MediaBox.x),
+		round_box_coord(page->MediaBox.y));
+    }
     pdf_print_orientation(pdev, page);
     pprintld1(s, "/Parent %ld 0 R\n", pdev->Pages->id);
+    if (pdev->ForOPDFRead) {
+	if (page->NumCopies_set)
+	    pprintld1(s, "/NumCopies %ld\n", page->NumCopies);
+    }
+    if (page->group_id > 0) {
+	pprintld1(s, "/Group %ld 0 R\n", page->group_id);
+    }
     stream_puts(s, "/Resources<</ProcSet[/PDF");
     if (page->procsets & ImageB)
 	stream_puts(s, " /ImageB");
@@ -985,7 +909,7 @@ pdf_write_page(gx_device_pdf *pdev, int page_num)
 	int i;
 
 	for (i = 0; i < countof(page->resource_ids); ++i)
-	    if (page->resource_ids[i]) {
+	    if (page->resource_ids[i] && pdf_resource_type_names[i]) {
 		stream_puts(s, pdf_resource_type_names[i]);
 		pprintld1(s, " %ld 0 R\n", page->resource_ids[i]);
 	    }
@@ -1071,11 +995,26 @@ pdf_close(gx_device * dev)
 	    pdf_write_page(pdev, i);
     }
 
+    if (pdev->PrintStatistics)
+	pdf_print_resource_statistics(pdev);
+
     /* Write the font resources and related resources. */
     code1 = pdf_write_resource_objects(pdev, resourceXObject);
     if (code >= 0)
 	code = code1;
     code1 = pdf_free_resource_objects(pdev, resourceXObject);
+    if (code >= 0)
+	code = code1;
+    code1 = pdf_write_resource_objects(pdev, resourceGroup);
+    if (code >= 0)
+	code = code1;
+    code1 = pdf_free_resource_objects(pdev, resourceGroup);
+    if (code >= 0)
+	code = code1;
+    code1 = pdf_write_resource_objects(pdev, resourceSoftMaskDict);
+    if (code >= 0)
+	code = code1;
+    code1 = pdf_free_resource_objects(pdev, resourceSoftMaskDict);
     if (code >= 0)
 	code = code1;
     code1 = pdf_close_text_document(pdev);
@@ -1087,6 +1026,19 @@ pdf_close(gx_device * dev)
     code1 = pdf_free_resource_objects(pdev, resourceCMap);
     if (code >= 0)
 	code = code1;
+    if (pdev->ResourcesBeforeUsage)
+	pdf_reverse_resource_chain(pdev, resourcePage);
+    code1 = pdf_write_resource_objects(pdev, resourcePage);
+    if (code >= 0)
+	code = code1;
+    code1 = pdf_free_resource_objects(pdev, resourcePage);
+    if (code >= 0)
+	code = code1;
+
+    code1 = pdf_free_resource_objects(pdev, resourceOther);
+    if (code >= 0)
+	code = code1;
+
 
     /* Create the Pages tree. */
 
@@ -1279,8 +1231,8 @@ pdf_close(gx_device * dev)
     pprintld3(s, "<< /Size %ld /Root %ld 0 R /Info %ld 0 R\n",
 	      pdev->next_id, Catalog_id, Info_id);
     stream_puts(s, "/ID [");
-    pdf_put_string(pdev, pdev->fileID, sizeof(pdev->fileID));
-    pdf_put_string(pdev, pdev->fileID, sizeof(pdev->fileID));
+    psdf_write_string(pdev->strm, pdev->fileID, sizeof(pdev->fileID), 0);
+    psdf_write_string(pdev->strm, pdev->fileID, sizeof(pdev->fileID), 0);
     stream_puts(s, "]\n");
     if (pdev->OwnerPassword.size > 0) {
 	pprintld1(s, "/Encrypt %ld 0 R ", Encrypt_id);
@@ -1316,8 +1268,31 @@ pdf_close(gx_device * dev)
     pdev->pages = 0;
     pdev->num_pages = 0;
 
+    if (pdev->ForOPDFRead && pdev->OPDFReadProcsetPath.size) {
+        /* pdf_open_dcument could set up filters for entire document.
+           Removing them now. */
+	int status;
+    
+	stream_putc(s, 0x04);
+	while (s->strm) {
+	    s = s->strm;
+	}
+	status = s_close_filters(&pdev->strm, s);
+	if (status < 0 && code == 0)
+	    code = gs_error_ioerror;
+    }
     code1 = gdev_vector_close_file((gx_device_vector *) pdev);
     if (code >= 0)
 	code = code1;
+    if (pdev->max_referred_page >= pdev->next_page + 1) {
+        /* Note : pdev->max_referred_page counts from 1, 
+	   and pdev->next_page counts from 0. */
+	eprintf2("ERROR: A pdfmark destination page %d points beyond the last page %d.\n",
+		pdev->max_referred_page, pdev->next_page);
+#if 0 /* Temporary disabled due to Bug 687686. */
+	if (code >= 0)
+	    code = gs_note_error(gs_error_rangecheck);
+#endif
+    }
     return pdf_close_files(pdev, code);
 }
