@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdfm.c,v 1.28.2.1 2003/12/12 22:08:01 giles Exp $ */
+/* $Id: gdevpdfm.c,v 1.50 2005/10/17 19:23:44 leonardo Exp $ */
 /* pdfmark processing for PDF-writing driver */
 #include "math_.h"
 #include "memory_.h"
@@ -49,6 +49,8 @@ private_st_pdf_article();
 				/* in 1st argument */
 #define PDFMARK_NO_REFS 8	/* don't substitute references for names */
 				/* anywhere */
+#define PDFMARK_TRUECTM 16	/* pass the true CTM to the procedure, */
+				/* not the one transformed to reflect the default user space */
 typedef struct pdfmark_name_s {
     const char *mname;
     pdfmark_proc((*proc));
@@ -117,6 +119,8 @@ pdfmark_page_number(gx_device_pdf * pdev, const gs_param_string * pnstr)
 	--page;
     else if (pdfmark_scan_int(pnstr, &page) < 0)
 	page = 0;
+    if (pdev->max_referred_page < page)
+	pdev->max_referred_page = page;
     return page;
 }
 
@@ -223,11 +227,12 @@ pdfmark_make_rect(char str[MAX_RECT_STRING], const gs_rect * prect)
 {
     /*
      * We have to use a stream and pprintf, rather than sprintf,
-     * because printf formats can't express the PDF restriction son
+     * because printf formats can't express the PDF restrictions on
      * the form of the output.
      */
     stream s;
 
+    s_init(&s, NULL);
     swrite_string(&s, (byte *)str, MAX_RECT_STRING - 1);
     pprintg4(&s, "[%g %g %g %g]",
 	     prect->p.x, prect->p.y, prect->q.x, prect->q.y);
@@ -479,6 +484,7 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 	    char bstr[MAX_BORDER_STRING + 1];
 	    int code;
 
+	    s_init(&s, NULL);
 	    swrite_string(&s, (byte *)bstr, MAX_BORDER_STRING + 1);
 	    code = pdfmark_write_border(&s, pair + 1, pctm);
 	    if (code < 0)
@@ -578,6 +584,19 @@ pdfmark_put_ao_pairs(gx_device_pdf * pdev, cos_dict_t *pcd,
 			 COS_OBJECT_VALUE(&avalue, adict));
 	} else if (pdf_key_eq(Action + 1, "/GoTo"))
 	    pdfmark_put_pair(pcd, Action);
+	else if (Action[1].size < 30) {
+	    /* Hack: we could substitute names in pdfmark_process,
+	       now should recognize whether it was done. 
+	       Not a perfect method though. 
+	       Go with it for a while. */
+	    char buf[30];
+	    int d0, d1;
+
+	    memcpy(buf, Action[1].data, Action[1].size);
+	    buf[Action[1].size] = 0;
+	    if (sscanf(buf, "%d %d R", &d0, &d1) == 2)
+		pdfmark_put_pair(pcd, Action);
+	}
     }
     /*
      * If we have /Dest or /File without the right kind of action,
@@ -673,6 +692,7 @@ pdfmark_write_outline(gx_device_pdf * pdev, pdf_outline_node_t * pnode,
     stream *s;
 
     pdf_open_separate(pdev, pnode->id);
+    pnode->action->id = pnode->id;
     s = pdev->strm;
     stream_puts(s, "<< ");
     cos_dict_elements_write(pnode->action, pdev);
@@ -1024,6 +1044,37 @@ pdfmark_write_ps(stream *s, const gs_param_string * psource)
     return size + 1;
 }
 
+/* Start a XObject. */
+private int
+start_XObject(gx_device_pdf * pdev, bool compress, cos_stream_t **ppcs)
+{   pdf_resource_t *pres;
+    cos_stream_t *pcs;
+    int code;
+
+    code = pdf_open_page(pdev, PDF_IN_STREAM);
+    if (code < 0)
+	return code;
+    code = pdf_enter_substream(pdev, resourceXObject, gs_no_id, &pres, false, 
+		pdev->CompressFonts /* Have no better switch*/);
+    if (code < 0)
+	return code;
+    pdev->accumulating_a_global_object = true;
+    pcs = (cos_stream_t *)pres->object;
+    pdev->substream_Resources = cos_dict_alloc(pdev, "start_XObject");
+    if (!pdev->substream_Resources)
+	return_error(gs_error_VMerror);
+    if (pdev->ForOPDFRead) {
+	code = cos_dict_put_c_key_bool((cos_dict_t *)pres->object, "/.Global", true);
+	if (code < 0)
+	    return code;
+    }
+    pres->named = true;
+    pres->where_used = 0;	/* initially not used */
+    pcs->pres = pres;
+    *ppcs = pcs;
+    return 0;
+}
+
 /* PS pdfmark */
 #define MAX_PS_INLINE 100
 private int
@@ -1051,61 +1102,75 @@ pdfmark_PS(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	stream_puts(s, " PS\n");
     } else {
 	/* Put the PostScript code in a resource. */
-	pdf_resource_t *pres;
 	cos_stream_t *pcs;
-	uint size;
 	int code;
+	gs_id level1_id = gs_no_id;
+	pdf_resource_t *pres;
+	cos_value_t value;
 
-	code = pdf_make_named(pdev, objname, cos_type_stream,
-			      (cos_object_t **)&pcs, true);
+	if (level1.data != 0) {
+	    pdf_resource_t *pres;
+
+	    code = pdf_enter_substream(pdev, 
+			resourceXObject, 
+			gs_no_id, &pres, true, 
+			pdev->CompressFonts /* Have no better switch*/);
+	    if (code < 0)
+		return code;
+	    pcs = (cos_stream_t *)pres->object;
+	    if (pdev->ForOPDFRead && objname != 0) {
+		code = cos_dict_put_c_key_bool((cos_dict_t *)pres->object, "/.Global", true);
+		if (code < 0)
+		    return code;
+	    }
+	    pres->named = (objname != 0);
+	    pres->where_used = 0;
+	    pcs->pres = pres;
+	    DISCARD(pdfmark_write_ps(pdev->strm, &level1));
+	    code = pdf_exit_substream(pdev);
+	    if (code < 0)
+		return code;
+	    code = cos_write_object(pres->object, pdev);
+	    if (code < 0)
+		return code;
+	    level1_id = pres->object->id;
+	}
+	code = start_XObject(pdev, pdev->params.CompressPages, &pcs);
 	if (code < 0)
 	    return code;
-	code = pdf_alloc_resource(pdev, resourceXObject, gs_no_id, &pres,
-				  pcs->id);
-	if (code < 0)
-	    return code;
-	pres->object = COS_OBJECT(pcs);
+	pres = pdev->accumulating_substream_resource;
 	code = cos_stream_put_c_strings(pcs, "/Type", "/XObject");
 	if (code < 0)
 	    return code;
 	code = cos_stream_put_c_strings(pcs, "/Subtype", "/PS");
 	if (code < 0)
 	    return code;
-	if (level1.data != 0) {
-	    long level1_id = pdf_obj_ref(pdev);
-	    char r[10 + 5];	/* %ld 0 R\0 */
-	    stream *s;
-	    long length_id = pdf_obj_ref(pdev);
+	if (level1_id != gs_no_id) {
+	    char r[MAX_DEST_STRING];
 
 	    sprintf(r, "%ld 0 R", level1_id);
 	    code = cos_dict_put_c_key_string(cos_stream_dict(pcs), "/Level1",
 					     (byte *)r, strlen(r));
 	    if (code < 0)
 		return code;
-	    pdf_open_separate(pdev, level1_id);
-	    s = pdev->strm;
-	    pprintld1(s, "<</Length %ld 0 R>>stream\n", length_id);
-	    size = pdfmark_write_ps(s, &level1);
-	    stream_puts(s, "endstream\n");
-	    pdf_end_separate(pdev);
-	    pdf_open_separate(pdev, length_id);
-	    pprintld1(s, "%ld\n", (long)size);
-	    pdf_end_separate(pdev);
 	}
-	size = pdfmark_write_ps(pdev->streams.strm, &source);
-	code = cos_stream_add(pcs, size);
+	DISCARD(pdfmark_write_ps(pdev->strm, &source));
+	code = pdf_exit_substream(pdev);
 	if (code < 0)
 	    return code;
-	if (objname)
-	    pres->named = true;
-	else {
-	    /* Write the resource now, since it won't be written later. */
-	    COS_WRITE_OBJECT(pcs, pdev);
-	    COS_RELEASE(pcs, "pdfmark_PS");
+	code = pdf_substitute_resource(pdev, &pres, resourceXObject, NULL, false);
+	if (code < 0)
+	    return code;
+	if (objname != 0) {
+	    code = cos_dict_put(pdev->local_named_objects, objname->data,
+				objname->size, cos_object_value(&value, (cos_object_t *)pcs));
+	    if (code < 0)
+		return code;
 	}
 	code = pdf_open_contents(pdev, PDF_IN_STREAM);
 	if (code < 0)
 	    return code;
+        pcs->pres->where_used |= pdev->used_mask;
 	pprintld1(pdev->strm, "/R%ld Do\n", pcs->id);
     }
     return 0;
@@ -1359,49 +1424,52 @@ pdfmark_BP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     gs_rect bbox;
     cos_stream_t *pcs;
     int code;
-    double xscale = pdev->HWResolution[0] / 72.0,
-	yscale = pdev->HWResolution[1] / 72.0;
-    char bbox_str[6 + 4 * 12];
+    gs_matrix ictm;
+    byte bbox_str[6 + 6 * 15], matrix_str[6 + 6 * 15];
+    int bbox_str_len, matrix_str_len;
+    stream s;
 
     if (objname == 0 || count != 2 || !pdf_key_eq(&pairs[0], "/BBox"))
 	return_error(gs_error_rangecheck);
+    code = gs_matrix_invert(pctm, &ictm);
+    if (code < 0)
+	return code;
     if (sscanf((const char *)pairs[1].data, "[%lg %lg %lg %lg]",
-	       &bbox.p.x, &bbox.p.y, &bbox.q.x, &bbox.q.y) != 4
-	)
+	       &bbox.p.x, &bbox.p.y, &bbox.q.x, &bbox.q.y) != 4)
 	return_error(gs_error_rangecheck);
     if ((pdev->used_mask << 1) == 0)
 	return_error(gs_error_limitcheck);
-    {	pdf_resource_t *pres;
-	cos_value_t value;
-
-	code = pdf_open_page(pdev, PDF_IN_STREAM);
-	if (code < 0)
-	    return code;
-	code = pdf_enter_substream(pdev, resourceXObject, gs_no_id, &pres);
-	if (code < 0)
-	    return code;
-	pcs = (cos_stream_t *)pres->object;
-	pdev->substream_Resources = cos_stream_dict(pcs);
-	code = cos_dict_put(pdev->local_named_objects, objname->data,
-				objname->size, cos_object_value(&value, pres->object));
-	if (code < 0)
-	    return code;
-	pres->named = true;
-	pres->where_used = 0;	/* initially not used */
-	pcs->pres = pres;
+    code = start_XObject(pdev, pdev->params.CompressPages, &pcs);
+    if (code < 0)
+	return code;
+    {	byte *s = gs_alloc_string(pdev->memory, objname->size, "pdfmark_PS");
+	
+	if (s == NULL)
+	    return_error(gs_error_VMerror);
+	memcpy(s, objname->data, objname->size);
+	pdev->objname.data = s;
+	pdev->objname.size = objname->size;
     }
     pcs->is_graphics = true;
     gs_bbox_transform(&bbox, pctm, &bbox);
-    sprintf(bbox_str, "[%.8g %.8g %.8g %.8g]",
-	    bbox.p.x * xscale, bbox.p.y * yscale,
-	    bbox.q.x * xscale, bbox.q.y * yscale);
+    s_init(&s, NULL);
+    swrite_string(&s, bbox_str, sizeof(bbox_str));
+    pprintg4(&s, "[%g %g %g %g]",
+	    bbox.p.x, bbox.p.y, bbox.q.x, bbox.q.y);
+    bbox_str_len = stell(&s);
+    swrite_string(&s, matrix_str, sizeof(bbox_str));
+    pprintg6(&s, "[%g %g %g %g %g %g]",
+	    ictm.xx, ictm.xy, ictm.yx, ictm.yy, ictm.tx, ictm.ty);
+    matrix_str_len = stell(&s);
     if ((code = cos_stream_put_c_strings(pcs, "/Type", "/XObject")) < 0 ||
 	(code = cos_stream_put_c_strings(pcs, "/Subtype", "/Form")) < 0 ||
 	(code = cos_stream_put_c_strings(pcs, "/FormType", "1")) < 0 ||
-	(code = cos_stream_put_c_strings(pcs, "/Matrix", "[1 0 0 1 0 0]")) < 0 ||
 	(code = cos_dict_put_c_key_string(cos_stream_dict(pcs), "/BBox",
-					  (byte *)bbox_str,
-					  strlen(bbox_str))) < 0
+					  bbox_str, bbox_str_len)) < 0 ||
+	(code = cos_dict_put_c_key_string(cos_stream_dict(pcs), "/Matrix",
+				      matrix_str, matrix_str_len)) < 0 ||
+ 	(code = cos_dict_put_c_key_object(cos_stream_dict(pcs), "/Resources", 
+ 					  COS_OBJECT(pdev->substream_Resources))) < 0
 	)
 	return code;
     return 0;
@@ -1413,11 +1481,26 @@ pdfmark_EP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	   const gs_matrix * pctm, const gs_param_string * no_objname)
 {
     int code;
+    cos_value_t value;
+    pdf_resource_t *pres = pdev->accumulating_substream_resource;
+    gs_const_string objname = pdev->objname;
 
     code = pdf_add_procsets(pdev->substream_Resources, pdev->procsets);
     if (code < 0)
 	return code;
     code = pdf_exit_substream(pdev);
+    if (code < 0)
+	return code;
+    code = pdf_substitute_resource(pdev, &pres, resourceXObject, NULL, true);
+    if (code < 0)
+	return code;
+    if (objname.size) {
+	code = cos_dict_put(pdev->local_named_objects, objname.data,
+				objname.size, cos_object_value(&value, (cos_object_t *)pres->object));
+	if (code < 0)
+	    return code;
+    }
+    gs_free_const_string(pdev->memory, objname.data, objname.size, "pdfmark_EP");
     if (code < 0)
 	return code;
     return 0;
@@ -1429,7 +1512,6 @@ pdfmark_SP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
 	   const gs_matrix * pctm, const gs_param_string * no_objname)
 {
     cos_object_t *pco;		/* stream */
-    gs_matrix ctm;
     int code;
 
     if (count != 1)
@@ -1441,10 +1523,7 @@ pdfmark_SP(gx_device_pdf * pdev, gs_param_string * pairs, uint count,
     code = pdf_open_contents(pdev, PDF_IN_STREAM);
     if (code < 0)
 	return code;
-    ctm = *pctm;
-    ctm.tx *= pdev->HWResolution[0] / 72.0;
-    ctm.ty *= pdev->HWResolution[1] / 72.0;
-    pdf_put_matrix(pdev, "q ", &ctm, "cm");
+    pdf_put_matrix(pdev, "q ", pctm, "cm");
     pprintld1(pdev->strm, "/R%ld Do Q\n", pco->id);
     pco->pres->where_used |= pdev->used_mask;
     return 0;
@@ -1824,9 +1903,9 @@ private const pdfmark_name mark_names[] =
     {"DOCINFO",      pdfmark_DOCINFO,     0},
     {"DOCVIEW",      pdfmark_DOCVIEW,     0},
 	/* Named objects. */
-    {"BP",           pdfmark_BP,          PDFMARK_NAMEABLE},
+    {"BP",           pdfmark_BP,          PDFMARK_NAMEABLE | PDFMARK_TRUECTM},
     {"EP",           pdfmark_EP,          0},
-    {"SP",           pdfmark_SP,          PDFMARK_ODD_OK | PDFMARK_KEEP_NAME},
+    {"SP",           pdfmark_SP,          PDFMARK_ODD_OK | PDFMARK_KEEP_NAME | PDFMARK_TRUECTM},
     {"OBJ",          pdfmark_OBJ,         PDFMARK_NAMEABLE},
     {"PUT",          pdfmark_PUT,         PDFMARK_ODD_OK | PDFMARK_KEEP_NAME},
     {".PUTDICT",     pdfmark_PUTDICT,     PDFMARK_ODD_OK | PDFMARK_KEEP_NAME},
@@ -1880,18 +1959,6 @@ pdfmark_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
 	       &ctm.xx, &ctm.xy, &ctm.yx, &ctm.yy, &ctm.tx, &ctm.ty) != 6
 	)
 	return_error(gs_error_rangecheck);
-    /*
-     * Our coordinate system is scaled so that user space is always
-     * default user space.  Adjust the CTM to match this.
-     */
-    {
-	double xscale = 72.0 / pdev->HWResolution[0],
-	    yscale = 72.0 / pdev->HWResolution[1];
-
-	ctm.xx *= xscale, ctm.xy *= yscale;
-	ctm.yx *= xscale, ctm.yy *= yscale;
-	ctm.tx *= xscale, ctm.ty *= yscale;
-    }
     size -= 2;			/* remove CTM & pdfmark name */
     for (pmn = mark_names; pmn->mname != 0; ++pmn)
 	if (pdf_key_eq(pts, pmn->mname)) {
@@ -1900,6 +1967,20 @@ pdfmark_process(gx_device_pdf * pdev, const gs_param_string_array * pma)
 	    gs_param_string *pairs;
 	    int j;
 
+	    /*
+	     * Our coordinate system is scaled so that user space is always
+	     * default user space.  Adjust the CTM to match this, except if this
+	     * particular pdfmark requires the "true" CTM.
+	     */
+	    if (pmn->options & PDFMARK_TRUECTM)
+		DO_NOTHING;
+	    else {
+		double xscale = 72.0 / pdev->HWResolution[0],
+		       yscale = 72.0 / pdev->HWResolution[1];
+		ctm.xx *= xscale, ctm.xy *= yscale;
+		ctm.yx *= xscale, ctm.yy *= yscale;
+		ctm.tx *= xscale, ctm.ty *= yscale;
+	    }
 	    if (size & !odd_ok)
 		return_error(gs_error_rangecheck);
 	    if (pmn->options & PDFMARK_NAMEABLE) {
