@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdfo.c,v 1.19.2.5 2004/02/13 08:32:25 igor Exp $ */
+/* $Id: gdevpdfo.c,v 1.35 2005/10/18 09:05:58 leonardo Exp $ */
 /* Cos object support */
 #include "memory_.h"
 #include "string_.h"
@@ -315,9 +315,8 @@ cos_value_write_spaced(const cos_value_t *pcv, gx_device_pdf *pdev,
 	    case '/': case '(': case '<': break;
 	    default: stream_putc(s, ' ');
 	    }
-	pdf_write_value(pdev, pcv->contents.chars.data,
+	return pdf_write_value(pdev, pcv->contents.chars.data,
 			pcv->contents.chars.size, object_id);
-	break;
     case COS_VALUE_RESOURCE:
 	pprintld1(s, "/R%ld", pcv->contents.object->id);
 	break;
@@ -523,7 +522,7 @@ cos_array_write(const cos_object_t *pco, gx_device_pdf *pdev, gs_id object_id)
 	    stream_putc(s, '\n');
 	for (; pcae->index > last_index; ++last_index)
 	    stream_puts(s, "null\n");
-	cos_value_write(&pcae->value, pdev);
+	cos_value_write_spaced(&pcae->value, pdev, false, object_id);
     }
     DISCARD(cos_array_reorder(pca, first));
     stream_puts(s, "]");
@@ -646,6 +645,7 @@ cos_array_add_object(cos_array_t *pca, cos_object_t *pco)
 {
     cos_value_t value;
 
+    value.contents.chars.size = 0; /* Quiet a warning appeared with MSVC6 inline optimization. */
     return cos_array_add(pca, cos_object_value(&value, pco));
 }
 
@@ -803,7 +803,8 @@ cos_dict_objects_write(const cos_dict_t *pcd, gx_device_pdf *pdev)
 
     for (; pcde; pcde = pcde->next)
 	if (COS_VALUE_IS_OBJECT(&pcde->value) &&
-	    pcde->value.contents.object->id)
+	    pcde->value.contents.object->id  &&
+	    !pcde->value.contents.object->written /* ForOPDFRead only. */)
 	    cos_write_object(pcde->value.contents.object, pdev);
     return 0;
 }
@@ -925,6 +926,13 @@ cos_dict_put_c_key_int(cos_dict_t *pcd, const char *key, int value)
     return cos_dict_put_c_key_string(pcd, key, (byte *)str, strlen(str));
 }
 int
+cos_dict_put_c_key_bool(cos_dict_t *pcd, const char *key, bool value)
+{
+    return cos_dict_put_c_key_string(pcd, key, 
+		(const byte *)(value ? "true" : "false"),
+			      (value ? 4 : 5));
+}
+int
 cos_dict_put_c_key_real(cos_dict_t *pcd, const char *key, floatp value)
 {
     byte str[50];		/****** ADHOC ******/
@@ -966,6 +974,11 @@ cos_dict_put_string(cos_dict_t *pcd, const byte *key_data, uint key_size,
 			cos_string_value(&cvalue, value_data, value_size));
 }
 int
+cos_dict_put_string_copy(cos_dict_t *pcd, const char *key, const char *value)
+{
+    return cos_dict_put_c_key_string(pcd, key, (byte *)value, strlen(value));
+}
+int
 cos_dict_put_c_strings(cos_dict_t *pcd, const char *key, const char *value)
 {
     cos_value_t cvalue;
@@ -1038,7 +1051,7 @@ cos_dict_equal(const cos_object_t *pco0, const cos_object_t *pco1, gx_device_pdf
 	if (code < 0)
 	    return code;
 	if (!code)
-		    return false;
+	    return false;
     }
     return true;
 }
