@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdtc.c,v 1.17.2.2 2004/01/29 09:47:43 igor Exp $ */
+/* $Id: gdevpdtc.c,v 1.42 2005/04/27 16:40:44 igor Exp $ */
 /* Composite and CID-based text processing for pdfwrite. */
 #include "memory_.h"
 #include "string_.h"
@@ -147,6 +147,7 @@ process_composite_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
 		pte->returned.total_width.y = total_width.y +=
 		    out.returned.total_width.y;
 	    }
+	    pdf_text_release_cgp(penum);
 	}
 	if (font_code == 2)
 	    break;
@@ -158,9 +159,7 @@ process_composite_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
     }
     if (!return_width)
 	return 0;
-    return gx_path_add_point(penum->path,
-			     penum->origin.x + float2fixed(total_width.x),
-			     penum->origin.y + float2fixed(total_width.y));
+    return pdf_shift_text_currentpoint(penum, &total_width);
 }
 
 /* ---------------- CMap-based composite font ---------------- */
@@ -312,7 +311,11 @@ scan_cmap_text(pdf_text_enum_t *pte)
     uint index = scan.index, xy_index = scan.xy_index;
     uint font_index0 = 0x7badf00d;
     bool done = false;
+    pdf_char_glyph_pairs_t p;
 
+    p.num_all_chars = 1;
+    p.num_unused_chars = 1;
+    p.unused_offset = 0;
     pte->returned.total_width.x = pte->returned.total_width.y = 0;;
     for (;;) {
 	uint break_index, break_xy_index;
@@ -332,10 +335,9 @@ scan_cmap_text(pdf_text_enum_t *pte)
 	    gs_glyph glyph;
 	    pdf_font_descriptor_t *pfd;
 	    byte *glyph_usage;
-	    double *real_widths, *w, *v;
+	    double *real_widths, *w, *v, *w0;
 	    int char_cache_size, width_cache_size;
 	    uint cid;
-	    gs_char unicode_char;
 
 	    break_index = scan.index;
 	    break_xy_index = scan.xy_index;
@@ -360,7 +362,9 @@ scan_cmap_text(pdf_text_enum_t *pte)
 	    if (glyph == GS_NO_GLYPH)
 		glyph = GS_MIN_CID_GLYPH;
 	    cid = glyph - GS_MIN_CID_GLYPH;
-	    code = pdf_obtain_cidfont_resource(pdev, subfont, &pdsubf, &glyph, 1);
+	    p.s[0].glyph = glyph;
+	    p.s[0].chr = cid;
+	    code = pdf_obtain_cidfont_resource(pdev, subfont, &pdsubf, &p);
 	    if (code < 0)
 		return code;
 	    font_change = (pdsubf != pdsubf0 && pdsubf0 != NULL);
@@ -374,21 +378,27 @@ scan_cmap_text(pdf_text_enum_t *pte)
 	    if (code < 0)
 		return code;
 	    pfd = pdsubf->FontDescriptor;
-	    code = pdf_resize_resource_arrays(pdev, pdsubf, cid);
+	    code = pdf_resize_resource_arrays(pdev, pdsubf, cid + 1);
 	    if (code < 0)
 		return code;
-	    code = pdf_obtain_cidfont_widths_arrays(pdev, pdsubf, wmode, &w, &v);
+	    code = pdf_obtain_cidfont_widths_arrays(pdev, pdsubf, wmode, &w, &w0, &v);
 	    if (code < 0)
 		return code;
-	    unicode_char = subfont->procs.decode_glyph(subfont, glyph);
-    	    if (unicode_char != GS_NO_CHAR) {
+	    {
 		pdf_font_resource_t *pdfont;
 
 		code = pdf_obtain_parent_type0_font_resource(pdev, pdsubf, 
 				&font->data.CMap->CMapName, &pdfont);
 		if (code < 0)
 		    return code;
-		code = pdf_add_ToUnicode(pdev, subfont, pdfont, glyph, cid);
+		if (pdf_is_CID_font(subfont)) {
+		    /* Since PScript5.dll creates GlyphNames2Unicode with character codes
+		    instead CIDs, and with the WinCharSetFFFF-H2 CMap
+		    character codes appears different than CIDs (Bug 687954),
+		    pass the character code intead the CID. */
+		    code = pdf_add_ToUnicode(pdev, subfont, pdfont, chr + GS_MIN_CID_GLYPH, chr);
+		} else
+		    code = pdf_add_ToUnicode(pdev, subfont, pdfont, glyph, cid);
 		if (code < 0)
 		    return code;
 	    }
@@ -410,7 +420,7 @@ scan_cmap_text(pdf_text_enum_t *pte)
 		cid = 0, code = 1;  /* undefined glyph. */
 	    } else if (code < 0)
 		return code;
-	    if (cid < 0 || cid >= char_cache_size || cid >= width_cache_size)
+	    if (cid >= char_cache_size || cid >= width_cache_size)
 		return_error(gs_error_unregistered); /* Must not happen */
 	    if (code == 0 /* just copied */ || pdsubf->Widths[cid] == 0) {
 		pdf_glyph_widths_t widths;
@@ -435,6 +445,16 @@ scan_cmap_text(pdf_text_enum_t *pte)
 		    }
 		    real_widths[cid] = widths.real_width.w;
 		}
+		if (wmode) {
+		    /* Since AR5 use W or DW to compute the x-coordinate of
+		       v-vector, comupte and store the glyph width for WMode 0. */
+		    /* fixme : skip computing real_width here. */
+		    code = pdf_glyph_widths(pdsubf, 0, glyph, (gs_font *)subfont, &widths,
+				    pte->cdevproc_callout ? pte->cdevproc_result : NULL);
+		    if (code < 0)
+			return code;
+		    w0[cid] = widths.Width.w;
+		}
 		if (pdsubf->u.cidfont.CIDToGIDMap != 0) {
 		    gs_font_cid2 *subfont2 = (gs_font_cid2 *)subfont;
 
@@ -443,6 +463,8 @@ scan_cmap_text(pdf_text_enum_t *pte)
 		}
 	    }
 	    pdsubf->used[cid >> 3] |= 0x80 >> (cid & 7);
+	    if (wmode)
+		pdsubf->u.cidfont.used2[cid >> 3] |= 0x80 >> (cid & 7);
 	    if (pte->cdevproc_callout) {
 		 /* Only handle a single character because its width is stored 
 		    into pte->cdevproc_result, and process_text_modify_width neds it. 
@@ -457,7 +479,7 @@ scan_cmap_text(pdf_text_enum_t *pte)
 	    gs_matrix m0, m1, m2, m3;
 	    int xy_index_step = (pte->text.x_widths != NULL && /* see gs_text_replaced_width */
 				 pte->text.x_widths == pte->text.y_widths ? 2 : 1);
-	    gs_const_string save_text;
+	    gs_text_params_t save_text;
 
 	    code = pdf_font_orig_matrix(subfont0, &m0);
 	    if (code < 0)
@@ -493,8 +515,7 @@ scan_cmap_text(pdf_text_enum_t *pte)
 	    /* process_text_modify_width breaks text parameters. 
 	       We would like to improve it someday. 
 	       Now save them locally and restore after the call. */
-	    save_text.data = pte->text.data.bytes;
-	    save_text.size = pte->text.size;
+	    save_text = pte->text;
 	    str.data = scan.text.data.bytes + index;
 	    str.size = break_index - index;
 	    if (pte->text.x_widths != NULL)
@@ -508,8 +529,7 @@ scan_cmap_text(pdf_text_enum_t *pte)
 		pte->text.x_widths -= xy_index * xy_index_step;
 	    if (pte->text.y_widths != NULL)
 		pte->text.y_widths -= xy_index * xy_index_step;
-	    pte->text.data.bytes = save_text.data;
-	    pte->text.size = save_text.size;
+	    pte->text = save_text;
 	    pte->cdevproc_callout = false;
 	    if (code < 0) {
 		pte->index = index;
@@ -518,13 +538,11 @@ scan_cmap_text(pdf_text_enum_t *pte)
 	    }
 	    pte->index = break_index;
 	    pte->xy_index = break_xy_index;
-
-	    code = gx_path_add_point(pte->path,
-				 pte->origin.x + float2fixed(wxy.x),
-				 pte->origin.y + float2fixed(wxy.y));
+	    code = pdf_shift_text_currentpoint(pte, &wxy);
 	    if (code < 0)
 		return code;
 	} 
+	pdf_text_release_cgp(pte);
 	index = break_index;
 	xy_index = break_xy_index;
 	if (done || rcode != 0)
@@ -568,6 +586,7 @@ process_cmap_text(gs_text_enum_t *penum, void *vbuf, uint bsize)
 int
 process_cid_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
 {
+    pdf_text_enum_t *penum = (pdf_text_enum_t *)pte;
     uint operation = pte->text.operation;
     gs_text_enum_t save;
     gs_font *scaled_font = pte->current_font; /* CIDFont */
@@ -605,7 +624,7 @@ process_cid_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
 
 	    if (gnum & ~0xffffL)
 		return_error(gs_error_rangecheck);
-	    *pchars++ = (byte)(gnum >> 8);
+	    *pchars++ = (byte)(gnum >> 8);  
 	    *pchars++ = (byte)gnum;
 	}
     }
@@ -620,7 +639,7 @@ process_cid_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
 
     /* Find or create the CIDFont resource. */
 
-    code = pdf_obtain_font_resource(pte, NULL, &pdsubf);
+    code = pdf_obtain_font_resource(penum, NULL, &pdsubf);
     if (code < 0)
 	return code;
 
@@ -628,7 +647,7 @@ process_cid_text(gs_text_enum_t *pte, void *vbuf, uint bsize)
 
     if (pdsubf->u.cidfont.glyphshow_font_id != 0)
  	font0 = (gs_font_type0 *)gs_find_font_by_id(font->dir, 
- 		    pdsubf->u.cidfont.glyphshow_font_id);
+ 		    pdsubf->u.cidfont.glyphshow_font_id, &scaled_font->FontMatrix);
     if (font0 == NULL) {
   	code = gs_font_type0_from_cidfont(&font0, font, font->WMode,
  					  &scale_matrix, font->memory);
