@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdtd.c,v 1.11.2.1 2003/12/12 22:08:01 giles Exp $ */
+/* $Id: gdevpdtd.c,v 1.22 2005/04/05 15:44:44 igor Exp $ */
 /* FontDescriptor implementation for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
@@ -216,7 +216,9 @@ pdf_font_descriptor_alloc(gx_device_pdf *pdev, pdf_font_descriptor_t **ppfd,
 {
     pdf_font_descriptor_t *pfd;
     pdf_base_font_t *pbfont;
-    int code = pdf_base_font_alloc(pdev, &pbfont, font, &font->FontMatrix, false, !embed);
+    int code = pdf_base_font_alloc(pdev, &pbfont, font, 
+		(font->orig_FontMatrix.xx == 0 && font->orig_FontMatrix.xy == 0 
+		    ? &font->FontMatrix : &font->orig_FontMatrix), false, !embed);
 
     if (code < 0)
 	return code;
@@ -550,14 +552,17 @@ int
 pdf_finish_FontDescriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
 {
     int code = 0;
+    cos_dict_t *pcd = 0;
 
     if (!pfd->common.object->written &&
 	(code = pdf_compute_font_descriptor(pfd)) >= 0 &&
 	(!pfd->embed ||
 	 (code = pdf_write_embedded_font(pdev, pfd->base_font, 
-				&pfd->common.values.FontBBox, pfd->common.rid)) >= 0)
-	)
-	DO_NOTHING;
+				&pfd->common.values.FontBBox, 
+				pfd->common.rid, &pcd)) >= 0)
+	) {
+        pdf_set_FontFile_object(pfd->base_font, pcd);
+    }
     return code;
 }
 
@@ -633,5 +638,24 @@ pdf_write_FontDescriptor(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
     stream_puts(s, ">>\n");
     pdf_end_separate(pdev);
     pfd->common.object->written = true;
+    {	const cos_object_t *pco = (const cos_object_t *)pdf_get_FontFile_object(pfd->base_font);
+	if (pco != NULL) {
+	    code = COS_WRITE_OBJECT(pco, pdev);
+	    if (code < 0)
+		return code;
+	}
+    }
+    return 0;
+}
+
+/*
+ * Release a FontDescriptor components.
+ */
+int
+pdf_release_FontDescriptor_components(gx_device_pdf *pdev, pdf_font_descriptor_t *pfd)
+{
+    gs_free_object(pdev->pdf_memory, pfd->base_font, "pdf_release_FontDescriptor_components");
+    pfd->base_font = NULL;
+    /* fixme: underimplemented. */
     return 0;
 }
