@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdti.c,v 1.14.2.3 2004/01/29 12:08:47 igor Exp $ */
+/* $Id: gdevpdti.c,v 1.53 2005/10/12 08:16:50 leonardo Exp $ */
 /* Bitmap font implementation for pdfwrite */
 #include "memory_.h"
 #include "string_.h"
@@ -42,6 +42,8 @@ struct pdf_char_proc_s {
     int y_offset;		/* of character (0,0) */
     gs_char char_code;
     gs_const_string char_name;
+    gs_point real_width;        /* Not used with synthesised bitmap fonts. */
+    gs_point v;			/* Not used with synthesised bitmap fonts. */
 };
 
 /* The descriptor is public for pdf_resource_type_structs. */
@@ -109,7 +111,9 @@ assign_char_code(gx_device_pdf * pdev, int width)
 	pdfont->u.simple.FirstChar = 0;
     }
     c = ++(pdfont->u.simple.LastChar);
-    pdfont->Widths[c] = pdev->char_width.x;
+    pdfont->Widths[c] = psdf_round(pdev->char_width.x, 100, 10); /* See 
+			pdf_write_Widths about rounding. We need to provide 
+			a compatible data for Tj. */
     if (c > pbfs->max_embedded_code)
 	pbfs->max_embedded_code = c;
 
@@ -348,7 +352,7 @@ pdf_do_char_image(gx_device_pdf * pdev, const pdf_char_proc_t * pcp,
 int
 pdf_write_bitmap_fonts_Encoding(gx_device_pdf *pdev)
 {
-    const pdf_bitmap_fonts_t *pbfs = pdev->text->bitmap_fonts;
+    pdf_bitmap_fonts_t *pbfs = pdev->text->bitmap_fonts;
 
     if (pbfs->bitmap_encoding_id) {
 	stream *s;
@@ -371,119 +375,104 @@ pdf_write_bitmap_fonts_Encoding(gx_device_pdf *pdev)
 	}
 	stream_puts(s, "\n] >>\n");
 	pdf_end_separate(pdev);
+	pbfs->bitmap_encoding_id = 0;
     }
     return 0;
 }
 
+/*
+ * Start charproc accumulation for a Type 3 font.
+ */
+int
+pdf_start_charproc_accum(gx_device_pdf *pdev)
+{
+    pdf_char_proc_t *pcp;
+    pdf_resource_t *pres;
+    int code = pdf_enter_substream(pdev, resourceCharProc, gs_next_ids(pdev->memory, 1), 
+				   &pres, false, pdev->CompressFonts);
+
+    if (code < 0)
+       return code;
+    pcp = (pdf_char_proc_t *)pres;
+    pcp->char_next = NULL;
+    pcp->font = NULL;
+    pcp->char_code = GS_NO_CHAR;
+    pcp->char_name.data = NULL;
+    pcp->char_name.size = 0;
+    return 0;
+}
+
 /*
  * Install charproc accumulator for a Type 3 font.
  */
 int
-pdf_install_charproc_accum(gx_device_pdf *pdev, gs_font *font, const double *pw, 
+pdf_set_charproc_attrs(gx_device_pdf *pdev, gs_font *font, const double *pw, int narg,
 		gs_text_cache_control_t control, gs_char ch, gs_const_string *gnstr)
 {
     pdf_font_resource_t *pdfont;
-    byte *glyph_usage;
-    pdf_resource_t *pres;
+    pdf_resource_t *pres = pdev->accumulating_substream_resource;
     pdf_char_proc_t *pcp;
-    double *real_widths;
-    int char_cache_size, width_cache_size;
     int code;
 
-    code = pdf_attached_font_resource(pdev, font, &pdfont,
-		&glyph_usage, &real_widths, &char_cache_size, &width_cache_size);
+    code = pdf_attached_font_resource(pdev, font, &pdfont, NULL, NULL, NULL, NULL);
     if (code < 0)
 	return code;
-    if (ch != GS_NO_CHAR) {
-	int i;
-	gs_glyph glyph0 = font->procs.encode_char(font, ch, GLYPH_SPACE_NAME);
-
-	if (ch >= char_cache_size || ch >= width_cache_size)
-	    return_error(gs_error_unregistered);
-	real_widths[ch * 2    ] = pdfont->Widths[ch] = pw[font->WMode ? 6 : 0];
-	real_widths[ch * 2 + 1] = pw[font->WMode ? 7 : 1];
-	glyph_usage[ch / 8] |= 0x80 >> (ch & 7);
-	pdfont->used[ch >> 3] |= 0x80 >> (ch & 7);
-	if (pdfont->u.simple.v != NULL && font->WMode) {
-	    pdfont->u.simple.v[ch].x = pw[8];
-	    pdfont->u.simple.v[ch].y = pw[9];
-	}
-	for (i = 0; i < 256; i++) {
-	    gs_glyph glyph = font->procs.encode_char(font, i, 
-			font->FontType == ft_user_defined ? GLYPH_SPACE_NOGEN
-							  : GLYPH_SPACE_NAME);
-
-	    if (glyph == glyph0) {
-		real_widths[i * 2    ] = real_widths[ch * 2    ];
-		real_widths[i * 2 + 1] = real_widths[ch * 2 + 1];
-		glyph_usage[i / 8] |= 0x80 >> (i & 7);
-		pdfont->used[i >> 3] |= 0x80 >> (i & 7);
-		pdfont->u.simple.v[i] = pdfont->u.simple.v[ch];
-		pdfont->Widths[i] = pdfont->Widths[ch];
-	    }
-	}
-    }
-    code = pdf_enter_substream(pdev, resourceCharProc, gs_next_ids(1), &pres);
-    if (code < 0)
-	return code;
-    pcp = (pdf_char_proc_t *) pres;
+    pcp = (pdf_char_proc_t *)pres;
+    pcp->char_next = NULL;
     pcp->font = pdfont;
-    pcp->char_next = pdfont->u.simple.s.type3.char_procs;
-    pdfont->u.simple.s.type3.char_procs = pcp;
     pcp->char_code = ch;
     pcp->char_name = *gnstr;
+    pcp->real_width.x = pw[font->WMode && narg > 6 ? 6 : 0];
+    pcp->real_width.y = pw[font->WMode && narg > 6 ? 7 : 1];
+    pcp->v.x = (narg > 8 ? pw[8] : 0);
+    pcp->v.y = (narg > 8 ? pw[9] : 0);
     if (control == TEXT_SET_CHAR_WIDTH) {
 	/* PLRM 5.7.1 "BuildGlyph" reads : "Normally, it is unnecessary and 
 	undesirable to initialize the current color parameter, because show 
 	is defined to paint glyphs with the current color."
 	However comparefiles/Bug687044.ps doesn't follow that. */
-	pdev->skip_colors = false;
+	pdev->skip_colors = false; 
 	pprintg2(pdev->strm, "%g %g d0\n", (float)pw[0], (float)pw[1]);
     } else {
 	pdev->skip_colors = true;
 	pprintg6(pdev->strm, "%g %g %g %g %g %g d1\n", 
 	    (float)pw[0], (float)pw[1], (float)pw[2], 
 	    (float)pw[3], (float)pw[4], (float)pw[5]);
+	pdfont->u.simple.s.type3.cached[ch >> 3] |= 0x80 >> (ch & 7);
     }
+    pdfont->used[ch >> 3] |= 0x80 >> (ch & 7);
     pdev->font3 = (pdf_resource_t *)pdfont;
     return 0;
 }
 
 /*
- * Enter the substream accumulation mode.
+ * Open a stream object in the temporary file.
  */
+
 int
-pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype, 
-			    gs_id id, pdf_resource_t **ppres) 
+pdf_open_aside(gx_device_pdf *pdev, pdf_resource_type_t rtype, 
+	gs_id id, pdf_resource_t **ppres, bool reserve_object_id, int options) 
 {
-    int sbstack_ptr = pdev->sbstack_depth;
-    stream *s, *save_strm = pdev->strm;
-    pdf_resource_t *pres;
-    pdf_data_writer_t writer;
     int code;
+    pdf_resource_t *pres;
+    stream *s, *save_strm = pdev->strm;
+    pdf_data_writer_t writer;
     static const pdf_filter_names_t fnames = {
 	PDF_FILTER_NAMES
     };
 
-    if (pdev->sbstack_depth >= pdev->sbstack_size)
-	return_error(gs_error_unregistered); /* Must not happen. */
+    pdev->streams.save_strm = pdev->strm;
     code = pdf_alloc_aside(pdev, PDF_RESOURCE_CHAIN(pdev, rtype, id),
-		pdf_resource_type_structs[rtype], &pres, 0);
+		pdf_resource_type_structs[rtype], &pres, reserve_object_id ? 0 : -1);
     if (code < 0)
 	return code;
     cos_become(pres->object, cos_type_stream);
     s = cos_write_stream_alloc((cos_stream_t *)pres->object, pdev, "pdf_enter_substream");
     if (s == 0)
 	return_error(gs_error_VMerror);
-    if (pdev->sbstack[sbstack_ptr].text_state == 0) {
-	pdev->sbstack[sbstack_ptr].text_state = pdf_text_state_alloc(pdev->pdf_memory);
-	if (pdev->sbstack[sbstack_ptr].text_state == 0)
-	    return_error(gs_error_VMerror);
-    }
     pdev->strm = s;
-    code = pdf_begin_data_stream(pdev, &writer,
-			     DATA_STREAM_NOT_BINARY | DATA_STREAM_NOLENGTH |
-			     (pdev->CompressFonts ? DATA_STREAM_COMPRESS : 0), 0);
+    code = pdf_append_data_stream_filters(pdev, &writer,
+			     options | DATA_STREAM_NOLENGTH, pres->object->id);
     if (code < 0) {
 	pdev->strm = save_strm;
 	return code;
@@ -494,6 +483,60 @@ pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype,
 	return code;
     }
     pdev->strm = writer.binary.strm;
+    *ppres = pres;
+    return 0;
+}
+
+/*
+ * Close a stream object in the temporary file.
+ */
+int
+pdf_close_aside(gx_device_pdf *pdev) 
+{
+    /* We should call pdf_end_data here, but we don't want to put pdf_data_writer_t
+       into pdf_substream_save stack to simplify garbager descriptors. 
+       Use a lower level functions instead that. */
+    stream *s = pdev->strm;
+    int status = s_close_filters(&s, cos_write_stream_from_pipeline(s));
+    cos_stream_t *pcs = cos_stream_from_pipeline(s);
+    int code = 0;
+
+    if (status < 0)
+	 code = gs_note_error(gs_error_ioerror);
+    pcs->is_open = false;
+    sclose(s);
+    pdev->strm = pdev->streams.save_strm;
+    return code;
+}
+
+/*
+ * Enter the substream accumulation mode.
+ */
+int
+pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype, 
+	gs_id id, pdf_resource_t **ppres, bool reserve_object_id, bool compress) 
+{
+    int sbstack_ptr = pdev->sbstack_depth;
+    pdf_resource_t *pres;
+    stream *save_strm = pdev->strm;
+    int code;
+
+    if (pdev->sbstack_depth >= pdev->sbstack_size)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (pdev->sbstack[sbstack_ptr].text_state == 0) {
+	pdev->sbstack[sbstack_ptr].text_state = pdf_text_state_alloc(pdev->pdf_memory);
+	if (pdev->sbstack[sbstack_ptr].text_state == 0)
+	    return_error(gs_error_VMerror);
+    }
+    code = pdf_open_aside(pdev, rtype, id, &pres, reserve_object_id, 
+		    (compress ? DATA_STREAM_COMPRESS : 0));
+    if (code < 0)
+	return code;
+    code = pdf_save_viewer_state(pdev, NULL);
+    if (code < 0) {
+	pdev->strm = save_strm;
+	return code;
+    }
     pdev->sbstack[sbstack_ptr].context = pdev->context;
     pdf_text_state_copy(pdev->sbstack[sbstack_ptr].text_state, pdev->text->text_state);
     pdf_set_text_state_default(pdev->text->text_state);
@@ -508,11 +551,22 @@ pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype,
     pdev->sbstack[sbstack_ptr].substream_Resources = pdev->substream_Resources;
     pdev->sbstack[sbstack_ptr].skip_colors = pdev->skip_colors;
     pdev->sbstack[sbstack_ptr].font3 = pdev->font3;
+    pdev->sbstack[sbstack_ptr].accumulating_substream_resource = pdev->accumulating_substream_resource;
+    pdev->sbstack[sbstack_ptr].charproc_just_accumulated = pdev->charproc_just_accumulated;
+    pdev->sbstack[sbstack_ptr].accumulating_a_global_object = pdev->accumulating_a_global_object;
+    pdev->sbstack[sbstack_ptr].pres_soft_mask_dict = pdev->pres_soft_mask_dict;
+    pdev->sbstack[sbstack_ptr].objname = pdev->objname;
     pdev->skip_colors = false;
+    pdev->charproc_just_accumulated = false;
+    pdev->pres_soft_mask_dict = NULL;
+    pdev->objname.data = NULL;
+    pdev->objname.size = 0;
+    /* Do not reset pdev->accumulating_a_global_object - it inherits. */
     pdev->sbstack_depth++;
     pdev->procsets = 0;
     pdev->font3 = 0;
     pdev->context = PDF_IN_STREAM;
+    pdev->accumulating_substream_resource = pres;
     pdf_reset_graphics(pdev);
     *ppres = pres;
     return 0;
@@ -524,33 +578,23 @@ pdf_enter_substream(gx_device_pdf *pdev, pdf_resource_type_t rtype,
 int
 pdf_exit_substream(gx_device_pdf *pdev) 
 {
-    int code = pdf_open_contents(pdev, PDF_IN_STREAM), code1;
+    int code, code1;
     int sbstack_ptr;
-    stream *s = pdev->strm;
-    pdf_procset_t procsets;
 
     if (pdev->sbstack_depth <= 0)
 	return_error(gs_error_unregistered); /* Must not happen. */
+    code = pdf_open_contents(pdev, PDF_IN_STREAM);
     sbstack_ptr = pdev->sbstack_depth - 1;
     while (pdev->vgstack_depth > pdev->vgstack_bottom) {
-	code1 = pdf_restore_viewer_state(pdev, s);
+	code1 = pdf_restore_viewer_state(pdev, pdev->strm);
 	if (code >= 0)
 	    code = code1;
     }
     if (pdev->clip_path != 0)
 	gx_path_free(pdev->clip_path, "pdf_end_charproc_accum");
-    {	/* We should call pdf_end_data here, but we don't want to put pdf_data_writer_t
-	   into pdf_substream_save stack to simplify garbager descriptors. 
-	   Use a lower level functions instead that. */
-	int status = s_close_filters(&s, cos_write_stream_from_pipeline(s));
-	cos_stream_t *pcs = cos_stream_from_pipeline(s);
-
-
-	if (status < 0 && code >=0)
-	     code = gs_note_error(gs_error_ioerror);
-	pcs->is_open = false;
-    }
-    sclose(s);
+    code1 = pdf_close_aside(pdev);
+    if (code1 < 0 && code >= 0)
+	code = code1;
     pdev->context = pdev->sbstack[sbstack_ptr].context;
     pdf_text_state_copy(pdev->text->text_state, pdev->sbstack[sbstack_ptr].text_state);
     pdev->clip_path = pdev->sbstack[sbstack_ptr].clip_path;
@@ -559,24 +603,208 @@ pdf_exit_substream(gx_device_pdf *pdev)
     pdev->vgstack_bottom = pdev->sbstack[sbstack_ptr].vgstack_bottom;
     pdev->strm = pdev->sbstack[sbstack_ptr].strm;
     pdev->sbstack[sbstack_ptr].strm = 0;
-    procsets = pdev->procsets;
     pdev->procsets = pdev->sbstack[sbstack_ptr].procsets;
     pdev->substream_Resources = pdev->sbstack[sbstack_ptr].substream_Resources;
     pdev->sbstack[sbstack_ptr].substream_Resources = 0;
     pdev->skip_colors = pdev->sbstack[sbstack_ptr].skip_colors;
     pdev->font3 = pdev->sbstack[sbstack_ptr].font3;
     pdev->sbstack[sbstack_ptr].font3 = 0;
+    pdev->accumulating_substream_resource = pdev->sbstack[sbstack_ptr].accumulating_substream_resource;
+    pdev->sbstack[sbstack_ptr].accumulating_substream_resource = 0;
+    pdev->charproc_just_accumulated = pdev->sbstack[sbstack_ptr].charproc_just_accumulated;
+    pdev->accumulating_a_global_object = pdev->sbstack[sbstack_ptr].accumulating_a_global_object;
+    pdev->pres_soft_mask_dict = pdev->sbstack[sbstack_ptr].pres_soft_mask_dict;
+    pdev->objname = pdev->sbstack[sbstack_ptr].objname;
     pdev->sbstack_depth = sbstack_ptr;
+    code1 = pdf_restore_viewer_state(pdev, NULL);
+    if (code1 < 0 && code >= 0)
+	code = code1;
     return code;
 }
 
+private bool 
+pdf_is_same_charproc1(gx_device_pdf * pdev, pdf_char_proc_t *pcp0, pdf_char_proc_t *pcp1)
+{
+    if (pcp0->char_code != pcp1->char_code)
+	return false; /* We need same encoding. */
+    if (pcp0->font->u.simple.Encoding[pcp0->char_code].glyph !=
+	pcp1->font->u.simple.Encoding[pcp1->char_code].glyph)
+	return false; /* We need same encoding. */
+    if (bytes_compare(pcp0->char_name.data, pcp0->char_name.size, 
+		      pcp1->char_name.data, pcp1->char_name.size))
+	return false; /* We need same encoding. */
+    if (pcp0->real_width.x != pcp1->real_width.x)
+	return false;
+    if (pcp0->real_width.y != pcp1->real_width.y)
+	return false;
+    if (pcp0->v.x != pcp1->v.x)
+	return false;
+    if (pcp0->v.y != pcp1->v.y)
+	return false;
+    if (pcp0->font->u.simple.s.type3.bitmap_font != pcp1->font->u.simple.s.type3.bitmap_font)
+	return false;
+    if (memcmp(&pcp0->font->u.simple.s.type3.FontMatrix, &pcp1->font->u.simple.s.type3.FontMatrix,
+		sizeof(pcp0->font->u.simple.s.type3.FontMatrix)))
+	return false;
+    return pdf_check_encoding_compatibility(pcp1->font, pdev->cgp->s, pdev->cgp->num_all_chars);
+}
+
+private int 
+pdf_is_same_charproc(gx_device_pdf * pdev, pdf_resource_t *pres0, pdf_resource_t *pres1)
+{
+    return pdf_is_same_charproc1(pdev, (pdf_char_proc_t *)pres0, (pdf_char_proc_t *)pres1);
+}
+
+private int 
+pdf_find_same_charproc(gx_device_pdf *pdev, 
+	    pdf_font_resource_t *pdfont, const pdf_char_glyph_pairs_t *cgp, 
+	    pdf_char_proc_t **ppcp)
+{
+    pdf_char_proc_t *pcp;
+    int code;
+
+    pdev->cgp = cgp;
+    for (pcp = pdfont->u.simple.s.type3.char_procs; pcp != NULL; pcp = pcp->char_next) {
+	if (*ppcp != pcp && pdf_is_same_charproc1(pdev, *ppcp, pcp)) {
+    	    cos_object_t *pco0 = pcp->object;
+    	    cos_object_t *pco1 = (*ppcp)->object;
+
+	    code = pco0->cos_procs->equal(pco0, pco1, pdev);
+	    if (code < 0)
+		return code;
+	    if (code) {
+		*ppcp = pcp;
+		pdev->cgp = NULL;
+		return 1;
+	    }
+	}
+    }
+    pcp = *ppcp;
+    code = pdf_find_same_resource(pdev, resourceCharProc, (pdf_resource_t **)ppcp, pdf_is_same_charproc);
+    pdev->cgp = NULL;
+    if (code <= 0)
+	return code;
+    /* fixme : do we need more checks here ? */
+    return 1;
+}
+
+private bool
+pdf_is_charproc_defined(gx_device_pdf *pdev, pdf_font_resource_t *pdfont, gs_char ch)
+{
+    pdf_char_proc_t *pcp;
+
+    for (pcp = pdfont->u.simple.s.type3.char_procs; pcp != NULL; pcp = pcp->char_next) {
+	if (pcp->char_code == ch) {
+	    return true;
+	}
+    }
+    return false;
+}
+
 /*
  * Complete charproc accumulation for a Type 3 font.
  */
 int
-pdf_end_charproc_accum(gx_device_pdf *pdev) 
+pdf_end_charproc_accum(gx_device_pdf *pdev, gs_font *font, const pdf_char_glyph_pairs_t *cgp) 
 {
-    return pdf_exit_substream(pdev);
+    int code;
+    pdf_resource_t *pres = (pdf_resource_t *)pdev->accumulating_substream_resource;
+    /* We could use pdfont->u.simple.s.type3.char_procs insted the thing above
+       unless the font is defined recursively.
+       But we don't want such assumption. */
+    pdf_char_proc_t *pcp = (pdf_char_proc_t *)pres;
+    pdf_font_resource_t *pdfont;
+    gs_char ch = pcp->char_code;
+    double *real_widths;
+    byte *glyph_usage;
+    int char_cache_size, width_cache_size;
+    gs_glyph glyph0;
+    bool checking_glyph_variation = false;
+    int i;
+
+    code = pdf_attached_font_resource(pdev, font, &pdfont, NULL, NULL, NULL, NULL);
+    if (code < 0)
+	return code;
+    if (pdfont != (pdf_font_resource_t *)pdev->font3)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (ch == GS_NO_CHAR)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (ch >= 256)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    code = pdf_exit_substream(pdev);
+    if (code < 0)
+	return code;
+    if (pdfont->used[ch >> 3] & (0x80 >> (ch & 7))) {
+	if (!(pdfont->u.simple.s.type3.cached[ch >> 3] & (0x80 >> (ch & 7)))) {
+	    checking_glyph_variation = true;
+	    code = pdf_find_same_charproc(pdev, pdfont, cgp, &pcp);
+	    if (code < 0)
+		return code;
+	    if (code != 0) {
+		code = pdf_cancel_resource(pdev, pres, resourceCharProc);
+		if (code < 0)
+		    return code;
+		pdf_forget_resource(pdev, pres, resourceCharProc);
+		if (pcp->font != pdfont) {
+		    code = pdf_attach_font_resource(pdev, font, pcp->font);
+		    if (code < 0)
+			return code;
+		}
+		pdev->charproc_just_accumulated = true;
+		return 0;
+	    }
+	    if (pdf_is_charproc_defined(pdev, pdfont, ch)) {
+		gs_font *base_font = font, *below;
+
+		while ((below = base_font->base) != base_font &&
+			base_font->procs.same_font(base_font, below, FONT_SAME_OUTLINES))
+		    base_font = below;
+		code = pdf_make_font3_resource(pdev, base_font, &pdfont);
+		if (code < 0)
+		    return code;
+		code = pdf_attach_font_resource(pdev, font, pdfont);
+		if (code < 0)
+		    return code;
+	    }
+	}
+    } 
+    pdf_reserve_object_id(pdev, pres, 0);
+    code = pdf_attached_font_resource(pdev, font, &pdfont,
+		&glyph_usage, &real_widths, &char_cache_size, &width_cache_size);
+    if (code < 0)
+	return code;
+    if (ch >= char_cache_size || ch >= width_cache_size)
+	return_error(gs_error_unregistered); /* Must not happen. */
+    if (checking_glyph_variation)
+	pdev->charproc_just_accumulated = true;
+    glyph0 = font->procs.encode_char(font, ch, GLYPH_SPACE_NAME);
+    pcp->char_next = pdfont->u.simple.s.type3.char_procs;
+    pdfont->u.simple.s.type3.char_procs = pcp;
+    pcp->font = pdfont;
+    pdfont->Widths[ch] = pcp->real_width.x;
+    real_widths[ch * 2    ] = pcp->real_width.x;
+    real_widths[ch * 2 + 1] = pcp->real_width.y;
+    glyph_usage[ch / 8] |= 0x80 >> (ch & 7);
+    pdfont->used[ch >> 3] |= 0x80 >> (ch & 7);
+    if (pdfont->u.simple.v != NULL && font->WMode) {
+	pdfont->u.simple.v[ch].x = pcp->v.x;
+	pdfont->u.simple.v[ch].y = pcp->v.x;
+    }
+    for (i = 0; i < 256; i++) {
+	gs_glyph glyph = font->procs.encode_char(font, i, 
+		    font->FontType == ft_user_defined ? GLYPH_SPACE_NOGEN
+						      : GLYPH_SPACE_NAME);
+
+	if (glyph == glyph0) {
+	    real_widths[i * 2    ] = real_widths[ch * 2    ];
+	    real_widths[i * 2 + 1] = real_widths[ch * 2 + 1];
+	    glyph_usage[i / 8] |= 0x80 >> (i & 7);
+	    pdfont->used[i >> 3] |= 0x80 >> (i & 7);
+	    pdfont->u.simple.v[i] = pdfont->u.simple.v[ch];
+	    pdfont->Widths[i] = pdfont->Widths[ch];
+	}
+    }
+    return 0;
 }
 
 /* Add procsets to substream Resources. */
@@ -610,6 +838,12 @@ pdf_add_resource(gx_device_pdf *pdev, cos_dict_t *pcd, const char *key, pdf_reso
 	int code;
 	char buf[10 + (sizeof(long) * 8 / 3 + 1)], buf1[sizeof(pres->rname) + 1];
 
+	if (pdev->ForOPDFRead && !pres->global && pdev->accumulating_a_global_object) {
+	    pres->global = true;
+	    code = cos_dict_put_c_key_bool((cos_dict_t *)pres->object, "/.Global", true);
+	    if (code < 0)
+		return code;
+	}
 	sprintf(buf, "%ld 0 R\n", pres->object->id);
 	if (v != NULL) {
 	    if (v->value_type != COS_VALUE_OBJECT && 
