@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpdts.c,v 1.24.2.1 2003/12/12 22:08:01 giles Exp $ */
+/* $Id: gdevpdts.c,v 1.28 2005/01/24 15:37:48 igor Exp $ */
 /* Text state management for pdfwrite */
 #include "math_.h"
 #include "memory_.h"
@@ -434,7 +434,7 @@ sync_text_state(gx_device_pdf *pdev)
 	pts->wmode =
 	    (pdfont->FontType == ft_composite ?
 	     pdfont->u.type0.WMode : 0);
-	code = pdf_used_charproc_fonts(pdev, pdfont);
+	code = pdf_used_charproc_resources(pdev, pdfont);
 	if (code < 0)
 	    return code;
     }
