@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevprn.h,v 1.13 2003/03/11 11:36:41 ghostgum Exp $ */
+/* $Id: gdevprn.h,v 1.15 2005/05/27 05:43:24 dan Exp $ */
 /* Common header file for memory-buffered printers */
 
 #ifndef gdevprn_INCLUDED
@@ -231,6 +231,7 @@ struct gdev_prn_space_params_s {
 		/* ------ Other device parameters ------ */\
 	bool OpenOutputFile;\
 	bool ReopenPerPage;\
+        bool page_uses_transparency; /* PDF 1.4 transparency is used on page */\
 	bool Duplex;\
 	  int Duplex_set;		/* -1 = not supported */\
 		/* ------ End of parameters ------ */\
@@ -381,6 +382,7 @@ extern const gx_device_procs prn_std_procs;
 	 { 0 },		/* fname */\
 	0/*false*/,	/* OpenOutputFile */\
 	0/*false*/,	/* ReopenPerPage */\
+	0/*false*/,	/* page_uses_transparency */\
 	0/*false*/, -1,	/* Duplex[_set] */\
 	0/*false*/, 0, 0, 0, /* file_is_new ... buf */\
 	0, 0, 0, 0, 0/*false*/, 0, 0, /* buffer_memory ... clist_dis'_mask */\
