@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevpsft.c,v 1.28.2.1 2003/12/12 22:08:02 giles Exp $ */
+/* $Id: gdevpsft.c,v 1.35 2004/12/08 21:35:13 stefan Exp $ */
 /* Write an embedded TrueType font */
 #include "memory_.h"
 #include <assert.h>
@@ -385,6 +385,7 @@ size_cmap(gs_font *font, uint first_code, int num_glyphs, gs_glyph max_glyph,
 {
     stream poss;
 
+    s_init(&poss, NULL);
     swrite_position_only(&poss);
     write_cmap(&poss, font, first_code, num_glyphs, max_glyph, options, 0);
     return stell(&poss);
@@ -788,6 +789,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	    return_error(gs_error_invalidfont);
 	glyph_index = glyph  & ~GS_GLYPH_TAG;
 	if_debug1('L', "[L]glyph_index %u\n", glyph_index);
+	glyph_data.memory = pfont->memory;
 	if ((code = pfont->data.get_outline(pfont, glyph_index, &glyph_data)) >= 0) {
 	    /* Since indexToLocFormat==0 assumes even glyph lengths,
 	       round it up here. If later we choose indexToLocFormat==1,
@@ -1017,6 +1019,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 	for (offset = 0; psf_enumerate_glyphs_next(penum, &glyph) != 1; ) {
 	    gs_glyph_data_t glyph_data;
 
+	    glyph_data.memory = pfont->memory;
 	    if ((code = pfont->data.get_outline(pfont,
 						glyph & ~GS_GLYPH_TAG,
 						&glyph_data)) >= 0
@@ -1050,6 +1053,7 @@ psf_write_truetype_data(stream *s, gs_font_type42 *pfont, int options,
 
 	    for (; glyph_prev <= glyph_index; ++glyph_prev)
 		put_loca(s, offset, indexToLocFormat);
+	    glyph_data.memory = pfont->memory;
 	    if ((code = pfont->data.get_outline(pfont, glyph_index,
 						&glyph_data)) >= 0
 		) {
