@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gdevvec.c,v 1.18.2.1 2003/12/12 22:08:02 giles Exp $ */
+/* $Id: gdevvec.c,v 1.27 2005/08/23 11:26:26 igor Exp $ */
 /* Utilities for "vector" devices */
 #include "math_.h"
 #include "memory_.h"
@@ -74,7 +74,7 @@ gdev_vector_dopath(gx_device_vector *vdev, const gx_path * ppath,
      * which requires (untransformed) device coordinates.
      */
     if (rtype != prt_none &&
-	!((type & gx_path_type_stroke) && rtype == prt_open) &&
+	(!(type & gx_path_type_stroke) || rtype == prt_closed) &&
 	(pmat == 0 || is_xxyy(pmat) || is_xyyx(pmat)) &&
 	(state.scale_mat.xx == 1.0 && state.scale_mat.yy == 1.0 &&
 	 is_xxyy(&state.scale_mat) &&
@@ -255,7 +255,7 @@ gdev_vector_reset(gx_device_vector * vdev)
     gx_hld_saved_color_init(&vdev->saved_fill_color);
     gx_hld_saved_color_init(&vdev->saved_stroke_color);
     vdev->clip_path_id =
-	vdev->no_clip_path_id = gs_next_ids(1);
+	vdev->no_clip_path_id = gs_next_ids(vdev->memory, 1);
 }
 
 /* Open the output file and stream. */
@@ -316,7 +316,8 @@ gdev_vector_open_file_options(gx_device_vector * vdev, uint strmbuf_size,
      */
     vdev->strm->procs.close = vdev->strm->procs.flush;
     if (vdev->bbox_device) {
-	gx_device_bbox_init(vdev->bbox_device, NULL);
+	gx_device_bbox_init(vdev->bbox_device, NULL, vdev->v_memory);
+        rc_increment(vdev->bbox_device);
 	gx_device_set_resolution((gx_device *) vdev->bbox_device,
 				 vdev->HWResolution[0],
 				 vdev->HWResolution[1]);
@@ -621,28 +622,36 @@ gdev_vector_dopath_segment(gdev_vector_dopath_state_t *state, int pe_op,
 
     switch (pe_op) {
 	case gs_pe_moveto:
-	    gs_point_transform_inverse(fixed2float(vs[0].x),
+	    code = gs_point_transform_inverse(fixed2float(vs[0].x),
 				       fixed2float(vs[0].y), pmat, &vp[0]);
+	    if (code < 0)
+		return code;
 	    if (state->first)
 		state->start = vp[0], state->first = false;
 	    code = vdev_proc(vdev, moveto)
-		(vdev, state->prev.x, state->prev.y, vp[0].x, vp[0].y,
+		(vdev, 0/*unused*/, 0/*unused*/, vp[0].x, vp[0].y,
 		 state->type);
 	    state->prev = vp[0];
 	    break;
 	case gs_pe_lineto:
-	    gs_point_transform_inverse(fixed2float(vs[0].x),
+	    code = gs_point_transform_inverse(fixed2float(vs[0].x),
 				       fixed2float(vs[0].y), pmat, &vp[0]);
+	    if (code < 0)
+		return code;
 	    code = vdev_proc(vdev, lineto)
 		(vdev, state->prev.x, state->prev.y, vp[0].x, vp[0].y,
 		 state->type);
 	    state->prev = vp[0];
 	    break;
 	case gs_pe_curveto:
-	    gs_point_transform_inverse(fixed2float(vs[0].x),
+	    code = gs_point_transform_inverse(fixed2float(vs[0].x),
 				       fixed2float(vs[0].y), pmat, &vp[0]);
-	    gs_point_transform_inverse(fixed2float(vs[1].x),
+	    if (code < 0)
+		return code;
+	    code = gs_point_transform_inverse(fixed2float(vs[1].x),
 				       fixed2float(vs[1].y), pmat, &vp[1]);
+	    if (code < 0)
+		return code;
 	    gs_point_transform_inverse(fixed2float(vs[2].x),
 				       fixed2float(vs[2].y), pmat, &vp[2]);
 	    code = vdev_proc(vdev, curveto)
@@ -805,7 +814,8 @@ gdev_vector_close_file(gx_device_vector * vdev)
     vdev->file = 0;
     err = ferror(f);
     /* We prevented sclose from closing the file. */
-    if (fclose(f) != 0 || err != 0)
+    if (gx_device_close_output_file((gx_device *)vdev, vdev->fname, f) != 0 
+	|| err != 0)
 	return_error(gs_error_ioerror);
     return 0;
 }
