@@ -1,4 +1,4 @@
-/* Copyright (C) 1992, 2000 Aladdin Enterprises.  All rights reserved.
+/* Copyright (C) 1992, 2000-2003 artofcode LLC.  All rights reserved.
   
   This software is provided AS-IS with no warranty, either express or
   implied.
@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gp_mswin.c,v 1.16.2.1 2003/12/12 22:08:02 giles Exp $ */
+/* $Id: gp_mswin.c,v 1.25 2005/03/04 21:58:55 ghostgum Exp $ */
 /*
  * Microsoft Windows platform support for Ghostscript.
  *
@@ -93,6 +93,23 @@ gp_do_exit(int exit_status)
     exit(exit_status);
 }
 
+/* ------ Persistent data cache ------*/
+
+/* insert a buffer under a (type, key) pair */
+int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
+{
+    /* not yet implemented */
+    return 0;
+}
+
+/* look up a (type, key) in the cache */
+int gp_cache_query(int type, byte* key, int keylen, void **buffer,
+    gp_cache_alloc alloc, void *userdata)
+{
+    /* not yet implemented */
+    return -1;
+}
+
 /* ------ Printer accessing ------ */
 
 /* Forward references */
@@ -132,7 +149,7 @@ gp_close_printer(FILE * pfile, const char *fname)
 
 
 /* Dialog box to select printer port */
-BOOL CALLBACK
+DLGRETURN CALLBACK
 SpoolDlgProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
 {
     LPSTR entry;
@@ -547,7 +564,7 @@ gp_printfile_gs16spl(const char *filename, const char *port)
     hwndspl = FindWindow(NULL, "GS Win32s/Win16 spooler");
 
     while (IsWindow(hwndspl)) {
-	gp_check_interrupts();
+	gp_check_interrupts(NULL);
     }
 
     return 0;
