@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gp_vms.c,v 1.29.2.1 2003/12/12 22:08:02 giles Exp $ */
+/* $Id: gp_vms.c,v 1.38 2004/04/08 16:18:25 giles Exp $ */
 /* VAX/VMS specific routines for Ghostscript */
 
 #include "string_.h"
@@ -152,6 +152,24 @@ gp_get_usertime(long *pdt)
     gp_get_realtime(pdt);	/* Use an approximation for now.  */
 }
 
+
+/* ------ Persistent data cache ------*/
+
+/* insert a buffer under a (type, key) pair */
+int gp_cache_insert(int type, byte *key, int keylen, void *buffer, int buflen)
+{
+    /* not yet implemented */
+    return 0;
+}
+
+/* look up a (type, key) in the cache */
+int gp_cache_query(int type, byte* key, int keylen, void **buffer,
+    gp_cache_alloc alloc, void *userdata)
+{
+    /* not yet implemented */
+    return -1;
+}
+
 /* ------ Screen management ------ */
 
 /* Get the environment variable that specifies the display to use. */
