@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsfont.c,v 1.20.2.2 2004/01/29 09:47:43 igor Exp $ */
+/* $Id: gsfont.c,v 1.37 2005/07/27 11:24:38 igor Exp $ */
 /* Font operators for Ghostscript library */
 #include "gx.h"
 #include "memory_.h"
@@ -109,7 +109,7 @@ ENUM_PTRS_WITH(font_dir_enum_ptrs, gs_font_dir *dir)
 
 	if (cc != 0 && !--count) {
 	    (*dir->ccache.mark_glyph)
-		(cc->code, dir->ccache.mark_glyph_data);
+		(mem, cc->code, dir->ccache.mark_glyph_data);
 	    /****** HACK: break const.  We'll fix this someday. ******/
 	    ((gs_font_dir *)dir)->enum_index = cci;
 	    ((gs_font_dir *)dir)->enum_offset = offset;
@@ -209,7 +209,7 @@ RELOC_PTRS_END
 
 /* Allocate a font directory */
 private bool
-cc_no_mark_glyph(gs_glyph glyph, void *ignore_data)
+cc_no_mark_glyph(const gs_memory_t *mem, gs_glyph glyph, void *ignore_data)
 {
     return false;
 }
@@ -260,16 +260,12 @@ gs_font_dir_alloc2_limits(gs_memory_t * struct_mem, gs_memory_t * bits_mem,
     pdir->scaled_fonts = 0;
     pdir->ssize = 0;
     pdir->smax = smax;
-    pdir->align_to_pixels = true;
+    pdir->align_to_pixels = false;
     pdir->glyph_to_unicode_table = NULL;
-#if NEW_TT_INTERPRETER
-    pdir->grid_fit_tt = false;
+    pdir->grid_fit_tt = 2;
     pdir->memory = struct_mem;
     pdir->tti = 0;
-#if TT_GRID_FITTING
     pdir->san = 0;
-#endif
-#endif
     pdir->global_glyph_code = NULL;
     return pdir;
 }
@@ -284,12 +280,26 @@ gs_font_alloc(gs_memory_t *mem, gs_memory_type_ptr_t pstype,
 
     if (pfont == 0)
 	return 0;
+#if 1 /* Clear entire structure to avoid unitialized pointers 
+         when the initialization exits prematurely by error. */
+    memset(pfont, 0, pstype->ssize);
+    pfont->memory = mem;
+    pfont->dir = dir;
+    gs_font_notify_init(pfont);
+    pfont->id = gs_next_ids(mem, 1);
+    pfont->base = pfont;
+    pfont->ExactSize = pfont->InBetweenSize = pfont->TransformedChar =
+	fbit_use_outlines;
+    pfont->procs = *procs;
+#else
+    /* For clarity we leave old initializations here
+       to know which fields needs to be initialized. */
     pfont->next = pfont->prev = 0;
     pfont->memory = mem;
     pfont->dir = dir;
     pfont->is_resource = false;
     gs_font_notify_init(pfont);
-    pfont->id = gs_next_ids(1);
+    pfont->id = gs_next_ids(mem, 1);
     pfont->base = pfont;
     pfont->client_data = 0;
     /* not FontMatrix, FontType */
@@ -301,6 +311,7 @@ gs_font_alloc(gs_memory_t *mem, gs_memory_type_ptr_t pstype,
     pfont->StrokeWidth = 0;
     pfont->procs = *procs;
     memset(&pfont->orig_FontMatrix, 0, sizeof(pfont->orig_FontMatrix));
+#endif
     /* not key_name, font_name */
     return pfont;
 }
@@ -395,6 +406,26 @@ gs_definefont(gs_font_dir * pdir, gs_font * pfont)
     return 0;
 }
 
+/* Find a sililar registered font of same type. */
+int
+gs_font_find_similar(const gs_font_dir * pdir, const gs_font **ppfont, 
+		       int (*similar)(const gs_font *, const gs_font *))
+{
+    const gs_font *pfont0 = *ppfont;
+    const gs_font *pfont1 = pdir->orig_fonts;
+
+    for (; pfont1 != NULL; pfont1 = pfont1->next) {
+	if (pfont1 != pfont0 && pfont1->FontType == pfont0->FontType) {
+	    int code = similar(pfont0, pfont1);
+	    if (code != 0) {
+		*ppfont = pfont1;
+		return code;
+	    }
+	}
+    }
+    return 0;
+}
+
 /* scalefont */
 int
 gs_scalefont(gs_font_dir * pdir, const gs_font * pfont, floatp scale,
@@ -586,14 +617,12 @@ gs_setaligntopixels(gs_font_dir * pdir, uint v)
     pdir->align_to_pixels = v;
     return 0;
 }
-#if NEW_TT_INTERPRETER
 int
 gs_setgridfittt(gs_font_dir * pdir, uint v)
 {
     pdir->grid_fit_tt = v;
     return 0;
 }
-#endif
 
 /* currentcacheparams */
 uint
@@ -616,13 +645,11 @@ gs_currentaligntopixels(const gs_font_dir * pdir)
 {
     return pdir->align_to_pixels;
 }
-#if NEW_TT_INTERPRETER
 uint
 gs_currentgridfittt(const gs_font_dir * pdir)
 {
     return pdir->grid_fit_tt;
 }
-#endif
 
 /* Purge a font from all font- and character-related tables. */
 /* This is only used by restore (and, someday, the GC). */
@@ -665,12 +692,13 @@ gs_purge_font(gs_font * pfont)
 
 /* Locate a gs_font by gs_id. */
 gs_font *
-gs_find_font_by_id(gs_font_dir *pdir, gs_id id)
+gs_find_font_by_id(gs_font_dir *pdir, gs_id id, gs_matrix *FontMatrix)
  {
     gs_font *pfont = pdir->orig_fonts;
 
     for(; pfont != NULL; pfont = pfont->next)
-	if(pfont->id == id)
+	if(pfont->id == id &&
+	    !memcmp(&pfont->FontMatrix, FontMatrix, sizeof(pfont->FontMatrix)))
 	    return pfont;
     return NULL;
  }
@@ -893,12 +921,17 @@ gs_default_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
     int returned = 0;
     int code;
     int wmode = ((members & GLYPH_INFO_WIDTH1) != 0);
+    double sbw[4] = {0, 0, 0, 0};
+    /* Currently glyph_outline retrieves sbw only with type 1,2,9 fonts. */
+    bool charstrings_font = (font->FontType == ft_encrypted || 
+			     font->FontType == ft_encrypted2 || 
+			     font->FontType == ft_CID_encrypted);
 
     gx_path_init_bbox_accumulator(&path);
     code = gx_path_add_point(&path, fixed_0, fixed_0);
     if (code < 0)
 	goto out;
-    code = font->procs.glyph_outline(font, wmode, glyph, pmat, &path);
+    code = font->procs.glyph_outline(font, wmode, glyph, pmat, &path, sbw);
     if (code < 0)
 	goto out;
     if (members & GLYPH_INFO_WIDTHS) {
@@ -928,6 +961,28 @@ gs_default_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 	info->bbox.q.y = fixed2float(bbox.q.y);
 	returned |= GLYPH_INFO_BBOX;
     }
+    if (members & (GLYPH_INFO_WIDTH0 << wmode) && charstrings_font) {
+	if (pmat == 0) {
+	    info->width[wmode].x = sbw[2];
+	    info->width[wmode].y = sbw[3];
+	} else {
+	    code = gs_distance_transform(sbw[2], sbw[3], pmat, &info->width[wmode]);
+	    if (code < 0)
+		return code;
+	}
+	returned |= GLYPH_INFO_WIDTH0 << wmode;
+    }
+    if (members & (GLYPH_INFO_VVECTOR0 << wmode) && charstrings_font) {
+	if (pmat == 0) {
+	    info->v.x = sbw[0];
+	    info->v.y = sbw[1];
+	} else {
+	    gs_distance_transform(sbw[0], sbw[1], pmat, &info->v);
+	    if (code < 0)
+		return code;
+	}
+	returned |= GLYPH_INFO_VVECTOR0 << wmode;
+    }
     if (members & GLYPH_INFO_NUM_PIECES) {
 	info->num_pieces = 0;
 	returned |= GLYPH_INFO_NUM_PIECES;
@@ -941,7 +996,7 @@ gs_default_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 /* Dummy glyph outline procedure */
 int
 gs_no_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,
-		    gx_path *ppath)
+		    gx_path *ppath, double sbw[4])
 {
     return_error(gs_error_undefined);
 }
