@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gsht.c,v 1.16.2.2 2004/01/05 23:32:04 giles Exp $ */
+/*$Id: gsht.c,v 1.23 2005/03/14 18:08:36 dan Exp $ */
 /* setscreen operator for Ghostscript library */
 #include "memory_.h"
 #include "string_.h"
@@ -629,7 +629,11 @@ gx_device_halftone_release(gx_device_halftone * pdht, gs_memory_t * mem)
  *
  * Two other checks are also made.  If the name is "Default" then a value
  * of GX_DEVICE_COLOR_MAX_COMPONENTS is returned.  This is done to
- * simplify the handling of default halftones.
+ * simplify the handling of default halftones.  Note:  The device also
+ * uses GX_DEVICE_COLOR_MAX_COMPONENTS to indicate colorants which are
+ * known but not being used due to the SeparationOrder parameter.  In this
+ * case we return -1 since the colorant is not currently being used by the
+ * device.
  *
  * If the halftone type is colorscreen or multiple colorscreen, then we
  * also check for Red/Cyan, Green/Magenta, Blue/Yellow, and Gray/Black
@@ -639,16 +643,16 @@ gx_device_halftone_release(gx_device_halftone * pdht, gs_memory_t * mem)
  * A negative value is returned if the color name is not found.
  */
 int
-gs_color_name_component_number(const gx_device * dev, const char * pname,
+gs_color_name_component_number(gx_device * dev, const char * pname,
 				int name_size, int halftonetype)
 {
     int num_colorant;
 
 #define check_colorant_name(dev, name) \
-    ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), 0))
+    ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), NO_COMP_NAME_TYPE))
 
 #define check_colorant_name_length(dev, name, length) \
-    ((*dev_proc(dev, get_color_comp_index)) (dev, name, length, 0))
+    ((*dev_proc(dev, get_color_comp_index)) (dev, name, length, NO_COMP_NAME_TYPE))
 
 #define check_name(str, pname, length) \
     ((strlen(str) == length) && (strncmp(pname, str, length) == 0))
@@ -657,8 +661,18 @@ gs_color_name_component_number(const gx_device * dev, const char * pname,
      * Check if this is a device colorant.
      */
     num_colorant = check_colorant_name_length(dev, pname, name_size);
-    if (num_colorant >= 0)
+    if (num_colorant >= 0) {
+	/*
+	 * The device will return GX_DEVICE_COLOR_MAX_COMPONENTS if the
+	 * colorant is logically present in the device but not being used
+	 * because a SeparationOrder parameter is specified.  Since we are
+	 * using this value to indicate 'Default', we use -1 to indicate
+	 * that the colorant is not really being used.
+	 */
+	if (num_colorant == GX_DEVICE_COLOR_MAX_COMPONENTS)
+	    num_colorant = -1;
 	return num_colorant;
+    }
 
     /*
      * Check if this is the default component
@@ -698,7 +712,7 @@ int
 gs_cname_to_colorant_number(gs_state * pgs, byte * pname, uint name_size,
 		int halftonetype)
 {
-    const gx_device * dev = pgs->device;
+    gx_device * dev = pgs->device;
 
     return gs_color_name_component_number(dev, (char *)pname, name_size,
 		    halftonetype);
@@ -752,7 +766,13 @@ gs_cname_to_colorant_number(gs_state * pgs, byte * pname, uint name_size,
  *  num_comp    For the operand halftone, this is the number of halftone
  *              components included in the specification. For the device
  *              halftone in the imager state, this is always the same as
- *              the number of color model components.
+ *              the number of color model components (see num_dev_comp).
+ *
+ *  num_dev_comp The number of components in the device process color model
+ *		when the operand halftone was created.  With some compositor
+ *		devices (for example PDF 1.4) we can have differences in the
+ *		process color model of the compositor versus the output device.
+ *		These compositor devices do not halftone.
  *
  *  components  For the operand halftone, this field is non-null only if
  *              multiple halftones are provided. In that case, the size
@@ -937,7 +957,7 @@ gx_imager_dev_ht_install(
     const gx_device *       dev )
 {
     gx_device_halftone      dht;
-    int                     num_comps = dev->color_info.num_components;
+    int                     num_comps = pdht->num_dev_comp;
     int                     i, code = 0;
     bool                    used_default = false;
     int                     lcm_width = 1, lcm_height = 1;
@@ -948,7 +968,7 @@ gx_imager_dev_ht_install(
     /* construct the new device halftone structure */
     memset(&dht.order, 0, sizeof(dht.order));
     /* the rc field is filled in later */
-    dht.id = gs_next_ids(1);
+    dht.id = gs_next_ids(pis->memory, 1);
     dht.type = type;
     dht.components =  gs_alloc_struct_array(
                           pis->memory,
@@ -958,7 +978,7 @@ gx_imager_dev_ht_install(
                           "gx_imager_dev_ht_install(components)" );
     if (dht.components == NULL)
 	return_error(gs_error_VMerror);
-    dht.num_comp = num_comps;
+    dht.num_comp = dht.num_dev_comp = num_comps;
     /* lcm_width, lcm_height are filled in later */
 
     /* initialize the components array */
@@ -1090,7 +1110,7 @@ gx_imager_dev_ht_install(
                     code = gs_error_VMerror;
                 else {
                     porder->cache = pcache;
-                    gx_ht_init_cache(pcache, porder);
+                    gx_ht_init_cache(pis->memory, pcache, porder);
                 }
             }
         }
@@ -1209,6 +1229,7 @@ gx_ht_install(gs_state * pgs, const gs_halftone * pht,
     gs_halftone *new_ht;
     int code;
 
+    pdht->num_dev_comp = pgs->device->color_info.num_components;
     if (old_ht != 0 && old_ht->rc.memory == mem &&
 	old_ht->rc.ref_count == 1
 	)
@@ -1250,7 +1271,7 @@ gx_ht_install(gs_state * pgs, const gs_halftone * pht,
  * A value of -1 indicates that the name is not valid.
  */
 #define check_colorant_name(name, dev) \
-   ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), 0))
+   ((*dev_proc(dev, get_color_comp_index)) (dev, name, strlen(name), NO_NAME_TYPE))
 
 /* Reestablish the effective transfer functions, taking into account */
 /* any overrides from halftone dictionaries. */
