@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gslparam.h,v 1.4 2002/02/21 22:24:52 giles Exp $ */
+/* $Id: gslparam.h,v 1.5 2004/07/29 17:46:38 igor Exp $ */
 /* Line parameter definitions */
 
 #ifndef gslparam_INCLUDED
@@ -25,7 +25,8 @@ typedef enum {
     gs_cap_butt = 0,
     gs_cap_round = 1,
     gs_cap_square = 2,
-    gs_cap_triangle = 3		/* not supported by PostScript */
+    gs_cap_triangle = 3,		/* not supported by PostScript */
+    gs_cap_unknown = 4
 } gs_line_cap;
 
 #define gs_line_cap_max 3
@@ -36,7 +37,8 @@ typedef enum {
     gs_join_round = 1,
     gs_join_bevel = 2,
     gs_join_none = 3,		/* not supported by PostScript */
-    gs_join_triangle = 4	/* not supported by PostScript */
+    gs_join_triangle = 4,	/* not supported by PostScript */
+    gs_join_unknown = 5
 } gs_line_join;
 
 #define gs_line_join_max 4
