@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsmisc.c,v 1.15 2002/09/23 01:21:02 dan Exp $ */
+/* $Id: gsmisc.c,v 1.22 2004/12/08 21:35:13 stefan Exp $ */
 /* Miscellaneous utilities for Ghostscript library */
 
 /*
@@ -49,16 +49,12 @@ orig_sqrt(double x)
 #include "gxfarith.h"
 #include "gxfixed.h"
 
-/* Define private replacements for stdin, stdout, and stderr. */
-FILE *gs_stdio[3];
-
-
 /* ------ Redirected stdout and stderr  ------ */
 
 #include <stdarg.h>
 #define PRINTF_BUF_LENGTH 1024
 
-int outprintf(const char *fmt, ...)
+int outprintf(const gs_memory_t *mem, const char *fmt, ...)
 {
     int count;
     char buf[PRINTF_BUF_LENGTH];
@@ -67,12 +63,12 @@ int outprintf(const char *fmt, ...)
     va_start(args, fmt);
 
     count = vsprintf(buf, fmt, args);
-    outwrite(buf, count);
+    outwrite(mem, buf, count);
     if (count >= PRINTF_BUF_LENGTH) {
 	count = sprintf(buf, 
 	    "PANIC: printf exceeded %d bytes.  Stack has been corrupted.\n", 
 	    PRINTF_BUF_LENGTH);
-	outwrite(buf, count);
+	outwrite(mem, buf, count);
     }
     va_end(args);
     return count;
@@ -158,14 +154,14 @@ dprintf_file_only(const char *file)
 }
 #endif
 void
-printf_program_ident(const char *program_name, long revision_number)
+printf_program_ident(const gs_memory_t *mem, const char *program_name, long revision_number)
 {
     if (program_name)
-	outprintf((revision_number ? "%s " : "%s"), program_name);
+        outprintf(mem, (revision_number ? "%s " : "%s"), program_name);
     if (revision_number) {
 	int fpart = revision_number % 100;
 
-	outprintf("%d.%02d", (int)(revision_number / 100), fpart);
+	outprintf(mem, "%d.%02d", (int)(revision_number / 100), fpart);
     }
 }
 void
@@ -214,12 +210,12 @@ gs_log_error(int err, const char *file, int line)
 
 /* Check for interrupts before a return. */
 int
-gs_return_check_interrupt(int code)
+gs_return_check_interrupt(const gs_memory_t *mem, int code)
 {
     if (code < 0)
 	return code;
     {
-	int icode = gp_check_interrupts();
+	int icode = gp_check_interrupts(mem);
 
 	return (icode == 0 ? code :
 		gs_note_error((icode > 0 ? gs_error_interrupt : icode)));
@@ -1141,10 +1137,16 @@ gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 static const int isincos[5] =
 {0, 1, 0, -1, 0};
 
+/* GCC with -ffast-math compiles ang/90. as ang*(1/90.), losing precission.
+ * This doesn't happen when the numeral is replaced with a non-const variable.
+ * So we define the variable to work around the GCC problem. 
+ */
+static double const_90_degrees = 90.;
+
 double
 gs_sin_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/*
@@ -1159,7 +1161,7 @@ gs_sin_degrees(double ang)
 double
 gs_cos_degrees(double ang)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
@@ -1171,7 +1173,7 @@ gs_cos_degrees(double ang)
 void
 gs_sincos_degrees(double ang, gs_sincos_t * psincos)
 {
-    double quot = ang / 90;
+    double quot = ang / const_90_degrees;
 
     if (floor(quot) == quot) {
 	/* See above re the following line. */
