@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gsparam.h,v 1.11 2002/06/16 08:45:42 lpd Exp $ */
+/* $Id: gsparam.h,v 1.13 2005/09/05 13:58:55 leonardo Exp $ */
 /* Client interface to parameter dictionaries */
 
 #ifndef gsparam_INCLUDED
@@ -73,7 +73,6 @@ typedef enum {
 
 #define _param_array_struct(sname,etype)\
   struct sname { const etype *data; uint size; bool persistent; }
-typedef _param_array_struct(gs_param_string_s, byte) gs_param_string;
 typedef _param_array_struct(gs_param_int_array_s, int) gs_param_int_array;
 typedef _param_array_struct(gs_param_float_array_s, float) gs_param_float_array;
 typedef _param_array_struct(gs_param_string_array_s, gs_param_string) gs_param_string_array;
@@ -532,6 +531,12 @@ typedef struct gs_c_param_list_s {
   gs_private_st_ptrs2(st_c_param_list, gs_c_param_list, "c_param_list",\
     c_param_list_enum_ptrs, c_param_list_reloc_ptrs, head, target)
 
+/* Define a GC descriptor for gs_param_string. */
+/* This structure descriptor is only for non persistent gs_param_strings. */
+#define private_st_gs_param_string()	/* in gdevdevn.c */\
+  gs_private_st_composite(st_gs_param_string, gs_param_string, "gs_param_string",\
+			param_string_enum_ptrs, param_string_reloc_ptrs)
+
 /* Set the target of a C parameter list. */
 void gs_c_param_list_set_target(gs_c_param_list *, gs_param_list *);
 
