@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gstext.c,v 1.11.2.2 2004/01/29 13:17:10 igor Exp $ */
+/*$Id: gstext.c,v 1.19 2004/12/22 18:52:23 igor Exp $ */
 /* Driver text interface support */
 #include "memory_.h"
 #include "gstypes.h"
@@ -80,16 +80,13 @@ RELOC_PTRS_END
 
 private ENUM_PTRS_WITH(text_enum_enum_ptrs, gs_text_enum_t *eptr)
 {
-#if NEW_TT_INTERPRETER
-    if (index == 8)
+    if (index == 8) {
 	if (eptr->pair != 0)
 	    ENUM_RETURN(eptr->pair - eptr->pair->index);
 	else
 	    ENUM_RETURN(0);
+    }
     index -= 9;
-#else
-    index -= 8;
-#endif
     if (index <= eptr->fstack.depth)
 	ENUM_RETURN(eptr->fstack.items[index].font);
     index -= eptr->fstack.depth + 1;
@@ -110,11 +107,9 @@ private RELOC_PTRS_WITH(text_enum_reloc_ptrs, gs_text_enum_t *eptr)
     eptr->imaging_dev = gx_device_reloc_ptr(eptr->imaging_dev, gcst);
     RELOC_PTR3(gs_text_enum_t, pis, orig_font, path);
     RELOC_PTR3(gs_text_enum_t, pdcolor, pcpath, current_font);
-#if NEW_TT_INTERPRETER
     if (eptr->pair != NULL)
 	eptr->pair = (cached_fm_pair *)RELOC_OBJ(eptr->pair - eptr->pair->index) +
 			     eptr->pair->index;
-#endif
     for (i = 0; i <= eptr->fstack.depth; i++)
 	RELOC_PTR(gs_text_enum_t, fstack.items[i].font);
 }
@@ -135,12 +130,15 @@ gx_device_text_begin(gx_device * dev, gs_imager_state * pis,
 	gx_path *tpath =
 	    ((text->operation & TEXT_DO_NONE) &&
 	     !(text->operation & TEXT_RETURN_WIDTH) ? 0 : path);
-	const gx_device_color *tcolor =
-	    (text->operation & TEXT_DO_DRAW ? pdcolor : 0);
 	const gx_clip_path *tcpath =
 	    (text->operation & TEXT_DO_DRAW ? pcpath : 0);
+
+	/* A high level device need to know an initial device color 
+	   for accumulates a charstring of a Type 3 font.
+	   Since the accumulation may happen while stringwidth. 
+	   we pass the device color unconditionally. */
 	return dev_proc(dev, text_begin)
-	    (dev, pis, text, font, tpath, tcolor, tcpath, mem, ppte);
+	    (dev, pis, text, font, tpath, pdcolor, tcpath, mem, ppte);
     }
 }
 
@@ -157,6 +155,7 @@ gs_text_enum_init_dynamic(gs_text_enum_t *pte, gs_font *font)
     pte->FontBBox_as_Metrics2.x = pte->FontBBox_as_Metrics2.y = 0;
     pte->pair = 0;
     pte->device_disabled_grid_fitting = 0;
+    pte->outer_CID = GS_NO_GLYPH;
     return font->procs.init_fstack(pte, font);
 }
 int
@@ -206,6 +205,7 @@ gs_text_enum_copy_dynamic(gs_text_enum_t *pto, const gs_text_enum_t *pfrom,
     pto->FontBBox_as_Metrics2 = pfrom->FontBBox_as_Metrics2;
     pto->pair = pfrom->pair;
     pto->device_disabled_grid_fitting = pfrom->device_disabled_grid_fitting;
+    pto->outer_CID = pfrom->outer_CID;
     if (depth >= 0)
 	memcpy(pto->fstack.items, pfrom->fstack.items,
 	       (depth + 1) * sizeof(pto->fstack.items[0]));
@@ -221,17 +221,22 @@ gs_text_begin(gs_state * pgs, const gs_text_params_t * text,
 	      gs_memory_t * mem, gs_text_enum_t ** ppte)
 {
     gx_clip_path *pcpath = 0;
+    int code;
 
     if (text->operation & TEXT_DO_DRAW) {
-	int code = gx_effective_clip_path(pgs, &pcpath);
-
-	if (code < 0)
-	    return code;
-	gx_set_dev_color(pgs);
-	code = gs_state_color_load(pgs);
+	code = gx_effective_clip_path(pgs, &pcpath);
 	if (code < 0)
 	    return code;
     }
+    /* We must load device color even with no TEXT_DO_DRAW,
+       because a high level device accumulates a charstring 
+       of a Type 3 font while stringwidth. 
+       Unfortunately we can't effectively know a leaf font type here,
+       so we load the color unconditionally . */
+    gx_set_dev_color(pgs);
+    code = gs_state_color_load(pgs);
+    if (code < 0)
+	return code;
     return gx_device_text_begin(pgs->device, (gs_imager_state *) pgs,
 				text, pgs->font, pgs->path, pgs->dev_color,
 				pcpath, mem, ppte);
@@ -624,7 +629,10 @@ gs_default_next_char_glyph(gs_text_enum_t *pte, gs_char *pchr, gs_glyph *pglyph)
     if (pte->text.operation & (TEXT_FROM_STRING | TEXT_FROM_BYTES)) {
 	/* ordinary string */
 	*pchr = pte->text.data.bytes[pte->index];
-	*pglyph = gs_no_glyph;
+	if (pte->outer_CID != GS_NO_GLYPH)
+	    *pglyph = pte->outer_CID;
+	else
+	    *pglyph = gs_no_glyph;
     } else if (pte->text.operation & TEXT_FROM_SINGLE_GLYPH) {
 	/* glyphshow or glyphpath */
 	*pchr = gs_no_char;
