@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gstype2.c,v 1.25.2.1 2003/12/12 22:08:03 giles Exp $ */
+/* $Id: gstype2.c,v 1.36 2004/10/28 08:39:21 igor Exp $ */
 /* Adobe Type 2 charstring interpreter */
 #include "math_.h"
 #include "memory_.h"
@@ -57,8 +57,8 @@ private int
 type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state_t * ipsp,
 	  bool explicit_width)
 {
-    fixed wx;
     t1_hinter *h = &pcis->h;
+    fixed sbx = fixed_0, sby = fixed_0, wx, wy = fixed_0;
     int code;
 
     if (explicit_width) {
@@ -67,7 +67,17 @@ type2_sbw(gs_type1_state * pcis, cs_ptr csp, cs_ptr cstack, ip_state_t * ipsp,
 	--csp;
     } else
 	wx = pcis->pfont->data.defaultWidthX;
-    code = t1_hinter__sbw(h, fixed_0, fixed_0, wx, fixed_0);
+    if (pcis->seac_accent < 0) {
+	if (pcis->sb_set) {
+	    sbx = pcis->lsb.x;
+	    sby = pcis->lsb.y;
+	}
+	if (pcis->width_set) {
+	    wx = pcis->width.x;
+	    wy = pcis->width.y;
+	}
+    }
+    code = t1_hinter__sbw(h, sbx, sby, wx, wy);
     if (code < 0)
 	return code;
     gs_type1_sbw(pcis, fixed_0, fixed_0, wx, fixed_0);
@@ -307,7 +317,7 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
 		 */
 		if (csp >= cstack + 3) {
 		    check_first_operator(csp > cstack + 3);
-		    code = gs_type1_seac(pcis, cstack, pcis->lsb.x, ipsp);
+		    code = gs_type1_seac(pcis, cstack, 0, ipsp);
 		    if (code < 0)
 			return code;
 		    clear;
@@ -326,6 +336,9 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
 		    code = t1_hinter__endglyph(h);
 		    if (code < 0)
 			return code;
+		    code = gx_setcurrentpoint_from_path(pcis->pis, pcis->path);
+		    if (code < 0)
+			return code;
 		} else
 		    t1_hinter__setcurrentpoint(h, pcis->save_adxy.x, pcis->save_adxy.y);
 		code = gs_type1_endchar(pcis);
@@ -345,7 +358,17 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
 		return code;
 	    case cx_rmoveto:
 		/* See vmoveto above re closing the subpath. */
-		check_first_operator(csp > cstack + 1);
+		check_first_operator(!((csp - cstack) & 1));
+		if (csp > cstack + 1) {
+		  /* Some Type 2 charstrings omit the vstemhm operator before rmoveto, 
+		     even though this is only allowed before hintmask and cntrmask.
+		     Thanks to Felix Pahl.
+		   */
+		  type2_vstem(pcis, csp - 2, cstack);
+		  cstack [0] = csp [-1];
+		  cstack [1] = csp [ 0];
+		  csp = cstack + 1;
+		}
                 code = t1_hinter__rmoveto(h, csp[-1], *csp);
 		goto move;
 	    case cx_hmoveto:
@@ -416,9 +439,6 @@ gs_type2_interpret(gs_type1_state * pcis, const gs_glyph_data_t *pgd,
 		 * A hintmask at the beginning of the CharString is
 		 * equivalent to vstemhm + hintmask.  For simplicity, we use
 		 * this interpretation everywhere.
-		 *
-		 * Even though the Adobe documentation doesn't say this,
-		 * it appears that the same holds true for cntrmask.
 		 */
 	    case c2_cntrmask:
 		check_first_operator(!((csp - cstack) & 1));
