@@ -16,7 +16,7 @@
 
   Author: Raph Levien <raph@artofcode.com>
 */
-/* $Id: gxblend.c,v 1.4.2.1 2003/12/12 22:08:03 giles Exp $ */
+/* $Id: gxblend.c,v 1.6 2004/08/18 04:48:56 dan Exp $ */
 /* PDF 1.4 blending functions */
 
 #include "memory_.h"
@@ -35,6 +35,10 @@ art_blend_luminosity_rgb_8(byte *dst, const byte *backdrop,
     int delta_y;
     int r, g, b;
 
+    /*
+     * From section 7.4 of the PDF 1.5 specification, for RGB, the luminosity
+     * is:  Y = 0.30 R + 0.59 G + 0.11 B)
+     */
     delta_y = ((rs - rb) * 77 + (gs - gb) * 151 + (bs - bb) * 28 + 0x80) >> 8;
     r = rb + delta_y;
     g = gb + delta_y;
@@ -66,6 +70,34 @@ art_blend_luminosity_rgb_8(byte *dst, const byte *backdrop,
     dst[2] = b;
 }
 
+/*
+ * The PDF 1.4 spec. does not give the details of the math involved in the
+ * luminosity blending.  All we are given is:
+ *   "Creates a color with the luminance of the source color and the hue
+ *    and saturation of the backdrop color. This produces an inverse
+ *    effect to that of the Color mode."
+ * From section 7.4 of the PDF 1.5 specification, which is duscussing soft
+ * masks, we are given that, for CMYK, the luminosity is:
+ *    Y = 0.30 (1 - C)(1 - K) + 0.59 (1 - M)(1 - K) + 0.11 (1 - Y)(1 - K)
+ * However the results of this equation do not match the results seen from
+ * Illustrator CS.  Very different results are obtained if process gray
+ * (.5, .5, .5, 0) is blended over pure cyan, versus gray (0, 0, 0, .5) over
+ * the same pure cyan.  The first gives a medium cyan while the later gives a
+ * medium gray.  This routine seems to match Illustrator's actions.  C, M and Y
+ * are treated similar to RGB in the previous routine and black is treated
+ * separately.
+ *
+ * Our component values have already been complemented, i.e. (1 - X).
+ */
+static void
+art_blend_luminosity_cmyk_8(byte *dst, const byte *backdrop,
+			   const byte *src)
+{
+    /* Treat CMY the same as RGB. */
+    art_blend_luminosity_rgb_8(dst, backdrop, src);
+    dst[3] = src[3];
+}
+
 static void
 art_blend_saturation_rgb_8(byte *dst, const byte *backdrop,
 			   const byte *src)
@@ -131,6 +163,16 @@ art_blend_saturation_rgb_8(byte *dst, const byte *backdrop,
     dst[2] = b;
 }
 
+/* Our component values have already been complemented, i.e. (1 - X). */
+static void
+art_blend_saturation_cmyk_8(byte *dst, const byte *backdrop,
+			   const byte *src)
+{
+    /* Treat CMY the same as RGB */
+    art_blend_saturation_rgb_8(dst, backdrop, src);
+    dst[3] = backdrop[3];
+}
+
 /* This array consists of floor ((x - x * x / 255.0) * 65536 / 255 +
    0.5) for x in [0..255]. */
 const unsigned int art_blend_sq_diff_8[256] = {
@@ -328,20 +370,73 @@ art_blend_pixel_8(byte *dst, const byte *backdrop,
 	    }
 	    break;
 	case BLEND_MODE_Luminosity:
-	    art_blend_luminosity_rgb_8(dst, backdrop, src);
+	    switch (n_chan) {
+		case 1:			/* DeviceGray */
+	    	    dlprintf(
+			"art_blend_pixel_8: DeviceGray luminosity blend mode not implemented\n");
+		    break;
+		case 3:			/* DeviceRGB */
+	    	    art_blend_luminosity_rgb_8(dst, backdrop, src);
+		    break;
+		case 4:			/* DeviceCMYK */
+	    	    art_blend_luminosity_cmyk_8(dst, backdrop, src);
+		    break;
+		default:		/* Should not happen */
+		    break;
+	    }
 	    break;
 	case BLEND_MODE_Color:
-	    art_blend_luminosity_rgb_8(dst, src, backdrop);
+	    switch (n_chan) {
+		case 1:			/* DeviceGray */
+	    	    dlprintf(
+			"art_blend_pixel_8: DeviceGray color blend mode not implemented\n");
+		    break;
+		case 3:			/* DeviceRGB */
+		    art_blend_luminosity_rgb_8(dst, src, backdrop);
+		    break;
+		case 4:			/* DeviceCMYK */
+		    art_blend_luminosity_cmyk_8(dst, src, backdrop);
+		    break;
+		default:		/* Should not happen */
+		    break;
+	    }
 	    break;
 	case BLEND_MODE_Saturation:
-	    art_blend_saturation_rgb_8(dst, backdrop, src);
+	    switch (n_chan) {
+		case 1:			/* DeviceGray */
+	    	    dlprintf(
+			"art_blend_pixel_8: DeviceGray saturation blend mode not implemented\n");
+		    break;
+		case 3:			/* DeviceRGB */
+	    	    art_blend_saturation_rgb_8(dst, backdrop, src);
+		    break;
+		case 4:			/* DeviceCMYK */
+	    	    art_blend_saturation_cmyk_8(dst, backdrop, src);
+		    break;
+		default:		/* Should not happen */
+		    break;
+	    }
 	    break;
 	case BLEND_MODE_Hue:
 	    {
-		byte tmp[3];
+		byte tmp[4];
 
-		art_blend_luminosity_rgb_8(tmp, src, backdrop);
-		art_blend_saturation_rgb_8(dst, tmp, backdrop);
+	        switch (n_chan) {
+		    case 1:		/* DeviceGray */
+	    		dlprintf(
+			    "art_blend_pixel_8: DeviceGray hue blend mode not implemented\n");
+		        break;
+		    case 3:		/* DeviceRGB */
+			art_blend_luminosity_rgb_8(tmp, src, backdrop);
+			art_blend_saturation_rgb_8(dst, tmp, backdrop);
+		        break;
+		    case 4:		/* DeviceCMYK */
+		        art_blend_luminosity_cmyk_8(tmp, src, backdrop);
+			art_blend_saturation_cmyk_8(dst, tmp, backdrop);
+		        break;
+		    default:		/* Should not happen */
+		        break;
+	        }
 	    }
 	    break;
 	default:
