@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxclbits.c,v 1.8 2002/09/19 18:36:03 dan Exp $ */
+/* $Id: gxclbits.c,v 1.9 2004/08/04 19:36:12 stefan Exp $ */
 /* Halftone and bitmap writing for command lists */
 #include "memory_.h"
 #include "gx.h"
@@ -127,7 +127,7 @@ cmd_put_bits(gx_device_clist_writer * cldev, gx_clist_state * pcls,
     clist_bitmap_bytes(width_bits, height, compression_mask,
 		       &uncompressed_raster, &full_raster);
     uint max_size = cbuf_size - op_size;
-    gs_memory_t *mem = (cldev->memory ? cldev->memory : &gs_memory_default);
+    gs_memory_t *mem = cldev->memory;
     byte *dp;
     int compress = 0;
 
