@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxclrast.c,v 1.26.2.1 2003/12/12 22:08:03 giles Exp $ */
+/*$Id: gxclrast.c,v 1.33 2005/03/14 18:08:36 dan Exp $ */
 /* Command list interpreter/rasterizer */
 #include "memory_.h"
 #include "gx.h"
@@ -85,6 +85,7 @@ cmd_print_bits(const byte * data, int width, int height, int raster)
     if ( *p < 0x80 ) var = *p++;\
     else { const byte *_cbp; var = cmd_get_w(p, &_cbp); p = _cbp; }\
   END
+
 private long
 cmd_get_w(const byte * p, const byte ** rp)
 {
@@ -139,7 +140,7 @@ top_up_cbuf(command_buf_t *pcb, const byte *cbp)
 	nread = 1;
     }
     set_cb_end(pcb, cb_top + nread);
-    process_interrupts();
+    process_interrupts(pcb->s->memory);
     return pcb->data;
 }
 
@@ -243,11 +244,10 @@ clist_playback_band(clist_playback_action playback_action,
 #define data_bits_size cbuf_size
     byte *data_bits = 0;
     register const byte *cbp;
-    int dev_depth = cdev->color_info.depth;
-    int dev_depth_bytes = (dev_depth + 7) >> 3;
-    int odd_delta_shift = (dev_depth_bytes - 3) * 8;
+    int dev_depth;		/* May vary due to compositing devices */
+    int dev_depth_bytes;
+    int odd_delta_shift;
     int num_zero_bytes;
-    gx_color_index delta_offset = cmd_delta_offsets[dev_depth_bytes];
     gx_device *tdev;
     gx_clist_state state;
     gx_color_index *set_colors;
@@ -350,7 +350,7 @@ in:				/* Initialize for a new page. */
     gx_imager_setscreenphase(&imager_state, -x0, -y0, gs_color_select_all);
     halftone_type = ht_type_none;
     fill_params.fill_zero_width = false;
-    gs_cspace_init_DeviceGray(&cs);
+    gs_cspace_init_DeviceGray(mem, &cs);
     pcs = &cs;
     color_unset(&dev_color);
     color_space.params.indexed.use_proc = 0;
@@ -553,6 +553,8 @@ in:				/* Initialize for a new page. */
 			    gx_color_index delta = 0;
 			    uint data;
 
+			    dev_depth = cdev->color_info.depth;
+			    dev_depth_bytes = (dev_depth + 7) >> 3;
 		            switch (dev_depth_bytes) {
 				/* For cases with an even number of bytes */
 			        case 8:
@@ -583,13 +585,14 @@ in:				/* Initialize for a new page. */
 				        ((data & 0xf0) << 4) + (data & 0x0f));
 			        case 3:
 			            data = *cbp++;
+				    odd_delta_shift = (dev_depth_bytes - 3) * 8;
 			            delta |= ((gx_color_index)
 				        ((data & 0xe0) << 3) + (data & 0x1f)) << odd_delta_shift;
 				    data = *cbp++;
 			            delta |= ((gx_color_index) ((data & 0xf8) << 2) + (data & 0x07))
 				    			<< (odd_delta_shift + 11);
 		            }
-		            *pcolor += delta - delta_offset;;
+		            *pcolor += delta - cmd_delta_offsets[dev_depth_bytes];
 			}
 			if_debug1('L', " 0x%lx\n", *pcolor);
 			continue;
@@ -625,6 +628,8 @@ in:				/* Initialize for a new page. */
 		else {
 		    gx_color_index color = 0;
 
+		    dev_depth = cdev->color_info.depth;
+		    dev_depth_bytes = (dev_depth + 7) >> 3;
 		    switch (dev_depth_bytes - num_zero_bytes) {
 			case 8:
 			    color = (gx_color_index) * cbp++;
@@ -686,8 +691,8 @@ in:				/* Initialize for a new page. */
 		if (state.color_is_alpha) {
 		    if (!(op & 8))
 			depth = *cbp++;
-		} else
-		    depth = dev_depth;
+		} else 
+		    depth = cdev->color_info.depth;
 	      copy:cmd_getw(state.rect.x, cbp);
 		cmd_getw(state.rect.y, cbp);
 		if (op & 8) {	/* Use the current "tile". */
@@ -1199,6 +1204,12 @@ idata:			data_size = 0;
 				break;
 			    case cmd_opv_ext_create_compositor:
 				cbuf.ptr = cbp;
+				/*
+				 * The screen phase may have been changed during
+				 * the processing of masked images.
+				 */
+				gx_imager_setscreenphase(&imager_state,
+					    -x0, -y0, gs_color_select_all);
 				code = read_create_compositor(&cbuf, &imager_state,
 							      cdev, mem, &target);
 				cbp = cbuf.ptr;
@@ -1505,6 +1516,11 @@ idata:			data_size = 0;
     ht_buff.ht_size = 0;
     ht_buff.read_size = 0;
 
+    if (color_space.params.indexed.lookup.table.size)
+        gs_free_const_string(mem,
+			     color_space.params.indexed.lookup.table.data,
+			     color_space.params.indexed.lookup.table.size,
+			     "color_space indexed table");
     gx_cpath_free(&clip_path, "clist_render_band exit");
     gx_path_free(&path, "clist_render_band exit");
     gs_imager_state_release(&imager_state);
@@ -1862,15 +1878,15 @@ read_set_color_space(command_buf_t *pcb, gs_imager_state *pis,
 	      (b & 4 ? "(proc)" : ""));
     switch (index) {
     case gs_color_space_index_DeviceGray:
-        gs_cspace_init_DeviceGray(&gray_cs);
+        gs_cspace_init_DeviceGray(mem, &gray_cs);
         pcs = &gray_cs;
 	break;
     case gs_color_space_index_DeviceRGB:
-        gs_cspace_init_DeviceRGB(&rgb_cs);
+        gs_cspace_init_DeviceRGB(mem, &rgb_cs);
         pcs = &rgb_cs;
 	break;
     case gs_color_space_index_DeviceCMYK:
-        gs_cspace_init_DeviceCMYK(&cmyk_cs);
+        gs_cspace_init_DeviceCMYK(mem, &cmyk_cs);
         pcs = &cmyk_cs;
 	break;
     default:
@@ -1890,9 +1906,9 @@ read_set_color_space(command_buf_t *pcb, gs_imager_state *pis,
     } else {
 	if (pcolor_space->params.indexed.lookup.table.size)
 	    gs_free_const_string(mem,
-			pcolor_space->params.indexed.lookup.table.data,
-			pcolor_space->params.indexed.lookup.table.size,
-				 "old indexed table");
+				 pcolor_space->params.indexed.lookup.table.data,
+				 pcolor_space->params.indexed.lookup.table.size,
+				 "color_spapce indexed table");
 	pcolor_space->params.indexed.lookup.table.size = 0;
     }
     if (b & 8) {
@@ -1915,7 +1931,7 @@ read_set_color_space(command_buf_t *pcb, gs_imager_state *pis,
 	    data = (byte *)map->values;
 	    data_size = num_values * sizeof(map->values[0]);
 	} else {
-	    byte *table = gs_alloc_string(mem, num_values, "indexed table");
+	    byte *table = gs_alloc_string(mem, num_values, "color_space indexed table");
 
 	    if (table == 0) {
 		code = gs_note_error(gs_error_VMerror);
@@ -1952,6 +1968,7 @@ read_begin_image(command_buf_t *pcb, gs_image_common_t *pic,
 
     /* This is sloppy, but we don't have enough information to do better. */
     pcb->ptr = top_up_cbuf(pcb, pcb->ptr);
+    s_init(&s, NULL);
     sread_string(&s, pcb->ptr, pcb->end - pcb->ptr);
     code = image_type->sget(pic, &s, pcs);
     pcb->ptr = sbufptr(&s);
@@ -2078,8 +2095,16 @@ read_create_compositor(
      * change the target device.
      */
     code = dev_proc(tdev, create_compositor)(tdev, &tdev, pcomp, pis, mem);
-    if (code >= 0 && tdev != *ptarget)
+    if (code >= 0 && tdev != *ptarget) {
+        rc_increment(tdev);
         *ptarget = tdev;
+    }
+
+    /* Perform any updates for the clist device required */
+    code = pcomp->type->procs.clist_compositor_read_update(pcomp,
+		    			(gx_device *)cdev, tdev, pis, mem);
+    if (code < 0)
+        return code;
 
     /* free the compositor object */
     if (pcomp != 0)
@@ -2149,6 +2174,7 @@ cmd_read_matrix(gs_matrix * pmat, const byte * cbp)
 {
     stream s;
 
+    s_init(&s, NULL);
     sread_string(&s, cbp, 1 + sizeof(*pmat));
     sget_matrix(&s, pmat);
     return cbp + stell(&s);
