@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/*$Id: gxdcolor.c,v 1.10.2.1 2003/12/12 22:08:04 giles Exp $ */
+/*$Id: gxdcolor.c,v 1.13 2005/06/20 08:59:23 igor Exp $ */
 /* Pure and null device color implementation */
 #include "gx.h"
 #include "memory_.h"
@@ -701,7 +701,7 @@ gx_dc_write_color(
         *psize = 1;
         *pdata = 0xff;
     } else {
-        if (depth < 8 * sizeof(gx_color_index))
+        if (depth < 8 * arch_sizeof_color_index)
             color &= ((gx_color_index)1 << depth) - 1;
         while (--num_bytes >= 0) {
             pdata[num_bytes] = color & 0xff;
@@ -750,8 +750,8 @@ gx_dc_read_color(
         return 1;
     }
 
-    /* num_bytes > sizeof(gx_color_index), discard first byte */
-    for (i = (num_bytes >= sizeof(gx_color_index) ? 1 : 0); i < num_bytes; i++)
+    /* num_bytes > arch_sizeof_color_index, discard first byte */
+    for (i = (num_bytes >= arch_sizeof_color_index ? 1 : 0); i < num_bytes; i++)
         color = (color << 8) | pdata[i];
     *pcolor = color;
     return num_bytes;
