@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: gxfont.h,v 1.19.2.1 2003/12/12 22:08:04 giles Exp $ */
+/* $Id: gxfont.h,v 1.23 2004/10/13 15:31:58 igor Exp $ */
 /* Font object structure */
 /* Requires gsmatrix.h, gxdevice.h */
 
@@ -279,10 +279,11 @@ typedef struct gs_font_procs_s {
      * because for font descendents it is inherited from an upper font.
      * This is especially important for Type 42 fonts with hmtx and vmtx.
      */
+    /* Currently glyph_outline retrieves sbw only with type 1,2,9 fonts. */
 
 #define font_proc_glyph_outline(proc)\
   int proc(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,\
-	   gx_path *ppath)
+	   gx_path *ppath, double sbw[4])
     font_proc_glyph_outline((*glyph_outline));
 
     /*
@@ -473,10 +474,19 @@ gs_font_base *
 		     const gs_font_procs *procs, gs_font_dir *dir,
 		     client_name_t cname);
 
+/* Define a string to interact with unique_name in lib/pdf_font.ps .
+   The string is used to resolve glyph name conflict while
+   converting PDF Widths into Metrics.
+ */
+extern const char gx_extendeg_glyph_name_separator[];
+
 /*
  * Test whether a glyph is the notdef glyph for a base font.
  * The test is somewhat adhoc: perhaps this should be a virtual procedure.
  */
 bool gs_font_glyph_is_notdef(gs_font_base *bfont, gs_glyph glyph);
 
+/* Get font parent (a Type 9 font). */
+const gs_font_base *gs_font_parent(const gs_font_base *pbfont);
+
 #endif /* gxfont_INCLUDED */
