@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: stream.c,v 1.21.2.1 2003/12/12 22:08:05 giles Exp $ */
+/* $Id: stream.c,v 1.26 2004/10/07 05:18:34 ray Exp $ */
 /* Stream package for Ghostscript interpreter */
 #include "stdio_.h"		/* includes std.h */
 #include "memory_.h"
@@ -325,16 +325,29 @@ s_filter_write_flush(register stream * s)
 }
 
 /* Close a filter.  If this is an encoding filter, flush it first. */
+/* If CloseTarget was specified (close_strm), then propagate the sclose */
 int
 s_filter_close(register stream * s)
 {
+    int status;
+    bool close = s->close_strm;
+    stream *stemp = s->strm;
+
     if (s_is_writing(s)) {
 	int status = s_process_write_buf(s, true);
 
+	if (status != 0 && status != EOFC)
+	    return status;
+        status = sflush(stemp);
 	if (status != 0 && status != EOFC)
 	    return status;
     }
-    return s_std_close(s);
+    status = s_std_close(s);
+    if (status != 0 && status != EOFC)
+	return status;
+    if (close && stemp != 0)
+	return sclose(stemp);
+    return status;
 }
 
 /* Disregard a stream error message. */
@@ -807,8 +820,9 @@ sreadbuf(stream * s, stream_cursor_write * pbuf)
 	    MOVE_AHEAD(curr, prev);
 	    stream_compact(curr, false);
 	}
-	/* If curr reached EOD and is a filter stream, close it. */
-	if (strm != 0 && status == EOFC &&
+	/* If curr reached EOD and is a filter or file stream, close it. */
+	/* (see PLRM 3rd, sec 3.8.2, p80) */
+	if ((strm != 0 || curr->file) && status == EOFC &&
 	    curr->cursor.r.ptr >= curr->cursor.r.limit &&
 	    curr->close_at_eod
 	    ) {
