@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: ttfmain.c,v 1.19.2.2 2004/01/29 13:17:10 igor Exp $ */
+/* $Id: ttfmain.c,v 1.34 2005/08/02 11:12:32 igor Exp $ */
 /* A Free Type interface adapter. */
 /* Uses code fragments from the FreeType project. */
 
@@ -30,11 +30,6 @@
 
 private const bool skip_instructions = 0; /* Debug purpose only. */
 
-typedef struct { 
-    F26Dot6 x;
-    F26Dot6 y;
-} F26Dot6Point;
-
 typedef struct { 
     Fixed a, b, c, d, tx, ty;
 } FixMatrix;
@@ -46,15 +41,6 @@ struct ttfSubGlyphUsage_s {
     short arg1, arg2;
 };
 
-typedef struct { 
-    bool      bCompound;
-    int32     contourCount;
-    uint32    pointCount;
-    F26Dot6Point  advance;
-    F26Dot6 sideBearing;
-    F26Dot6   xMinB, yMinB, xMaxB, yMaxB;
-} ttfGlyphOutline;
-
 /*------------------------------------------------------------------- */
 
 private Fixed AVE(F26Dot6 a, F26Dot6 b)
@@ -112,7 +98,7 @@ private ttfPtrElem *ttfFont__get_table_ptr(ttfFont *f, char *id)
     if (!memcmp(id, "cmap", 4))
 	return &f->t_cmap;
     return 0;
-  };
+}
 
 /*-------------------------------------------------------------------*/
 
@@ -198,7 +184,7 @@ void ttfInterpreter__release(ttfInterpreter **ptti)
 
 void ttfFont__init(ttfFont *this, ttfMemory *mem, 
 		    void (*DebugRepaint)(ttfFont *),
-		    void (*DebugPrint)(ttfFont *, const char *s, ...))
+		    int (*DebugPrint)(ttfFont *, const char *s, ...))
 {
     memset(this, 0, sizeof(*this));
     this->DebugRepaint = DebugRepaint;
@@ -305,7 +291,7 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
 		"ttfFont__Open");
 	if (tti->usage == NULL)
 	    return fMemoryError;
-	tti->usage_size = this->nMaxComponents;
+	tti->usage_size = this->nMaxComponents * MAX_SUBGLYPH_NESTING;
     }
     this->face = mem->alloc_struct(mem, (const ttfMemoryDescriptor *)&st_TFace, "ttfFont__Open");
     if (this->face == NULL)
@@ -337,12 +323,15 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
     code = Instance_Init(this->inst);
     if (code == TT_Err_Out_Of_Memory)
 	return fMemoryError;
+    if (code >= TT_Err_Invalid_Opcode && code <= TT_Err_Invalid_Displacement)
+	return fBadInstruction;
     if (code)
 	return fBadFontData;
     I.z = this->inst;
     if (design_grid)
 	ww = hh = shortToF26Dot6(this->nUnitsPerEm);
     else {
+	/* Round towards zero for a better view of mirrored characters : */
 	ww = floatToF26Dot6(w);
 	hh = floatToF26Dot6(h);
     }
@@ -352,6 +341,8 @@ FontError ttfFont__Open(ttfInterpreter *tti, ttfFont *this, ttfReader *r,
 	return fPatented;
     if (code == TT_Err_Out_Of_Memory)
 	return fMemoryError;
+    if (code >= TT_Err_Invalid_Opcode && code <= TT_Err_Invalid_Displacement)
+	return fBadInstruction;
     if (code)
 	return fBadFontData;
     return code;
@@ -469,7 +460,7 @@ private void MoveGlyphOutline(TGlyph_Zone *pts, int nOffset, ttfGlyphOutline *ou
     short count = out->pointCount;
     F26Dot6Point p;
 
-    if (m->a != 65536 && m->b == 0 && 
+    if (m->a == 65536 && m->b == 0 && 
 	m->c == 0 && m->d == 65536 && 
 	m->tx == 0 && m->ty == 0)
 	return;
@@ -537,6 +528,8 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
 	return fGlyphNotFound;
     if (r->Eof(r)) {
 	r->ReleaseGlyph(r, glyphIndex);
+	gOutline->xMinB = gOutline->yMinB = 0;
+	gOutline->xMaxB = gOutline->yMaxB = 0;
 	return fNoError;
     }
     if (r->Error(r))
@@ -585,7 +578,7 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
 	    }
 	    flags = ttfReader__UShort(r);
 	    index = ttfReader__UShort(r);
-	    bHaveInstructions |= (flags&WE_HAVE_INSTRUCTIONS);
+	    bHaveInstructions |= (flags & WE_HAVE_INSTRUCTIONS);
 	    if (flags & ARG_1_AND_2_ARE_WORDS) {
 		arg1 = ttfReader__Short(r);
 		arg2 = ttfReader__Short(r);
@@ -599,7 +592,7 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
 		    arg2 = ttfReader__Byte(r);
                 }
             }
-	    m.b = m.c = 0;
+	    m.b = m.c = m.tx = m.ty = 0;
 	    if (flags & WE_HAVE_A_SCALE)
 		m.a = m.d = (Fixed)ttfReader__Short(r) << 2;
 	    else if (flags & WE_HAVE_AN_X_AND_Y_SCALE) {
@@ -663,8 +656,8 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
 		e->m.tx = Scale_X( &exec->metrics, e->arg1 ) << 10;
 		e->m.ty = Scale_Y( &exec->metrics, e->arg2 ) << 10;
             } else {
-		e->m.tx = (pts->org_x[nPointsStored + e->arg1] - pts->org_x[nPointsStored + e->arg2]) << 10;
-		e->m.ty = (pts->org_y[nPointsStored + e->arg1] - pts->org_y[nPointsStored + e->arg2]) << 10;
+		e->m.tx = (pts->org_x[e->arg1] - pts->org_x[gOutline->pointCount + e->arg2]) << 10;
+		e->m.ty = (pts->org_y[e->arg1] - pts->org_y[gOutline->pointCount + e->arg2]) << 10;
             }
 	    MoveGlyphOutline(pts, nPointsStored, &out, &e->m);
 	    for (j = nContoursStored; j < out.contourCount + nContoursStored; j++)
@@ -839,7 +832,7 @@ private FontError ttfOutliner__BuildGlyphOutlineAux(ttfOutliner *this, int glyph
 	    else if (code == TT_Err_Invalid_Engine)
 		error = fPatented;
 	    else
-		error = fBadFontData;
+		error = fBadInstruction;
 	    gOutline->sideBearing = subglyph.bbox.xMin - subglyph.pp1.x;
 	    gOutline->advance.x = subglyph.pp2.x - subglyph.pp1.x;
         }
@@ -860,6 +853,7 @@ private FontError ttfOutliner__BuildGlyphOutline(ttfOutliner *this, int glyphInd
 {
     FixMatrix m_orig = {1 << 16, 0, 0, 1 << 16, 0, 0};
 
+    /* Round towards zero like old character coordinate conversions do. */
     m_orig.tx = floatToF16Dot16(orig_x);
     m_orig.ty = floatToF16Dot16(orig_y);
     return ttfOutliner__BuildGlyphOutlineAux(this, glyphIndex, &m_orig, gOutline);
@@ -868,8 +862,10 @@ private FontError ttfOutliner__BuildGlyphOutline(ttfOutliner *this, int glyphInd
 
 #define AVECTOR_BUG 1 /* Work around a bug in AVector fonts. */
 
-private void ttfOutliner__DrawGlyphOutline(ttfOutliner *this, ttfGlyphOutline* out, FloatMatrix *m)
-{   ttfFont *pFont = this->pFont;
+void ttfOutliner__DrawGlyphOutline(ttfOutliner *this)
+{   ttfGlyphOutline* out = &this->out;
+    FloatMatrix *m = &this->post_transform;
+    ttfFont *pFont = this->pFont;
     ttfExport *exp = this->exp;
     TExecution_Context *exec = pFont->exec;
     TGlyph_Zone *pts = &exec->pts;
@@ -897,6 +893,10 @@ private void ttfOutliner__DrawGlyphOutline(ttfOutliner *this, ttfGlyphOutline* o
 	yMax = Scale_X(&exec->metrics, yMaxB);
 #   endif
 
+    TransformF26Dot6PointFloat(&p1, out->advance.x, out->advance.y, m);
+    p1.x -= this->post_transform.tx;
+    p1.y -= this->post_transform.ty;
+    exp->SetWidth(exp, &p1);
     sp = -1;
     for (ctr = out->contourCount; ctr != 0; --ctr) {
 	short pt, pts = *endP - sp;
@@ -1016,54 +1016,27 @@ private void ttfOutliner__DrawGlyphOutline(ttfOutliner *this, ttfGlyphOutline* o
 FontError ttfOutliner__Outline(ttfOutliner *this, int glyphIndex,
 	float orig_x, float orig_y, FloatMatrix *m1)
 {   ttfFont *pFont = this->pFont;
-    ttfExport *exp = this->exp;
-    ttfGlyphOutline out;
     FontError error;
-    FloatPoint p1;
-    FloatMatrix m = *m1;
 
-    out.contourCount = 0;
-    out.pointCount = 0;
-    out.bCompound = FALSE;
+    this->post_transform = *m1;
+    this->out.contourCount = 0;
+    this->out.pointCount = 0;
+    this->out.bCompound = FALSE;
     this->nPointsTotal = 0;
     this->nContoursTotal = 0;
-    out.advance.x = out.advance.y = 0;
+    this->out.advance.x = this->out.advance.y = 0;
     ttfFont__StartGlyph(pFont);
-    error = ttfOutliner__BuildGlyphOutline(this, glyphIndex, orig_x, orig_y, &out);
+    error = ttfOutliner__BuildGlyphOutline(this, glyphIndex, orig_x, orig_y, &this->out);
     ttfFont__StopGlyph(pFont);
     if (pFont->nUnitsPerEm <= 0)
 	pFont->nUnitsPerEm = 1024;
     if (pFont->design_grid) {
-	m.a /= pFont->nUnitsPerEm;
-	m.b /= pFont->nUnitsPerEm;
-	m.c /= pFont->nUnitsPerEm;
-	m.d /= pFont->nUnitsPerEm;
+	this->post_transform.a /= pFont->nUnitsPerEm;
+	this->post_transform.b /= pFont->nUnitsPerEm;
+	this->post_transform.c /= pFont->nUnitsPerEm;
+	this->post_transform.d /= pFont->nUnitsPerEm;
     }
-    TransformF26Dot6PointFloat(&p1, out.advance.x, out.advance.y, &m);
-    exp->SetWidth(exp, &p1);
     if (error != fNoError && error != fPatented)
 	return error;
-    if (this->bOutline)
-	ttfOutliner__DrawGlyphOutline(this, &out, &m);
-    /* Draw a rectangle as a dummy glyph
-    else {
-	FixPoint p0, p2, p3;
-
-	TransformPointFix(&p0, 0, out.yMinG, &m1);
-	TransformPointFix(&p1, out.advance.x, out.yMaxG, &m1);
-	p2 = p0;
-	p3 = p1;
-	p2.x = p1.x;
-	p3.y = p0.y;
-	o.bOutline = TRUE;
-	o.MoveTo(&p0);
-	o.LineTo(&p1);
-	o.LineTo(&p2);
-	o.LineTo(&p3);
-	o.Close();
-	o.bOutline = bOutline;
-    }
-    */
     return error;
 }
-
