@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: ttobjs.c,v 1.6.4.1 2003/12/12 22:08:05 giles Exp $ */
+/* $Id: ttobjs.c,v 1.9 2004/12/21 20:13:41 igor Exp $ */
 
 /* Changes after FreeType: cut out the TrueType instruction interpreter. */
 
@@ -276,7 +276,6 @@ static int free_aux(ttfMemory *mem, void *ptr)
    exec->callTop  = 0;
 
    /* free glyph code range */
-   FREE( exec->glyphIns );
    exec->glyphSize = 0;
    exec->maxGlyphSize = 0;
 
@@ -315,10 +314,7 @@ static int free_aux(ttfMemory *mem, void *ptr)
    n_points        = face->maxPoints + 2;
    n_twilight      = maxp->maxTwilightPoints;
 
-   if ( ALLOC_ARRAY( exec->glyphIns, exec->maxGlyphSize, maxp->maxSizeOfInstructions, Byte )        ||
-        /* reserve glyph code range */
-
-        ALLOC_ARRAY( exec->callStack, exec->callSize, callSize, TCallRecord ) ||
+   if ( ALLOC_ARRAY( exec->callStack, exec->callSize, callSize, TCallRecord ) ||
         /* reserve interpreter call stack */
 
         ALLOC_ARRAY( exec->stack, exec->stackSize, stackSize, Long )           ||
@@ -377,6 +373,8 @@ static int free_aux(ttfMemory *mem, void *ptr)
    exec->numIDefs = ins->numIDefs;
    exec->FDefs    = ins->FDefs;
    exec->IDefs    = ins->IDefs;
+   exec->countIDefs = ins->countIDefs;
+   memcpy(exec->IDefPtr, ins->IDefPtr, sizeof(exec->IDefPtr));
 
    exec->metrics  = ins->metrics;
 
@@ -419,6 +417,9 @@ static int free_aux(ttfMemory *mem, void *ptr)
    }
    exec->numFDefs = 0;
    exec->numIDefs = 0;
+   memcpy(ins->IDefPtr, exec->IDefPtr, sizeof(ins->IDefPtr));
+   ins->countIDefs = exec->countIDefs;
+   exec->countIDefs = 0;
    exec->FDefs    = 0;
    exec->IDefs    = 0;
    exec->cvtSize = 0;
@@ -573,6 +574,12 @@ static int free_aux(ttfMemory *mem, void *ptr)
 
     ins->numFDefs = maxp->maxFunctionDefs;
     ins->numIDefs = maxp->maxInstructionDefs;
+    ins->countIDefs = 0;
+    if (maxp->maxInstructionDefs > 255)
+	goto Fail_Memory;
+    memset(ins->IDefPtr, (Byte)ins->numIDefs, sizeof(ins->IDefPtr));
+    if (ins->numFDefs < 50)
+	ins->numFDefs = 50; /* Bug 687858 */
     ins->cvtSize  = face->cvtSize;
 
     ins->metrics.pointSize    = 10 * 64;     /* default pointsize  = 10pts */
