@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: zbfont.c,v 1.18.2.1 2003/12/12 22:08:05 giles Exp $ */
+/* $Id: zbfont.c,v 1.30 2005/07/27 11:24:38 igor Exp $ */
 /* Font creation utilities */
 #include "memory_.h"
 #include "string_.h"
@@ -69,14 +69,14 @@ zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t gspace)
     const ref *pencoding = &pdata->Encoding;
     ulong index = chr;	/* work around VAX widening bug */
     ref cname;
-    int code = array_get(pencoding, (long)index, &cname);
+    int code = array_get(pfont->memory, pencoding, (long)index, &cname);
 
     if (code < 0 || !r_has_type(&cname, t_name))
 	return gs_no_glyph;
     if (pfont->FontType == ft_user_defined && r_type(&pdata->BuildGlyph) == t_null) {
 	ref nsref, tname;
 
-	name_string_ref(&cname, &nsref);
+	name_string_ref(pfont->memory, &cname, &nsref);
 	if (r_size(&nsref) == 7 && 
 	    !memcmp(nsref.value.const_bytes, ".notdef", r_size(&nsref))) {
 	    /* A special support for high level devices.
@@ -94,7 +94,7 @@ zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t gspace)
 	    if (gspace == GLYPH_SPACE_NOGEN)
 		return gs_no_glyph;    
 	    sprintf(buf, "j%ld", chr); /* 'j' is arbutrary. */
-	    code = name_ref((const byte *)buf, strlen(buf), &tname, 1);
+	    code = name_ref(pfont->memory, (const byte *)buf, strlen(buf), &tname, 1);
 	    if (code < 0) {
 		/* Can't propagate the error due to interface limitation,
 		   return with .notdef */
@@ -102,7 +102,7 @@ zfont_encode_char(gs_font *pfont, gs_char chr, gs_glyph_space_t gspace)
 		cname = tname;
 	}
     }
-    return (gs_glyph)name_index(&cname);
+    return (gs_glyph)name_index(pfont->memory, &cname);
 }
 
 /* Get the name of a glyph. */
@@ -116,20 +116,20 @@ zfont_glyph_name(gs_font *font, gs_glyph index, gs_const_string *pstr)
 	int code;
 
 	sprintf(cid_name, "%lu", (ulong) index);
-	code = name_ref((const byte *)cid_name, strlen(cid_name),
+	code = name_ref(font->memory, (const byte *)cid_name, strlen(cid_name),
 			&nref, 1);
 	if (code < 0)
 	    return code;
     } else
-	name_index_ref(index, &nref);
-    name_string_ref(&nref, &sref);
+	name_index_ref(font->memory, index, &nref);
+    name_string_ref(font->memory, &nref, &sref);
     pstr->data = sref.value.const_bytes;
     pstr->size = r_size(&sref);
     return 0;
 }
 
 private gs_char 
-gs_font_map_glyph_by_dict(const ref *map, gs_glyph glyph)
+gs_font_map_glyph_by_dict(const gs_memory_t *mem, const ref *map, gs_glyph glyph)
 {
     ref *v, n;
     if (glyph >= gs_min_cid_glyph) {
@@ -141,7 +141,7 @@ gs_font_map_glyph_by_dict(const ref *map, gs_glyph glyph)
 	    if (dict_find(map, &n, &v) > 0) {
 		ref vv;
 
-		if (array_get(v, cid % 256, &vv) == 0 && r_type(&vv) == t_integer)
+		if (array_get(mem, v, cid % 256, &vv) == 0 && r_type(&vv) == t_integer)
 		    return vv.value.intval;
 	    }
 	    return GS_NO_CHAR; /* Absent in the map. */
@@ -149,7 +149,7 @@ gs_font_map_glyph_by_dict(const ref *map, gs_glyph glyph)
 	/* This is GlyphNames2Unicode dictionary. */
 	make_int(&n, cid);
     } else
-	name_index_ref(glyph, &n);
+	name_index_ref(mem, glyph, &n);
     if (dict_find(map, &n, &v) > 0) {
 	if (r_has_type(v, t_string)) {
 	    int i, l = r_size(v);
@@ -173,7 +173,8 @@ gs_font_map_glyph_to_unicode(gs_font *font, gs_glyph glyph)
     const ref *UnicodeDecoding;
 
     if (r_type(&pdata->GlyphNames2Unicode) == t_dictionary) {
-	gs_char c = gs_font_map_glyph_by_dict(&pdata->GlyphNames2Unicode, glyph);
+	gs_char c = gs_font_map_glyph_by_dict(font->memory, 
+					      &pdata->GlyphNames2Unicode, glyph);
 
 	if (c != GS_NO_CHAR)
 	    return c;
@@ -185,7 +186,7 @@ gs_font_map_glyph_to_unicode(gs_font *font, gs_glyph glyph)
     }
     UnicodeDecoding = zfont_get_to_unicode_map(font->dir);
     if (UnicodeDecoding != NULL && r_type(UnicodeDecoding) == t_dictionary)
-	return gs_font_map_glyph_by_dict(UnicodeDecoding, glyph);
+	return gs_font_map_glyph_by_dict(font->memory, UnicodeDecoding, glyph);
     return GS_NO_CHAR; /* No map. */
 }
 
@@ -201,7 +202,7 @@ const op_def zbfont_op_defs[] =
 
 /* Convert strings to executable names for build_proc_refs. */
 int
-build_proc_name_refs(build_proc_refs * pbuild,
+build_proc_name_refs(const gs_memory_t *mem, build_proc_refs * pbuild,
 		     const char *bcstr, const char *bgstr)
 {
     int code;
@@ -209,14 +210,16 @@ build_proc_name_refs(build_proc_refs * pbuild,
     if (!bcstr)
 	make_null(&pbuild->BuildChar);
     else {
-	if ((code = name_ref((const byte *)bcstr, strlen(bcstr), &pbuild->BuildChar, 0)) < 0)
+	if ((code = name_ref(mem, (const byte *)bcstr, 
+			     strlen(bcstr), &pbuild->BuildChar, 0)) < 0)
 	    return code;
 	r_set_attrs(&pbuild->BuildChar, a_executable);
     }
     if (!bgstr)
 	make_null(&pbuild->BuildGlyph);
     else {
-	if ((code = name_ref((const byte *)bgstr, strlen(bgstr), &pbuild->BuildGlyph, 0)) < 0)
+	if ((code = name_ref(mem, (const byte *)bgstr, 
+			     strlen(bgstr), &pbuild->BuildGlyph, 0)) < 0)
 	    return code;
 	r_set_attrs(&pbuild->BuildGlyph, a_executable);
     }
@@ -251,6 +254,37 @@ build_gs_font_procs(os_ptr op, build_proc_refs * pbuild)
     return 0;
 }
 
+private int font_with_same_UID_and_another_metrics(const gs_font *pfont0, const gs_font *pfont1)
+{
+    const gs_font_base *pbfont0 = (const gs_font_base *)pfont0;
+    const gs_font_base *pbfont1 = (const gs_font_base *)pfont1;
+
+
+    if (uid_equal(&pbfont0->UID, &pbfont1->UID)) {
+	const ref *pfdict0 = &pfont_data(gs_font_parent(pbfont0))->dict;
+	const ref *pfdict1 = &pfont_data(gs_font_parent(pbfont1))->dict;
+	ref *pmdict0, *pmdict1;
+
+	if (pbfont0->WMode || dict_find_string(pfdict0, "Metrics", &pmdict0) <= 0)
+	    pmdict0 = NULL;
+	if (pbfont1->WMode || dict_find_string(pfdict1, "Metrics", &pmdict1) <= 0)
+	    pmdict1 = NULL;
+	if (!pmdict0 != !pmdict1)
+	    return 1;
+	if (pmdict0 != NULL && !obj_eq(pfont0->memory, pmdict0, pmdict1))
+	    return 1;
+	if (!pbfont0->WMode || dict_find_string(pfdict0, "Metrics2", &pmdict0) <= 0)
+	    pmdict0 = NULL;
+	if (!pbfont0->WMode || dict_find_string(pfdict1, "Metrics2", &pmdict1) <= 0)
+	    pmdict1 = NULL;
+	if (!pmdict0 != !pmdict1)
+	    return 1;
+	if (pmdict0 != NULL && !obj_eq(pfont0->memory, pmdict0, pmdict1))
+	    return 1;
+    }
+    return 0;
+}
+
 /* Do the common work for building a primitive font -- one whose execution */
 /* algorithm is implemented in C (Type 1, Type 2, Type 4, or Type 42). */
 /* The caller guarantees that *op is a dictionary. */
@@ -300,6 +334,16 @@ build_gs_primitive_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base ** ppfont,
 	!dict_check_uid_param(op, &pfont->UID)
 	)
 	uid_set_invalid(&pfont->UID);
+    if (uid_is_valid(&pfont->UID)) {
+	const gs_font *pfont0 = (const gs_font *)pfont;
+	
+	code = gs_font_find_similar(ifont_dir, &pfont0, 
+		       font_with_same_UID_and_another_metrics);
+	if (code < 0)
+	    return code; /* Must not happen. */
+	if (code)
+	    uid_set_invalid(&pfont->UID);
+    }
     return 0;
 }
 
@@ -393,14 +437,20 @@ build_gs_simple_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base ** ppfont,
     gs_uid uid;
     int code;
     gs_font_base *pfont;
-    ref *pfontinfo, *g2u = NULL;
+    ref *pfontinfo, *g2u = NULL, Glyph2Unicode;
 
     if (dict_find_string(op, "FontInfo", &pfontinfo) <= 0 ||
 	    !r_has_type(pfontinfo, t_dictionary) ||
 	    dict_find_string(pfontinfo, "GlyphNames2Unicode", &g2u) <= 0 ||
 	    !r_has_type(pfontinfo, t_dictionary))
 	g2u = NULL;
-    code = font_bbox_param(op, bbox);
+    else
+	/*
+	 * Since build_gs_font may resize the dictionary and cause
+	 * pointers to become invalid, save Glyph2Unicode
+	 */
+	Glyph2Unicode = *g2u;
+    code = font_bbox_param(imemory, op, bbox);
     if (code < 0)
 	return code;
     code = dict_uid_param(op, &uid, 0, imemory, i_ctx_p);
@@ -425,7 +475,7 @@ build_gs_simple_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font_base ** ppfont,
     if (g2u != NULL) {
 	font_data *pdata = pfont_data(pfont);
 
-	ref_assign_new(&pdata->GlyphNames2Unicode, g2u);
+	ref_assign_new(&pdata->GlyphNames2Unicode, &Glyph2Unicode);
     }
     return 0;
 }
@@ -462,14 +512,14 @@ lookup_gs_simple_font_encoding(gs_font_base * pfont)
 	for (i = 0; i < esize; ++i) {
 	    ref fchar;
 
-	    if (array_get(pfe, (long)i, &fchar) < 0 ||
+	    if (array_get(pfont->memory, pfe, (long)i, &fchar) < 0 ||
 		!r_has_type(&fchar, t_name)
 		)
 		fstrs[i].data = 0, fstrs[i].size = 0;
 	    else {
 		ref nsref;
 
-		name_string_ref(&fchar, &nsref);
+		name_string_ref(pfont->memory, &fchar, &nsref);
 		fstrs[i].data = nsref.value.const_bytes;
 		fstrs[i].size = r_size(&nsref);
 	    }
@@ -506,14 +556,14 @@ lookup_gs_simple_font_encoding(gs_font_base * pfont)
 
 /* Get FontMatrix and FontName parameters. */
 private int
-sub_font_params(const ref *op, gs_matrix *pmat, gs_matrix *pomat, ref *pfname)
+sub_font_params(const gs_memory_t *mem, const ref *op, gs_matrix *pmat, gs_matrix *pomat, ref *pfname)
 {
     ref *pmatrix;
     ref *pfontname;
     ref *porigfont;
 
     if (dict_find_string(op, "FontMatrix", &pmatrix) <= 0 ||
-	read_matrix(pmatrix, pmat) < 0
+	read_matrix(mem, pmatrix, pmat) < 0
 	)
 	return_error(e_invalidfont);
     if (dict_find_string(op, ".OrigFont", &porigfont) <= 0)
@@ -521,15 +571,19 @@ sub_font_params(const ref *op, gs_matrix *pmat, gs_matrix *pomat, ref *pfname)
     if (pomat!= NULL) {
 	if (porigfont == NULL ||
 	    dict_find_string(porigfont, "FontMatrix", &pmatrix) <= 0 ||
-	    read_matrix(pmatrix, pomat) < 0
+	    read_matrix(mem, pmatrix, pomat) < 0
 	    )
 	    memset(pomat, 0, sizeof(*pomat));
     }
-    if (dict_find_string((porigfont != NULL ? porigfont : op), ".Alias", &pfontname) > 0) {
+    /* Use the FontInfo/OrigFontName key preferrentially (created by MS PSCRIPT driver) */
+    if ((dict_find_string((porigfont != NULL ? porigfont : op), "FontInfo", &pfontname) > 0) &&
+        (dict_find_string(pfontname, "OrigFontName", &pfontname) > 0)) {
+	get_font_name(mem, pfname, pfontname);
+    } else if (dict_find_string((porigfont != NULL ? porigfont : op), ".Alias", &pfontname) > 0) {
         /* If we emulate the font, we want the requested name rather than a substitute. */
-	get_font_name(pfname, pfontname);
+	get_font_name(mem, pfname, pfontname);
     } else if (dict_find_string((porigfont != NULL ? porigfont : op), "FontName", &pfontname) > 0) {
-	get_font_name(pfname, pfontname);
+	get_font_name(mem, pfname, pfontname);
     } else
 	make_empty_string(pfname, a_readonly);
     return 0;
@@ -557,7 +611,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
     ref *pfid;
     ref *aop = dict_access_ref(op);
 
-    get_font_name(&kname, op - 1);
+    get_font_name(imemory, &kname, op - 1);
     if (dict_find_string(op, "FontType", &pftype) <= 0 ||
 	!r_has_type(pftype, t_integer) ||
 	pftype->value.intval != (int)ftype
@@ -593,7 +647,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
 	if (pfont->base == pfont) {	/* original font */
 	    if (!level2_enabled)
 		return_error(e_invalidfont);
-	    if (obj_eq(pfont_dict(pfont), op)) {
+	    if (obj_eq(pfont->memory, pfont_dict(pfont), op)) {
 		*ppfont = pfont;
 		return 1;
 	    }
@@ -607,7 +661,7 @@ build_gs_font(i_ctx_t *i_ctx_p, os_ptr op, gs_font ** ppfont, font_type ftype,
 	    gs_matrix mat;
 	    ref fname;			/* t_string */
 
-	    code = sub_font_params(op, &mat, NULL, &fname);
+	    code = sub_font_params(imemory, op, &mat, NULL, &fname);
 	    if (code < 0)
 		return code;
 	    code = 1;
@@ -666,7 +720,7 @@ build_gs_sub_font(i_ctx_t *i_ctx_p, const ref *op, gs_font **ppfont,
      * in the same VM as the font dictionary.
      */
     uint space = ialloc_space(idmemory);
-    int code = sub_font_params(op, &mat, &omat, &fname);
+    int code = sub_font_params(imemory, op, &mat, &omat, &fname);
 
     if (code < 0)
 	return code;
@@ -711,14 +765,14 @@ build_gs_sub_font(i_ctx_t *i_ctx_p, const ref *op, gs_font **ppfont,
 /* Get the string corresponding to a font name. */
 /* If the font name isn't a name or a string, return an empty string. */
 void
-get_font_name(ref * pfname, const ref * op)
+get_font_name(const gs_memory_t *mem, ref * pfname, const ref * op)
 {
     switch (r_type(op)) {
 	case t_string:
 	    *pfname = *op;
 	    break;
 	case t_name:
-	    name_string_ref(op, pfname);
+	    name_string_ref(mem, op, pfname);
 	    break;
 	default:
 	    /* This is weird, but legal.... */
