@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: zchar1.c,v 1.28.2.2 2003/12/20 17:00:18 giles Exp $ */
+/* $Id: zchar1.c,v 1.44 2005/04/14 19:10:14 raph Exp $ */
 /* Type 1 character display operator */
 #include "memory_.h"
 #include "ghost.h"
@@ -158,8 +158,6 @@ private int type1_continue_dispatch(i_ctx_t *, gs_type1exec_state *,
 				    const ref *, ref *, int);
 private int op_type1_cleanup(i_ctx_t *);
 private void op_type1_free(i_ctx_t *);
-private void
-     type1_cis_get_metrics(const gs_type1_state * pcis, double psbw[4]);
 private int bbox_getsbw_continue(i_ctx_t *);
 private int type1exec_bbox(i_ctx_t *, gs_type1exec_state *, gs_font *, op_proc_t *exec_cont);
 private int bbox_finish_fill(i_ctx_t *);
@@ -293,12 +291,8 @@ charstring_execchar(i_ctx_t *i_ctx_p, int font_type_mask)
 		return type1_call_OtherSubr(i_ctx_p, &cxs, nobbox_continue,
 					    &other_subr);
 	    case type1_result_sbw:	/* [h]sbw, just continue */
-		if (cxs.present != metricsSideBearingAndWidth) {
-		    if (!cxs.use_FontBBox_as_Metrics2)
- 		        type1_cis_get_metrics(pcis, cxs.sbw);
-	            else
-			cxs.present = metricsSideBearingAndWidth;
-		}
+		if (cxs.present != metricsSideBearingAndWidth)
+		    type1_cis_get_metrics(pcis, cxs.sbw);
 		opstr = 0;
 		goto icont;
 	}
@@ -396,7 +390,7 @@ bbox_getsbw_continue(i_ctx_t *i_ctx_p)
 	    type1_cis_get_metrics(pcis, sbw);
 	    bbox = pcxs->char_bbox;
 	    op_type1_free(i_ctx_p);
-	    code = zchar_set_cache(i_ctx_p, pbfont, op, sbw, sbw + 2, &bbox,
+	    code = zchar_set_cache(i_ctx_p, pbfont, op - 1, sbw, sbw + 2, &bbox,
 				   cont, &exec_cont, NULL);
 	    if (code >= 0 && exec_cont != 0)
 		code = (*exec_cont)(i_ctx_p);
@@ -575,6 +569,7 @@ bbox_draw(i_ctx_t *i_ctx_p, int (*draw)(gs_state *), op_proc_t *exec_cont)
 	if (code < 0)
 	    return code;
 	cxs.present = code;
+	cxs.use_FontBBox_as_Metrics2 = false;
     }  else {
 	cxs.sbw[0] = penum->FontBBox_as_Metrics2.x / 2;
 	cxs.sbw[1] = penum->FontBBox_as_Metrics2.y;
@@ -616,16 +611,6 @@ bbox_stroke(i_ctx_t *i_ctx_p)
 
 /* -------- Common code -------- */
 
-/* Get the metrics (l.s.b. and width) from the Type 1 interpreter. */
-private void
-type1_cis_get_metrics(const gs_type1_state * pcis, double psbw[4])
-{
-    psbw[0] = fixed2float(pcis->lsb.x);
-    psbw[1] = fixed2float(pcis->lsb.y);
-    psbw[2] = fixed2float(pcis->width.x);
-    psbw[3] = fixed2float(pcis->width.y);
-}
-
 /* Handle the results of interpreting the CharString. */
 /* pcref points to a t_string ref. */
 private int
@@ -637,6 +622,7 @@ type1_continue_dispatch(i_ctx_t *i_ctx_p, gs_type1exec_state *pcxs,
     gs_glyph_data_t cs_data;
     gs_glyph_data_t *pcsd;
 
+    cs_data.memory = imemory;
     if (pcref == 0) {
 	pcsd = 0;
     } else {
@@ -665,7 +651,7 @@ type1_continue_dispatch(i_ctx_t *i_ctx_p, gs_type1exec_state *pcxs,
 	     */
 	    const font_data *pfdata = pfont_data(gs_currentfont(igs));
 
-	    code = array_get(&pfdata->u.type1.OtherSubrs, (long)value, pos);
+	    code = array_get(imemory, &pfdata->u.type1.OtherSubrs, (long)value, pos);
 	    if (code >= 0)
 		return type1_result_callothersubr;
 	}
@@ -834,7 +820,7 @@ nobbox_finish(i_ctx_t *i_ctx_p, gs_type1exec_state * pcxs)
 	    code = type1exec_bbox(i_ctx_p, pcxs, pfont, &exec_cont);
 	} else {
 	    cont = (pbfont->PaintType == 0 ? nobbox_fill : nobbox_stroke), exec_cont = 0;
-	    code = zchar_set_cache(i_ctx_p, pbfont, op, NULL,
+	    code = zchar_set_cache(i_ctx_p, pbfont, op - 1, NULL,
 				   pcxs->sbw + 2,
 				   &pcxs->char_bbox,
 				   cont, &exec_cont,
@@ -864,7 +850,15 @@ nobbox_fill(i_ctx_t *i_ctx_p)
 private int
 nobbox_stroke(i_ctx_t *i_ctx_p)
 {
-    return nobbox_draw(i_ctx_p, gs_stroke);
+    /* As a compatibility to Adobe, use the exact "StrokeWidth".
+       Reset fill_adjust for that. */
+    int code;
+    gs_fixed_point fa = i_ctx_p->pgs->fill_adjust;
+
+    i_ctx_p->pgs->fill_adjust.x = i_ctx_p->pgs->fill_adjust.y = 0;
+    code = nobbox_draw(i_ctx_p, gs_stroke);
+    i_ctx_p->pgs->fill_adjust = fa;
+    return code;
 }
 
 /* <font> <array> .setweightvector - */
@@ -888,7 +882,7 @@ zsetweightvector(i_ctx_t *i_ctx_p)
     size = r_size(op);
     if (size != pfont1->data.WeightVector.count)
 	return_error(e_invalidfont);
-    code = process_float_array(op, size, pfont1->data.WeightVector.values);
+    code = process_float_array(imemory, op, size, pfont1->data.WeightVector.values);
     if (code < 0)
 	return code;
     pop(2);
@@ -919,7 +913,7 @@ z1_glyph_data(gs_font_type1 * pfont, gs_glyph glyph, gs_glyph_data_t *pgd)
 {
     ref gref;
 
-    glyph_ref(glyph, &gref);
+    glyph_ref(pfont->memory, glyph, &gref);
     return zchar_charstring_data((gs_font *)pfont, &gref, pgd);
 }
 
@@ -931,7 +925,7 @@ z1_subr_data(gs_font_type1 * pfont, int index, bool global,
     ref subr;
     int code;
 
-    code = array_get((global ? &pfdata->u.type1.GlobalSubrs :
+    code = array_get(pfont->memory, (global ? &pfdata->u.type1.GlobalSubrs :
 		      &pfdata->u.type1.Subrs),
 		     index, &subr);
     if (code < 0)
@@ -954,11 +948,11 @@ z1_seac_data(gs_font_type1 *pfont, int ccode, gs_glyph *pglyph,
     if (glyph == GS_NO_GLYPH)
 	return_error(e_rangecheck);
     if ((code = gs_c_glyph_name(glyph, gstr)) < 0 ||
-	(code = name_ref(gstr->data, gstr->size, &rglyph, 0)) < 0
+	(code = name_ref(pfont->memory, gstr->data, gstr->size, &rglyph, 0)) < 0
 	)
 	return code;
     if (pglyph)
-	*pglyph = name_index(&rglyph);
+	*pglyph = name_index(pfont->memory, &rglyph);
     if (pgd)
 	code = zchar_charstring_data((gs_font *)pfont, &rglyph, pgd);
     return code;
@@ -1008,18 +1002,19 @@ const gs_type1_data_procs_t z1_data_procs = {
  */
 int
 zchar1_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *pmat,
-		     gx_path *ppath)
+		     gx_path *ppath, double sbw[4])
 {
     gs_font_type1 *const pfont1 = (gs_font_type1 *)font;
     ref gref;
     gs_glyph_data_t gdata;
     int code;
 
-    glyph_ref(glyph, &gref);
+    glyph_ref(font->memory, glyph, &gref);
+    gdata.memory = font->memory;
     code = zchar_charstring_data(font, &gref, &gdata);
     if (code < 0)
 	return code;
-    return zcharstring_outline(pfont1, WMode, &gref, &gdata, pmat, ppath);
+    return zcharstring_outline(pfont1, WMode, &gref, &gdata, pmat, ppath, sbw);
 }
 /*
  * Get a glyph outline given a CharString.  The glyph_outline procedure
@@ -1028,26 +1023,25 @@ zchar1_glyph_outline(gs_font *font, int WMode, gs_glyph glyph, const gs_matrix *
 int
 zcharstring_outline(gs_font_type1 *pfont1, int WMode, const ref *pgref,
 		    const gs_glyph_data_t *pgd_orig,
-		    const gs_matrix *pmat, gx_path *ppath)
+		    const gs_matrix *pmat, gx_path *ppath, double sbw[4])
 {
     const gs_glyph_data_t *pgd = pgd_orig;
     int code;
     gs_type1exec_state cxs;
     gs_type1_state *const pcis = &cxs.cis;
     const gs_type1_data *pdata;
-    const ref *pfdict;
-    ref *pcdevproc;
     int value;
     gs_imager_state gis;
-    double sbw[4], wv[4];
+    double wv[4];
     gs_point mpt;
 
     pdata = &pfont1->data;
     if (pgd->bits.size <= max(pdata->lenIV, 0))
 	return_error(e_invalidfont);
-    pfdict = &pfont_data(pfont1)->dict;
-    if (dict_find_string(pfdict, "CDevProc", &pcdevproc) > 0)
+#if 0 /* Ignore CDevProc for now. */   
+    if (zchar_get_CDevProc((const gs_font_base *)pfont1, &pcdevproc))
 	return_error(e_rangecheck); /* can't call CDevProc from here */
+#endif
     switch (WMode) {
     default:
 	code = zchar_get_metrics2((gs_font_base *)pfont1, pgref, wv);
@@ -1103,6 +1097,7 @@ icont:
 	return_error(e_rangecheck); /* can't handle it */
     case type1_result_sbw:	/* [h]sbw, just continue */
 	type1_cis_get_metrics(pcis, cxs.sbw);
+	type1_cis_get_metrics(pcis, sbw);
 	pgd = 0;
 	goto icont;
     }
@@ -1120,7 +1115,6 @@ z1_glyph_info_generic(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
     ref gref;
     ref *pcdevproc;
     gs_font_base *const pbfont = (gs_font_base *)font;
-    const ref *pfdict = &pfont_data(pbfont)->dict;
     int width_members = members & (GLYPH_INFO_WIDTH0 << wmode);
     int outline_widths = members & GLYPH_INFO_OUTLINE_WIDTHS;
     bool modified_widths = false;
@@ -1132,7 +1126,7 @@ z1_glyph_info_generic(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 
     if (!width_members)
 	return (*proc)(font, glyph, pmat, members, info);
-    if (!outline_widths && dict_find_string(pfdict, "CDevProc", &pcdevproc) > 0) {
+    if (!outline_widths && zchar_get_CDevProc(pbfont, &pcdevproc)) {
 	done_members |= GLYPH_INFO_CDEVPROC;
 	if (members & GLYPH_INFO_CDEVPROC) {
 	    info->members = done_members;
@@ -1141,7 +1135,7 @@ z1_glyph_info_generic(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 	    /* Ignore CDevProc. Used to compure MissingWidth.*/
 	}
     }
-    glyph_ref(glyph, &gref);
+    glyph_ref(pbfont->memory, glyph, &gref);
     if (width_members == GLYPH_INFO_WIDTH1) {
 	double wv[4];
 	code = zchar_get_metrics2(pbfont, &gref, wv);
@@ -1176,7 +1170,7 @@ z1_glyph_info_generic(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
     }
 
     if (outline_widths) {
-	if (modified_widths || dict_find_string(pfdict, "CDevProc", &pcdevproc) > 0) {
+	if (modified_widths || zchar_get_CDevProc(pbfont, &pcdevproc)) {
 	    /* Discard the modified widths, but indicate they exist. */
 	    width_members |= done_members;
 	    done_members = outline_widths;
@@ -1204,3 +1198,28 @@ z1_glyph_info(gs_font *font, gs_glyph glyph, const gs_matrix *pmat,
 				    &gs_type1_glyph_info, wmode);
 }
 
+/* Get a Type 1 or Type 9 character metrics and set the cache device. */
+int
+z1_set_cache(i_ctx_t *i_ctx_p, gs_font_base *pbfont, ref *cnref, 
+	    gs_glyph glyph, op_proc_t cont, op_proc_t *exec_cont)
+{   /* This function is similar to zchar42_set_cache. */
+    double sbw[4];
+    gs_glyph_info_t info;
+    int wmode = gs_rootfont(igs)->WMode;
+    int code;
+    gs_matrix id_matrix = { identity_matrix_body };
+
+    code = gs_default_glyph_info((gs_font *)pbfont, glyph, &id_matrix,
+		((GLYPH_INFO_WIDTH0 | GLYPH_INFO_VVECTOR0) << wmode) | GLYPH_INFO_BBOX,
+	        &info);
+    if (code < 0)
+	return code;
+    sbw[0] = info.v.x;
+    sbw[1] = info.v.y;
+    sbw[2] = info.width[wmode].x;
+    sbw[3] = info.width[wmode].y;
+    return zchar_set_cache(i_ctx_p, pbfont, cnref, NULL,
+			   sbw + 2, &info.bbox,
+			   cont, exec_cont,
+			   wmode ? sbw : NULL);
+}
