@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: zcontext.c,v 1.10 2002/06/16 03:43:50 lpd Exp $ */
+/* $Id: zcontext.c,v 1.11 2004/08/04 19:36:13 stefan Exp $ */
 /* Display PostScript context operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -112,7 +112,7 @@ CLEAR_MARKS_PROC(context_clear_marks)
     gs_context_t *const pctx = vptr;
 
     (*st_context_state.clear_marks)
-	(&pctx->state, sizeof(pctx->state), &st_context_state);
+        (cmem, &pctx->state, sizeof(pctx->state), &st_context_state);
 }
 private 
 ENUM_PTRS_WITH(context_enum_ptrs, gs_context_t *pctx)
@@ -278,7 +278,7 @@ context_reclaim(vm_spaces * pspaces, bool global)
 #ifdef DEBUG
     if (!psched->current->visible) {
 	lprintf("Current context is invisible!\n");
-	gs_abort();
+	gs_abort((gs_memory_t *)lmem);
     }
 #endif
 
@@ -370,7 +370,7 @@ ctx_initialize(i_ctx_t **pi_ctx_p)
     /* Create an initial context. */
     if (context_create(psched, &psched->current, &gs_imemory, *pi_ctx_p, true) < 0) {
 	lprintf("Can't create initial context!");
-	gs_abort();
+	gs_abort(imemory);
     }
     psched->current->scheduler = psched;
     /* Hook into the interpreter. */
@@ -601,7 +601,7 @@ do_fork(i_ctx_t *i_ctx_p, os_ptr op, const ref * pstdin, const ref * pstdout,
 	/* Share global VM, private local VM. */
 	ref *puserdict;
 	uint userdict_size;
-	gs_raw_memory_t *parent = iimemory_local->parent;
+	gs_memory_t *parent = iimemory_local->non_gc_memory;
 	gs_ref_memory_t *lmem;
 	gs_ref_memory_t *lmem_stable;
 
@@ -695,7 +695,7 @@ do_fork(i_ctx_t *i_ctx_p, os_ptr op, const ref * pstdin, const ref * pstdout,
 	    for (i = 0; i < copy; ++i) {
 		ref *pdref = ref_stack_index(dstack, i);
 
-		if (obj_eq(pdref, &old_userdict))
+		if (obj_eq(imemory, pdref, &old_userdict))
 		    *pdref = new_userdict;
 	    }
 	}
@@ -1165,7 +1165,7 @@ context_create(gs_scheduler_t * psched, gs_context_t ** ppctx,
 	    return code;
 	}
     }
-    ctx_index = gs_next_ids(1);
+    ctx_index = gs_next_ids(mem, 1);
     pctx->scheduler = psched;
     pctx->status = cs_active;
     pctx->index = ctx_index;
