@@ -14,7 +14,7 @@
   San Rafael, CA  94903, U.S.A., +1(415)492-9861.
 */
 
-/* $Id: zfileio.c,v 1.15 2002/06/16 03:43:51 lpd Exp $ */
+/* $Id: zfileio.c,v 1.18 2005/08/30 23:19:01 ray Exp $ */
 /* File I/O operators */
 #include "memory_.h"
 #include "ghost.h"
@@ -74,15 +74,27 @@ zread(i_ctx_t *i_ctx_p)
     int ch;
 
     check_read_file(s, op);
+    /* We 'push' first in case of ostack block overflow and the */
+    /* usual case is we will need to push anyway. If we get EOF */
+    /* we will need to 'pop' and decrement the 'op' pointer.    */
+    /* This is required since the 'push' macro might return with*/
+    /* stackoverflow which will result in another stack block   */
+    /* added on, then the operator being retried. We can't read */
+    /* (sgetc) prior to having a place on the ostack to return  */
+    /* the character.						*/
+    push(1);
     ch = sgetc(s);
     if (ch >= 0) {
-	push(1);
 	make_int(op - 1, ch);
 	make_bool(op, 1);
-    } else if (ch == EOFC)
+    } else {
+	pop(1);		/* Adjust ostack back from preparatory 'pop' */
+	op--;
+	if (ch == EOFC) 
 	make_bool(op, 0);
     else
 	return handle_read_status(i_ctx_p, ch, op, NULL, zread);
+    }
     return 0;
 }
 
@@ -685,17 +697,17 @@ zpeekstring(i_ctx_t *i_ctx_p)
     while ((rlen = sbufavailable(s)) < len) {
 	int status = s->end_status;
 
-	/*
-	 * The following is a HACK.  It should reallocate the buffer to hold
-	 * at least len bytes.  However, this raises messy problems about
-	 * which allocator to use and how it should interact with restore.
-	 */
-	if (len >= s->bsize)
-	    return_error(e_rangecheck);
 	switch (status) {
 	case EOFC:
 	    break;
 	case 0:
+	    /*
+	     * The following is a HACK.  It should reallocate the buffer to hold
+	     * at least len bytes.  However, this raises messy problems about
+	     * which allocator to use and how it should interact with restore.
+	     */
+	    if (len >= s->bsize)
+		return_error(e_rangecheck);
 	    s_process_read_buf(s);
 	    continue;
 	default:
@@ -750,7 +762,7 @@ zwritecvp_at(i_ctx_t *i_ctx_p, os_ptr op, uint start, bool first)
     code = obj_cvp(op - 1, str, sizeof(str), &len, (int)op->value.intval,
 		   start, imemory);
     if (code == e_rangecheck) {
-	code = obj_string_data(op - 1, &data, &len);
+        code = obj_string_data(imemory, op - 1, &data, &len);
 	if (len < start)
 	    return_error(e_rangecheck);
 	data += start;
